@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -87,8 +87,11 @@
 #include <dbm_api.h>                    /* CheckDBRepairFlagFile() */
 #include <sys/types.h>                  /* checking umask on writing setxid log */
 #include <sys/stat.h>                   /* checking umask on writing setxid log */
+#include <simulate_mode.h>              /* ManifestChangedFiles(), DiffChangedFiles() */
 
+#include <syntax.h>                     /* IsBuiltInPromiseType() */
 #include <mod_common.h>
+#include <mod_custom.h>                 /* EvaluateCustomPromise(), Intialize/FinalizeCustomPromises() */
 
 #ifdef HAVE_AVAHI_CLIENT_CLIENT_H
 #ifdef HAVE_AVAHI_COMMON_ADDRESS_H
@@ -157,7 +160,7 @@ static int NoteBundleCompliance(const Bundle *bundle, int save_pr_kept, int save
 static void AllClassesReport(const EvalContext *ctx);
 static bool HasAvahiSupport(void);
 static int AutomaticBootstrap(GenericAgentConfig *config);
-static void BannerStatus(PromiseResult status, char *type, char *name);
+static void BannerStatus(PromiseResult status, const char *type, char *name);
 static PromiseResult DefaultVarPromise(EvalContext *ctx, const Promise *pp);
 static void WaitForBackgroundProcesses();
 
@@ -197,11 +200,13 @@ static const struct option OPTIONS[] =
     {"no-extensions", no_argument, 0, 'E'},
     {"timestamp", no_argument, 0, 'l'},
     /* Only long option for the rest */
+    {"ignore-preferred-augments", no_argument, 0, 0},
     {"log-modules", required_argument, 0, 0},
     {"show-evaluated-classes", optional_argument, 0, 0 },
     {"show-evaluated-vars", optional_argument, 0, 0 },
     {"skip-bootstrap-policy-run", no_argument, 0, 0 },
     {"skip-db-check", optional_argument, 0, 0 },
+    {"simulate", required_argument, 0, 0},
     {NULL, 0, 0, '\0'}
 };
 
@@ -227,14 +232,25 @@ static const char *const HINTS[] =
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Disable extension loading (used while upgrading)",
     "Log timestamps on each line of log output",
+    "Ignore def_preferred.json file in favor of def.json",
     "Enable even more detailed debug logging for specific areas of the implementation. Use together with '-d'. Use --log-modules=help for a list of available modules",
     "Show *final* evaluated classes, including those defined in common bundles in policy. Optionally can take a regular expression.",
     "Show *final* evaluated variables, including those defined without dependency to user-defined classes in policy. Optionally can take a regular expression.",
     "Do not run policy as the last step of the bootstrap process",
     "Do not run database integrity checks and repairs at startup",
+    "Run in simulate mode, either 'manifest', 'manifest-full' or 'diff'",
     NULL
 };
 
+/**
+ @brief
+ Wrapper around DefaultVarPromise to silence cast-function-type compiler warning in ScheduleAgentOperations
+ */
+static PromiseResult DefaultVarPromiseWrapper(EvalContext *ctx, const Promise *pp, void *param) {
+    UNUSED(param);
+    return DefaultVarPromise(ctx, pp);
+}
+
 /*******************************************************************/
 
 int main(int argc, char *argv[])
@@ -280,6 +296,7 @@ int main(int argc, char *argv[])
     ConnCache_Init();
 
     BeginAudit();
+
     KeepPromises(ctx, policy, config);
 
     if (EvalAborted(ctx))
@@ -299,6 +316,10 @@ int main(int argc, char *argv[])
     /* Update packages cache. */
     UpdatePackagesCache(ctx, false);
 
+    /* Finalize custom promises before waiting for background processes because
+     * they can be background processes and need special handling. */
+    FinalizeCustomPromises();
+
     /* Wait for background processes before generating reports because
      * GenerateReports() does nothing if it detects multiple cf-agent processes
      * running. */
@@ -334,12 +355,54 @@ int main(int argc, char *argv[])
     Nova_NoteAgentExecutionPerformance(config->input_file, start);
 
     GenericAgentFinalize(ctx, config);
+
     StringSetDestroy(SINGLE_COPY_CACHE);
 
+    StringSet *audited_files = NULL;
+    if ((EVAL_MODE == EVAL_MODE_SIMULATE_MANIFEST) ||
+        (EVAL_MODE == EVAL_MODE_SIMULATE_MANIFEST_FULL))
+    {
+        bool success = ManifestChangedFiles(&audited_files);
+        if (!success)
+        {
+            Log(LOG_LEVEL_ERR, "Failed to manifest changed files");
+        }
+        if (EVAL_MODE == EVAL_MODE_SIMULATE_MANIFEST_FULL)
+        {
+            /* Skips the files already manifested above. */
+            success = ManifestAllFiles(&audited_files);
+            if (!success)
+            {
+                Log(LOG_LEVEL_ERR, "Failed to manifest unmodified files");
+            }
+        }
+        success = ManifestPkgOperations();
+        if (!success)
+        {
+            Log(LOG_LEVEL_ERR, "Failed to manifest present and absent packages");
+        }
+    }
+    else if (EVAL_MODE == EVAL_MODE_SIMULATE_DIFF)
+    {
+        bool success = DiffChangedFiles(&audited_files);
+        if (!success)
+        {
+            Log(LOG_LEVEL_ERR, "Failed to show differences for changed files");
+        }
+        success = DiffPkgOperations();
+        if (!success)
+        {
+            Log(LOG_LEVEL_ERR, "Failed to show differences in installed packages");
+        }
+    }
+    StringSetDestroy(audited_files);
+
 #ifdef HAVE_LIBXML2
         xmlCleanupParser();
 #endif
 
+    CallCleanupFunctions();
+
     return ret;
 }
 
@@ -519,7 +582,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'n':
-            DONTDO = true;
+            EVAL_MODE = EVAL_MODE_DRY_RUN;
             config->ignore_locks = true;
             break;
 
@@ -604,7 +667,11 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 0:
         {
             const char *const option_name = OPTIONS[longopt_idx].name;
-            if (StringEqual(option_name, "log-modules"))
+            if (StringEqual(option_name, "ignore-preferred-augments"))
+            {
+                config->ignore_preferred_augments = true;
+            }
+            else if (StringEqual(option_name, "log-modules"))
             {
                 bool ret = LogEnableModulesFromString(optarg);
                 if (!ret)
@@ -654,6 +721,34 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                     DoCleanupAndExit(EXIT_FAILURE);
                 }
             }
+            else if (StringEqual(option_name, "simulate"))
+            {
+                if (optarg == NULL)
+                {
+                    Log(LOG_LEVEL_ERR,
+                        "Missing argument for --simulate, 'manifest', 'manifest-full', or 'diff' required");
+                    DoCleanupAndExit(EXIT_FAILURE);
+                }
+                else if (StringEqual_IgnoreCase(optarg, "manifest"))
+                {
+                    EVAL_MODE = EVAL_MODE_SIMULATE_MANIFEST;
+                }
+                else if (StringEqual_IgnoreCase(optarg, "manifest-full"))
+                {
+                    EVAL_MODE = EVAL_MODE_SIMULATE_MANIFEST_FULL;
+                }
+                else if (StringEqual_IgnoreCase(optarg, "diff"))
+                {
+                    EVAL_MODE = EVAL_MODE_SIMULATE_DIFF;
+                }
+                else
+                {
+                    Log(LOG_LEVEL_ERR,
+                        "Invalid argument for --simulate, 'manifest' or 'diff' required, not '%s'",
+                        optarg);
+                    DoCleanupAndExit(EXIT_FAILURE);
+                }
+            }
             break;
         }
         default:
@@ -826,6 +921,8 @@ static void ThisAgentInit(void)
     {
         fclose(fp);
     }
+
+    InitializeCustomPromises();
 }
 
 /*******************************************************************/
@@ -1046,8 +1143,8 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
 
             if (strcmp(cp->lval, CFA_CONTROLBODY[AGENT_CONTROL_DRYRUN].lval) == 0)
             {
-                DONTDO = BooleanFromString(value);
-                Log(LOG_LEVEL_VERBOSE, "Setting dryrun to %c", DONTDO);
+                EVAL_MODE = BooleanFromString(value) ? EVAL_MODE_DRY_RUN : EVAL_MODE_NORMAL;
+                Log(LOG_LEVEL_VERBOSE, "Setting dryrun to %d", DONTDO);
                 continue;
             }
 
@@ -1344,16 +1441,15 @@ static void KeepPromiseBundles(EvalContext *ctx, const Policy *policy, GenericAg
         const char *name = NULL;
         const Rlist *args = NULL;
 
-        switch (rp->val.type)
+        if (rp->val.type == RVAL_TYPE_FNCALL)
         {
-        case RVAL_TYPE_FNCALL:
             name = RlistFnCallValue(rp)->name;
             args = RlistFnCallValue(rp)->args;
-            break;
-        default:
+        }
+        else
+        {
             name = RlistScalarValue(rp);
             args = NULL;
-            break;
         }
 
         EvalContextSetBundleArgs(ctx, args);
@@ -1421,6 +1517,8 @@ static void AllClassesReport(const EvalContext *ctx)
 PromiseResult ScheduleAgentOperations(EvalContext *ctx, const Bundle *bp)
 // NB - this function can be called recursively through "methods"
 {
+    assert(bp != NULL);
+
     int save_pr_kept = PR_KEPT;
     int save_pr_repaired = PR_REPAIRED;
     int save_pr_notkept = PR_NOTKEPT;
@@ -1435,9 +1533,10 @@ PromiseResult ScheduleAgentOperations(EvalContext *ctx, const Bundle *bp)
 
     for (int pass = 1; pass < CF_DONEPASSES; pass++)
     {
+        // Evaluate built-in (non-custom) promise types, according to type sequence (normal order):
         for (TypeSequence type = 0; AGENT_TYPESEQUENCE[type] != NULL; type++)
         {
-            const PromiseType *sp = BundleGetPromiseType((Bundle *)bp, AGENT_TYPESEQUENCE[type]);
+            const BundleSection *sp = BundleGetSection((Bundle *)bp, AGENT_TYPESEQUENCE[type]);
 
             if (!sp || SeqLength(sp->promises) == 0)
             {
@@ -1447,7 +1546,7 @@ PromiseResult ScheduleAgentOperations(EvalContext *ctx, const Bundle *bp)
             NewTypeContext(type);
 
             SpecialTypeBanner(type, pass);
-            EvalContextStackPushPromiseTypeFrame(ctx, sp);
+            EvalContextStackPushBundleSectionFrame(ctx, sp);
 
             for (size_t ppi = 0; ppi < SeqLength(sp->promises); ppi++)
             {
@@ -1473,9 +1572,37 @@ PromiseResult ScheduleAgentOperations(EvalContext *ctx, const Bundle *bp)
             if (type == TYPE_SEQUENCE_CONTEXTS)
             {
                 BundleResolve(ctx, bp);
-                BundleResolvePromiseType(ctx, bp, "defaults", (PromiseActuator*)DefaultVarPromise);
+                BundleResolvePromiseType(ctx, bp, "defaults", DefaultVarPromiseWrapper);
             }
         }
+
+        // Custom promises are evaluated at the end of an evaluation pass:
+        const size_t sections = SeqLength(bp->custom_sections);
+        for (size_t i = 0; i < sections; ++i)
+        {
+            BundleSection *section = SeqAt(bp->custom_sections, i);
+
+            EvalContextStackPushBundleSectionFrame(ctx, section);
+
+            const size_t promises = SeqLength(section->promises);
+            for (size_t ppi = 0; ppi < promises; ppi++)
+            {
+                Promise *pp = SeqAt(section->promises, ppi);
+
+                EvalContextSetPass(ctx, pass);
+
+                PromiseResult promise_result = ExpandPromise(ctx, pp, KeepAgentPromise, NULL);
+                result = PromiseResultUpdate(result, promise_result);
+
+                if (EvalAborted(ctx) || BundleAbort(ctx))
+                {
+                    EvalContextStackPopFrame(ctx);
+                    NoteBundleCompliance(bp, save_pr_kept, save_pr_repaired, save_pr_notkept, start);
+                    return result;
+                }
+            }
+            EvalContextStackPopFrame(ctx);
+        }
     }
 
     NoteBundleCompliance(bp, save_pr_kept, save_pr_repaired, save_pr_notkept, start);
@@ -1681,11 +1808,13 @@ static void LogVariableValue(const EvalContext *ctx, const Promise *pp)
 static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_UNUSED void *param)
 {
     assert(param == NULL);
+    assert(pp != NULL);
+
     struct timespec start = BeginMeasure();
     PromiseResult result = PROMISE_RESULT_NOOP;
 
-    if (strcmp("meta", pp->parent_promise_type->name) == 0 ||
-        strcmp("vars", pp->parent_promise_type->name) == 0)
+    if (strcmp("meta", PromiseGetPromiseType(pp)) == 0 ||
+        strcmp("vars", PromiseGetPromiseType(pp)) == 0)
     {
         Log(LOG_LEVEL_VERBOSE, "V:     Computing value of '%s'", pp->promiser);
 
@@ -1698,15 +1827,15 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             }
         }
     }
-    else if (strcmp("defaults", pp->parent_promise_type->name) == 0)
+    else if (strcmp("defaults", PromiseGetPromiseType(pp)) == 0)
     {
         result = DefaultVarPromise(ctx, pp);
     }
-    else if (strcmp("classes", pp->parent_promise_type->name) == 0)
+    else if (strcmp("classes", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyClassPromise(ctx, pp, NULL);
     }
-    else if (strcmp("processes", pp->parent_promise_type->name) == 0)
+    else if (strcmp("processes", PromiseGetPromiseType(pp)) == 0)
     {
         if (!LoadProcessTable())
         {
@@ -1719,7 +1848,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("storage", pp->parent_promise_type->name) == 0)
+    else if (strcmp("storage", PromiseGetPromiseType(pp)) == 0)
     {
         result = FindAndVerifyStoragePromises(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1727,7 +1856,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("packages", pp->parent_promise_type->name) == 0)
+    else if (strcmp("packages", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyPackagesPromise(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1735,7 +1864,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("users", pp->parent_promise_type->name) == 0)
+    else if (strcmp("users", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyUsersPromise(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1744,7 +1873,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
         }
     }
 
-    else if (strcmp("files", pp->parent_promise_type->name) == 0)
+    else if (strcmp("files", PromiseGetPromiseType(pp)) == 0)
     {
         result = ParallelFindAndVerifyFilesPromises(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1752,7 +1881,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("commands", pp->parent_promise_type->name) == 0)
+    else if (strcmp("commands", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyExecPromise(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1760,7 +1889,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("databases", pp->parent_promise_type->name) == 0)
+    else if (strcmp("databases", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyDatabasePromises(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1768,7 +1897,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("methods", pp->parent_promise_type->name) == 0)
+    else if (strcmp("methods", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyMethodsPromise(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1776,7 +1905,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("services", pp->parent_promise_type->name) == 0)
+    else if (strcmp("services", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyServicesPromise(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1784,7 +1913,7 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("guest_environments", pp->parent_promise_type->name) == 0)
+    else if (strcmp("guest_environments", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyEnvironmentsPromise(ctx, pp);
         if (result != PROMISE_RESULT_SKIPPED)
@@ -1792,22 +1921,26 @@ static PromiseResult KeepAgentPromise(EvalContext *ctx, const Promise *pp, ARG_U
             EndMeasurePromise(start, pp);
         }
     }
-    else if (strcmp("reports", pp->parent_promise_type->name) == 0)
+    else if (strcmp("reports", PromiseGetPromiseType(pp)) == 0)
     {
         result = VerifyReportPromise(ctx, pp);
     }
+    else if (!IsBuiltInPromiseType(PromiseGetPromiseType(pp)))
+    {
+        result = EvaluateCustomPromise(ctx, pp);
+    }
     else
     {
         result = PROMISE_RESULT_NOOP;
     }
 
-    BannerStatus(result, pp->parent_promise_type->name, pp->promiser);
+    BannerStatus(result, PromiseGetPromiseType(pp), pp->promiser);
     EvalContextLogPromiseIterationOutcome(ctx, pp, result);
     return result;
 }
 
 
-static void BannerStatus(PromiseResult status, char *type, char *name)
+static void BannerStatus(PromiseResult status, const char *type, char *name)
 {
     if ((strcmp(type, "vars") == 0) || (strcmp(type, "classes") == 0))
     {
