@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -34,7 +34,7 @@ typedef struct CompressedArray_
     struct CompressedArray_ *next;
 } CompressedArray;
 
-bool FixCompressedArrayValue(int i, char *value, CompressedArray **start);
+bool FixCompressedArrayValue(int i, const char *value, CompressedArray **start);
 void DeleteCompressedArray(CompressedArray *start);
 bool CompressedArrayElementExists(CompressedArray *start, int key);
 char *CompressedArrayValue(CompressedArray *start, int key);
