@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -194,10 +194,10 @@ static void AddMigratedFileToDirectoryList(CF_DB *changes_db, const char *file,
         return;
     }
 
-    if (SeqBinaryIndexOf(files, basefile, (SeqItemComparator)strcmp) == -1)
+    if (SeqBinaryIndexOf(files, basefile, StrCmpWrapper) == -1)
     {
         SeqAppend(files, xstrdup(basefile));
-        SeqSort(files, (SeqItemComparator)strcmp, NULL);
+        SeqSort(files, StrCmpWrapper, NULL);
         bool changes;
         if (!FileChangesSetDirectoryList(changes_db, dir, files, &changes))
         {
@@ -459,26 +459,32 @@ static bool FileChangesSetDirectoryList(CF_DB *db, const char *path, const Seq *
     return true;
 }
 
-/* Returns false if filename never seen before, and adds a checksum
-   to the database. Returns true if hashes do not match and also
-   updates database to the new value if update is true */
-bool FileChangesCheckAndUpdateHash_impl(const char *filename,
-                                        unsigned char digest[EVP_MAX_MD_SIZE + 1],
-                                        HashMethod type,
-                                        bool update,
-                                        const Promise *pp,
-                                        PromiseResult *result)
+/**
+ * @return %false if #filename never seen before, and adds a checksum to the
+ *         database; %true if hashes do not match and also updates database to
+ *         the new value if #update is true.
+ */
+bool FileChangesCheckAndUpdateHash(EvalContext *ctx,
+                                   const char *filename,
+                                   unsigned char digest[EVP_MAX_MD_SIZE + 1],
+                                   HashMethod type,
+                                   const Attributes *attr,
+                                   const Promise *pp,
+                                   PromiseResult *result)
 {
+    assert(attr != NULL);
+
     const int size = HashSizeFromId(type);
     unsigned char dbdigest[EVP_MAX_MD_SIZE + 1];
     CF_DB *dbp;
     bool found;
     bool different;
     bool ret = false;
+    bool update = attr->change.update;
 
     if (!OpenChangesDB(&dbp))
     {
-        Log(LOG_LEVEL_ERR, "Unable to open the hash database!");
+        RecordFailure(ctx, pp, attr, "Unable to open the hash database!");
         *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
         return false;
     }
@@ -489,7 +495,7 @@ bool FileChangesCheckAndUpdateHash_impl(const char *filename,
         different = (memcmp(digest, dbdigest, size) != 0);
         if (different)
         {
-            Log(LOG_LEVEL_NOTICE, "Hash '%s' for '%s' changed!", HashNameFromId(type), filename);
+            Log(LOG_LEVEL_INFO, "Hash '%s' for '%s' changed!", HashNameFromId(type), filename);
             if (pp->comment)
             {
                 Log(LOG_LEVEL_VERBOSE, "Preceding promise '%s'", pp->comment);
@@ -504,13 +510,20 @@ bool FileChangesCheckAndUpdateHash_impl(const char *filename,
 
     if (different)
     {
-        if ((!found || update) && !DONTDO)
+        /* TODO: Should we compare the stored hash with the digest of the file
+         *       in the changes chroot in case of ChrootChanges()?  */
+        if (!MakingInternalChanges(ctx, pp, attr, result, "record change of hash for file '%s'",
+                                   filename))
         {
-            const char *action = found ? "Updating" : "Storing";
+            ret = true;
+        }
+        else if (!found || update)
+        {
+            const char *action = found ? "Updated" : "Stored";
             char buffer[CF_HOSTKEY_STRING_SIZE];
-            Log(LOG_LEVEL_NOTICE, "%s %s hash for '%s' (%s)", action,
-                HashNameFromId(type), filename,
-                HashPrintSafe(buffer, sizeof(buffer), digest, type, true));
+            RecordChange(ctx, pp, attr, "%s %s hash for '%s' (%s)",
+                         action, HashNameFromId(type), filename,
+                         HashPrintSafe(buffer, sizeof(buffer), digest, type, true));
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
 
             WriteHash(dbp, type, filename, digest);
@@ -518,14 +531,15 @@ bool FileChangesCheckAndUpdateHash_impl(const char *filename,
         }
         else
         {
-            Log(LOG_LEVEL_NOTICE, "Hash for file '%s' changed", filename);
+            /* FIXME: FAIL if found?!?!?! */
+            RecordFailure(ctx, pp, attr, "Hash for file '%s' changed, but not updating the records", filename);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
             ret = true;
         }
     }
     else
     {
-        Log(LOG_LEVEL_VERBOSE, "File hash for %s is correct", filename);
+        RecordNoChange(ctx, pp, attr, "File hash for %s is correct", filename);
         *result = PromiseResultUpdate(*result, PROMISE_RESULT_NOOP);
         ret = false;
     }
@@ -534,46 +548,31 @@ bool FileChangesCheckAndUpdateHash_impl(const char *filename,
     return ret;
 }
 
-bool FileChangesCheckAndUpdateHash(EvalContext *ctx,
-                                   const char *filename,
-                                   unsigned char digest[EVP_MAX_MD_SIZE + 1],
-                                   HashMethod type,
-                                   const Attributes *attr,
-                                   const Promise *pp,
-                                   PromiseResult *result)
-{
-    assert(attr != NULL);
-    bool ret = FileChangesCheckAndUpdateHash_impl(filename, digest, type, attr->change.update, pp, result);
-    // TODO: Move cfPS even further up the call stack.
-    cfPS(ctx, LOG_LEVEL_DEBUG, *result, pp, attr, "Updating promise status for files changes promise");
-    return ret;
-}
-
-void FileChangesLogNewFile(const char *path, const Promise *pp)
+bool FileChangesLogNewFile(const char *path, const Promise *pp)
 {
     Log(LOG_LEVEL_NOTICE, "New file '%s' found", path);
-    FileChangesLogChange(path, FILE_STATE_NEW, "New file found", pp);
+    return FileChangesLogChange(path, FILE_STATE_NEW, "New file found", pp);
 }
 
 // db_file_set should already be sorted.
-void FileChangesCheckAndUpdateDirectory(const char *name, const Seq *file_set, const Seq *db_file_set,
+void FileChangesCheckAndUpdateDirectory(EvalContext *ctx, const Attributes *attr,
+                                        const char *name, const Seq *file_set, const Seq *db_file_set,
                                         bool update, const Promise *pp, PromiseResult *result)
 {
     CF_DB *db;
     if (!OpenChangesDB(&db))
     {
-        Log(LOG_LEVEL_ERR, "Could not open changes database");
+        RecordFailure(ctx, pp, attr, "Could not open changes database");
         *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
         return;
     }
 
-    Seq *disk_file_set = SeqSoftSort(file_set, (SeqItemComparator)strcmp, NULL);
+    Seq *disk_file_set = SeqSoftSort(file_set, StrCmpWrapper, NULL);
 
     // We'll traverse the union of disk_file_set and db_file_set in merged order.
 
     int num_files = SeqLength(disk_file_set);
     int num_db_files = SeqLength(db_file_set);
-    bool changed = false;
     for (int disk_pos = 0, db_pos = 0; disk_pos < num_files || db_pos < num_db_files;)
     {
         int compare_result;
@@ -609,7 +608,8 @@ void FileChangesCheckAndUpdateDirectory(const char *name, const Seq *file_set, c
             FileChangesLogNewFile(path, pp);
 #endif
 
-            changed = true;
+            /* just make sure the change is reflected in the promise result */
+            *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
             disk_pos++;
         }
         else if (compare_result > 0)
@@ -619,11 +619,23 @@ void FileChangesCheckAndUpdateDirectory(const char *name, const Seq *file_set, c
             xsnprintf(path, sizeof(path), "%s/%s", name, db_file);
 
             Log(LOG_LEVEL_NOTICE, "File '%s' no longer exists", path);
-            FileChangesLogChange(path, FILE_STATE_REMOVED, "File removed", pp);
+            if (MakingInternalChanges(ctx, pp, attr, result,
+                                      "record removal of '%s'", path))
+            {
+                if (FileChangesLogChange(path, FILE_STATE_REMOVED, "File removed", pp))
+                {
+                    RecordChange(ctx, pp, attr, "Removal of '%s' recorded", path);
+                    *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+                }
+                else
+                {
+                    RecordFailure(ctx, pp, attr, "Failed to record removal of '%s'", path);
+                    *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+                }
+            }
 
             RemoveAllFileTraces(db, path);
 
-            changed = true;
             db_pos++;
         }
         else
@@ -634,35 +646,44 @@ void FileChangesCheckAndUpdateDirectory(const char *name, const Seq *file_set, c
         }
     }
 
-    bool dir_list_changes = false;
-    PromiseResult newres = PROMISE_RESULT_NOOP;
-    if (changed)
+    if (MakingInternalChanges(ctx, pp, attr, result,
+                              "record directory listing for '%s'", name))
     {
-        if (update && FileChangesSetDirectoryList(db, name, disk_file_set, &dir_list_changes))
+        bool changes = false;
+        if (update && FileChangesSetDirectoryList(db, name, disk_file_set, &changes))
         {
-            if (dir_list_changes)
+            if (changes)
             {
-                newres = PROMISE_RESULT_CHANGE;
+                RecordChange(ctx, pp, attr, "Recorded directory listing for '%s'", name);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
             }
         }
         else
         {
-            newres = PROMISE_RESULT_FAIL;
+            RecordChange(ctx, pp, attr, "Failed to record directory listing for '%s'", name);
+            *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
         }
     }
-    *result = PromiseResultUpdate(*result, newres);
 
     SeqSoftDestroy(disk_file_set);
     CloseDB(db);
 }
 
-void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, bool update, const Promise *pp)
+void FileChangesCheckAndUpdateStats(EvalContext *ctx,
+                                    const char *file,
+                                    const struct stat *sb,
+                                    bool update,
+                                    const Attributes *attr,
+                                    const Promise *pp,
+                                    PromiseResult *result)
 {
     struct stat cmpsb;
     CF_DB *dbp;
 
     if (!OpenChangesDB(&dbp))
     {
+        RecordFailure(ctx, pp, attr, "Could not open changes database");
+        *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
         return;
     }
 
@@ -671,15 +692,23 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
 
     if (!ReadDB(dbp, key, &cmpsb, sizeof(struct stat)))
     {
-        if (!DONTDO)
+        if (MakingInternalChanges(ctx, pp, attr, result,
+                                  "write stat information for '%s' to database", file))
         {
             if (!WriteDB(dbp, key, sb, sizeof(struct stat)))
             {
-                Log(LOG_LEVEL_ERR, "Could not write stat information to database");
+                RecordFailure(ctx, pp, attr, "Could not write stat information for '%s' to database", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+            else
+            {
+                RecordChange(ctx, pp, attr, "Wrote stat information for '%s' to database", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
             }
-            CloseDB(dbp);
-            return;
         }
+
+        CloseDB(dbp);
+        return;
     }
 
     if (cmpsb.st_mode == sb->st_mode
@@ -689,6 +718,7 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
         && cmpsb.st_ino == sb->st_ino
         && cmpsb.st_mtime == sb->st_mtime)
     {
+        RecordNoChange(ctx, pp, attr, "No stat information change for '%s'", file);
         CloseDB(dbp);
         return;
     }
@@ -702,7 +732,19 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
         snprintf(msg_temp, sizeof(msg_temp), "Permission: %04jo -> %04jo",
                  (uintmax_t)cmpsb.st_mode, (uintmax_t)sb->st_mode);
 
-        FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp);
+        if (MakingInternalChanges(ctx, pp, attr, result, "record permissions changes in '%s'", file))
+        {
+            if (FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp))
+            {
+                RecordChange(ctx, pp, attr, "Recorded permissions changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+            }
+            else
+            {
+                RecordFailure(ctx, pp, attr, "Failed to record permissions changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+        }
     }
 
     if (cmpsb.st_uid != sb->st_uid)
@@ -714,7 +756,20 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
         snprintf(msg_temp, sizeof(msg_temp), "Owner: %ju -> %ju",
                  (uintmax_t) cmpsb.st_uid, (uintmax_t) sb->st_uid);
 
-        FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp);
+        if (MakingInternalChanges(ctx, pp, attr, result,
+                                  "record ownership changes in '%s'", file))
+        {
+            if (FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp))
+            {
+                RecordChange(ctx, pp, attr, "Recorded ownership changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+            }
+            else
+            {
+                RecordFailure(ctx, pp, attr, "Failed to record ownership changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+        }
     }
 
     if (cmpsb.st_gid != sb->st_gid)
@@ -726,7 +781,20 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
         snprintf(msg_temp, sizeof(msg_temp), "Group: %ju -> %ju",
                  (uintmax_t)cmpsb.st_gid, (uintmax_t)sb->st_gid);
 
-        FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp);
+        if (MakingInternalChanges(ctx, pp, attr, result,
+                                  "record group changes in '%s'", file))
+        {
+            if (FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp))
+            {
+                RecordChange(ctx, pp, attr, "Recorded group changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+            }
+            else
+            {
+                RecordFailure(ctx, pp, attr, "Failed to record group changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+        }
     }
 
     if (cmpsb.st_dev != sb->st_dev)
@@ -738,7 +806,19 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
         snprintf(msg_temp, sizeof(msg_temp), "Device: %ju -> %ju",
                  (uintmax_t)cmpsb.st_dev, (uintmax_t)sb->st_dev);
 
-        FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp);
+        if (MakingInternalChanges(ctx, pp, attr, result, "record device changes in '%s'", file))
+        {
+            if (FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp))
+            {
+                RecordChange(ctx, pp, attr, "Recorded device changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+            }
+            else
+            {
+                RecordFailure(ctx, pp, attr, "Failed to record device changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+        }
     }
 
     if (cmpsb.st_ino != sb->st_ino)
@@ -749,20 +829,38 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
 
     if (cmpsb.st_mtime != sb->st_mtime)
     {
-        char from[CF_MAXVARSIZE];
-        char to[CF_MAXVARSIZE];
+        char from[25]; // ctime() string is 26 bytes (incl NUL)
+        char to[25];   // we ignore the newline at the end
+        // Example: "Thu Nov 24 18:22:48 1986\n"
+
+        // TODO: Should be possible using memcpy
+        //       but I ran into some weird issues when trying
+        //       to assert the contents of ctime()
+        StringCopy(ctime(&(cmpsb.st_mtime)), from, 25);
+        StringCopy(ctime(&(sb->st_mtime)), to, 25);
+
+        assert(strlen(from) == 24);
+        assert(strlen(to) == 24);
 
-        strcpy(from, ctime(&(cmpsb.st_mtime)));
-        strcpy(to, ctime(&(sb->st_mtime)));
-        Chop(from, CF_MAXVARSIZE);
-        Chop(to, CF_MAXVARSIZE);
         Log(LOG_LEVEL_NOTICE, "Last modified time for '%s' changed '%s' -> '%s'", file, from, to);
 
         char msg_temp[CF_MAXVARSIZE];
         snprintf(msg_temp, sizeof(msg_temp), "Modified time: %s -> %s",
                  from, to);
 
-        FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp);
+        if (MakingInternalChanges(ctx, pp, attr, result, "record mtime changes in '%s'", file))
+        {
+            if (FileChangesLogChange(file, FILE_STATE_STATS_CHANGED, msg_temp, pp))
+            {
+                RecordChange(ctx, pp, attr, "Recorded mtime changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+            }
+            else
+            {
+                RecordFailure(ctx, pp, attr, "Failed to record mtime changes in '%s'", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+        }
     }
 
     if (pp->comment)
@@ -770,11 +868,21 @@ void FileChangesCheckAndUpdateStats(const char *file, const struct stat *sb, boo
         Log(LOG_LEVEL_VERBOSE, "Preceding promise '%s'", pp->comment);
     }
 
-    if (update && !DONTDO)
+    if (update)
     {
-        if (!DeleteDB(dbp, key) || !WriteDB(dbp, key, sb, sizeof(struct stat)))
+        if (MakingInternalChanges(ctx, pp, attr, result,
+                                  "write stat information for '%s' to database", file))
         {
-            Log(LOG_LEVEL_ERR, "Could not write stat information to database");
+            if (!DeleteDB(dbp, key) || !WriteDB(dbp, key, sb, sizeof(struct stat)))
+            {
+                RecordFailure(ctx, pp, attr, "Failed to write stat information for '%s' to database", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
+            }
+            else
+            {
+                RecordChange(ctx, pp, attr, "Wrote stat information changes for '%s' to database", file);
+                *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
+            }
         }
     }
 
@@ -802,7 +910,7 @@ static char FileStateToChar(FileState status)
     }
 }
 
-void FileChangesLogChange(const char *file, FileState status, char *msg, const Promise *pp)
+bool FileChangesLogChange(const char *file, FileState status, char *msg, const Promise *pp)
 {
     char fname[CF_BUFSIZE];
     time_t now = time(NULL);
@@ -827,11 +935,12 @@ void FileChangesLogChange(const char *file, FileState status, char *msg, const P
     if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Could not write to the hash change log. (fopen: %s)", GetErrorStr());
-        return;
+        return false;
     }
 
     const char *handle = PromiseID(pp);
 
     fprintf(fp, "%lld,%s,%s,%c,%s\n", (long long) now, handle, file, FileStateToChar(status), msg);
     fclose(fp);
+    return true;
 }
