@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -44,9 +44,9 @@
 typedef struct
 {
     char *filename;
+    char *changes_filename;
     Item *file_start;
     int num_edits;
-    int num_rewrites;
 #ifdef HAVE_LIBXML2
     xmlDocPtr xmldoc;
 #endif
