@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -37,6 +37,8 @@
 #include <rlist.h>
 #include <policy.h>
 #include <eval_context.h>
+#include <changes_chroot.h>     /* RecordPkgOperationInChroot() */
+#include <simulate_mode.h>      /* CHROOT_PKG_OPERATION_* */
 
 #define INVENTORY_LIST_BUFFER_SIZE 100 * 80 /* 100 entries with 80 characters
                                              * per line */
@@ -92,18 +94,18 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
         if (access(interpreter_path, X_OK) != 0)
         {
             Log(LOG_LEVEL_ERR,
-                "can not find package wrapper interpreter in provided location '%s' "
-                "or access to file is restricted: %s",
-                wrapper->path, strerror(errno));
+                "Cannot find package module interpreter at location '%s'"
+                " or access to the file is restricted: %s",
+                wrapper->path, GetErrorStr());
             DeletePackageModuleWrapper(wrapper);
             return NULL;
         }
         if (access(script_path, R_OK) != 0)
         {
             Log(LOG_LEVEL_ERR,
-                "can not find the package wrapper script in provided location '%s' "
-                "or access to file is restricted: %s",
-                wrapper->script_path, strerror(errno));
+                "Cannot find package module script at location '%s'"
+                " or access to the file is restricted: %s",
+                wrapper->script_path, GetErrorStr());
             DeletePackageModuleWrapper(wrapper);
             return NULL;
         }
@@ -112,9 +114,8 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
     {
         /* no script path specified --> the given path has to be executable */
         Log(LOG_LEVEL_ERR,
-            "can not find package wrapper in provided location '%s' "
-            "or access to file is restricted: %s",
-            wrapper->path, strerror(errno));
+            "Cannot find package module at location '%s' or access to file is restricted: %s",
+            wrapper->path, GetErrorStr());
         DeletePackageModuleWrapper(wrapper);
         return NULL;
     }
@@ -124,7 +125,7 @@ PackageModuleWrapper *NewPackageModuleWrapper(PackageModuleBody *package_module)
     if (wrapper->supported_api_version != 1)
     {
         Log(LOG_LEVEL_ERR,
-            "unsupported package module wrapper API version: %d",
+            "Unsupported package module wrapper API version: %d",
             wrapper->supported_api_version);
         DeletePackageModuleWrapper(wrapper);
         return NULL;
@@ -215,10 +216,10 @@ void UpdatePackagesCache(EvalContext *ctx, bool force_update)
 PackagePromiseGlobalLock AcquireGlobalPackagePromiseLock(EvalContext *ctx)
 {
     Bundle bundle = {.name = "package_global"};
-    PromiseType promise_type = {.name = "package_global",
-                               .parent_bundle = &bundle};
+    BundleSection section = {.promise_type = "package_global",
+                             .parent_bundle = &bundle};
     Promise pp = {.promiser = "package_global",
-                  .parent_promise_type = &promise_type};
+                  .parent_section = &section};
 
     CfLock package_promise_global_lock;
 
@@ -233,10 +234,10 @@ PackagePromiseGlobalLock AcquireGlobalPackagePromiseLock(EvalContext *ctx)
 void YieldGlobalPackagePromiseLock(PackagePromiseGlobalLock lock)
 {
     Bundle bundle = {.name = "package_global"};
-    PromiseType promise_type = {.name = "package_global",
-                               .parent_bundle = &bundle};
+    BundleSection section = {.promise_type = "package_global",
+                             .parent_bundle = &bundle};
     Promise pp = {.promiser = "package_global",
-                  .parent_promise_type = &promise_type};
+                  .parent_section = &section};
 
     YieldCurrentLockAndRemoveFromCache(lock.lock_ctx, lock.g_lock,
                                        GLOBAL_PACKAGE_PROMISE_LOCK_NAME, &pp);
@@ -741,9 +742,9 @@ int UpdatePackagesDB(Rlist *data, const char *pm_name, UpdateType type)
         char *inventory_list = BufferClose(inventory_data);
 
         /* We can have empty list of installed software or available updates. */
-        if (inventory_list == NULL)
+        if (!inventory_list)
         {
-            WriteDB(db_cached, inventory_key, "", 0);
+            WriteDB(db_cached, inventory_key, "\n", 1);
         }
         else
         {
@@ -956,10 +957,10 @@ static PromiseResult InstallPackageGeneric(Rlist *options,
     return res;
 }
 
-PromiseResult InstallPackage(Rlist *options,
-        PackageType type, const char *package_to_install,
-        const char *version, const char *architecture,
-        const PackageModuleWrapper *wrapper)
+static PromiseResult InstallPackage(Rlist *options,
+                                    PackageType type, const char *package_to_install,
+                                    const char *version, const char *architecture,
+                                    const PackageModuleWrapper *wrapper)
 {
     Log(LOG_LEVEL_DEBUG, "Installing package '%s'", package_to_install);
 
@@ -1000,31 +1001,40 @@ PromiseResult InstallPackage(Rlist *options,
     return res;
 }
 
-PromiseResult FileInstallPackage(const char *package_file_path,
-                                 const PackageInfo *info,
-                                 const NewPackages *policy_data,
-                                 const PackageModuleWrapper *wrapper,
-                                 int is_in_cache, enum cfopaction action)
+static PromiseResult FileInstallPackage(EvalContext *ctx,
+                                        const Promise *pp,
+                                        const Attributes *attr,
+                                        const char *package_file_path,
+                                        const PackageInfo *info,
+                                        const PackageModuleWrapper *wrapper,
+                                        int is_in_cache)
 {
+    assert(attr != NULL);
+
+    const NewPackages *policy_data = &(attr->new_packages);
+
     Log(LOG_LEVEL_DEBUG, "Installing file type package.");
 
     /* We have some packages matching file package promise in cache. */
     if (is_in_cache == 1)
     {
         Log(LOG_LEVEL_VERBOSE, "Package exists in cache. Skipping installation.");
+        if (ChrootChanges())
+        {
+            RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_PRESENT, package_file_path, NULL, NULL);
+        }
         return PROMISE_RESULT_NOOP;
     }
 
     PromiseResult res;
-
-    if (action == cfa_warn || DONTDO)
-    {
-         Log(LOG_LEVEL_VERBOSE, "Should install file type package: %s",
-             package_file_path);
-        res = PROMISE_RESULT_WARN;
-    }
-    else
+    if (MakingChanges(ctx, pp, attr, &res, "install file type package: %s", package_file_path))
     {
+        if (ChrootChanges())
+        {
+            /* TODO: simulate file package installation */
+            RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_INSTALL, package_file_path, NULL, NULL);
+            return PROMISE_RESULT_CHANGE;
+        }
         res = InstallPackage(policy_data->package_options,
                              PACKAGE_TYPE_FILE, package_file_path,
                              NULL, NULL, wrapper);
@@ -1039,8 +1049,8 @@ PromiseResult FileInstallPackage(const char *package_file_path,
 }
 
 
-Seq *GetVersionsFromUpdates(EvalContext *ctx, const PackageInfo *info,
-                            const PackageModuleWrapper *module_wrapper)
+static Seq *GetVersionsFromUpdates(EvalContext *ctx, const PackageInfo *info,
+                                   const PackageModuleWrapper *module_wrapper)
 {
     assert(info && info->name);
 
@@ -1077,7 +1087,7 @@ Seq *GetVersionsFromUpdates(EvalContext *ctx, const PackageInfo *info,
             ReadDB(db_updates, package_key, buff, val_size);
             Seq* updates = SeqStringFromString(buff, '\n');
 
-            for (int i = 0; i < SeqLength(updates); i++)
+            for (size_t i = 0; i < SeqLength(updates); i++)
             {
                 char *package_line = SeqAt(updates, i);
                 Log(LOG_LEVEL_DEBUG, "Got line in updates database: '%s",
@@ -1109,14 +1119,19 @@ Seq *GetVersionsFromUpdates(EvalContext *ctx, const PackageInfo *info,
     return updates_list;
 }
 
-PromiseResult RepoInstall(EvalContext *ctx,
-                          PackageInfo *package_info,
-                          const NewPackages *policy_data,
-                          const PackageModuleWrapper *wrapper,
-                          int is_in_cache,
-                          enum cfopaction action,
-                          bool *verified)
+static PromiseResult RepoInstall(EvalContext *ctx,
+                                 const Promise *pp,
+                                 const Attributes *attr,
+                                 const PackageInfo *package_info,
+                                 const PackageModuleWrapper *wrapper,
+                                 int is_in_cache,
+                                 bool *verified)
 {
+    assert(attr != NULL);
+    assert(package_info != NULL);
+
+    const NewPackages *policy_data = &(attr->new_packages);
+
     Log(LOG_LEVEL_DEBUG, "Installing repo type package: %d", is_in_cache);
     const char *const package_version = package_info->version;
     const char *const package_name = package_info->name;
@@ -1136,177 +1151,208 @@ PromiseResult RepoInstall(EvalContext *ctx,
             Log(LOG_LEVEL_DEBUG, "Clearing latest package version");
             version = NULL;
         }
-        if (action == cfa_warn || DONTDO)
+        PromiseResult result = PROMISE_RESULT_FAIL;
+        if (MakingChanges(ctx, pp, attr, &result, "install repo type package: %s", package_name))
         {
-            Log(LOG_LEVEL_VERBOSE, "Should install repo type package: %s",
-                package_name);
-            return PROMISE_RESULT_WARN;
+            if (ChrootChanges())
+            {
+                /* TODO: simulate package installation */
+                RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_INSTALL, package_name,
+                                           package_version, package_info->arch);
+                return PROMISE_RESULT_CHANGE;
+            }
+            *verified = false; /* Verification will be done in RepoInstallPackage(). */
+            result = InstallPackage(policy_data->package_options, PACKAGE_TYPE_REPO,
+                                    package_name, version, package_info->arch,
+                                    wrapper);
         }
-
-        *verified = false; /* Verification will be done in RepoInstallPackage(). */
-        return InstallPackage(policy_data->package_options, PACKAGE_TYPE_REPO,
-                              package_name, version, package_info->arch,
-                              wrapper);
+        return result;
     }
 
 
     /* We have some packages matching already installed at this point. */
 
 
-    /* We have 'latest' version in policy. */
-    if (StringEqual(package_version, "latest"))
+    if (!StringEqual(package_version, "latest"))
     {
-        /* This can return more than one latest version if we have packages
-         * with different architectures installed. */
-        Seq *latest_versions =
-                GetVersionsFromUpdates(ctx, package_info, wrapper);
-        if (!latest_versions)
+        /* No version or explicit version specified. */
+        if (ChrootChanges())
         {
-            Log(LOG_LEVEL_VERBOSE,
-                "Package '%s' is already in the latest version. "
-                "Skipping installation.", package_name);
-
-            return PROMISE_RESULT_NOOP;
+            RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_PRESENT, package_name,
+                                       package_version, package_info->arch);
         }
-
-        PromiseResult res = PROMISE_RESULT_NOOP;
-        Buffer *install_buffer = BufferNew();
-        Seq *packages_to_install = SeqNew(1, NULL);
-
-        /* Loop through possible updates. */
-        for (int i = 0; i < SeqLength(latest_versions); i++)
-        {
-            PackageInfo *update_package = SeqAt(latest_versions, i);
-
-            /* We can have multiple packages with different architectures
-             * in updates available but we are interested only in updating
-             * package with specific architecture. */
-            if (package_info->arch &&
-                !StringEqual(package_info->arch, update_package->arch))
-            {
-                Log(LOG_LEVEL_DEBUG,
-                    "Skipping update check of package '%s' as updates"
-                    "architecure doesn't match specified in policy: %s != %s.",
-                    package_name, package_info->arch,
-                    update_package->arch);
-                continue;
-            }
-
-            const char *const update_version = update_package->version;
-
-            Log(LOG_LEVEL_DEBUG,
-                "Checking for package '%s' version '%s' in available updates",
-                package_name, update_version);
-
-            /* Just in case some package managers will report highest possible
-             * version in updates list instead of removing entry if package is
-             * already in the latest version. */
-            const int update_in_cache = IsPackageInCache(
-                ctx, wrapper, package_name, update_version, update_package->arch);
-            if (update_in_cache == 1)
-            {
-                Log(LOG_LEVEL_VERBOSE,
-                    "Package version from updates matches one installed. "
-                    "Skipping package installation.");
-                res = PromiseResultUpdate(res, PROMISE_RESULT_NOOP);
-                continue;
-            }
-            else if (update_in_cache == -1)
-            {
-                Log(LOG_LEVEL_INFO,
-                    "Skipping package installation due to error with checking "
-                    "packages cache.");
-                res = PromiseResultUpdate(res, PROMISE_RESULT_FAIL);
-                continue;
-            }
-            else
-            {
-                if (action == cfa_warn || DONTDO)
-                {
-                    Log(LOG_LEVEL_VERBOSE, "Should install repo type package: %s",
-                        package_name);
-                    res = PromiseResultUpdate(res, PROMISE_RESULT_WARN);
-                    continue;
-                }
-
-                /* Append package data to buffer. At the end all packages
-                 * data that need to be updated will be sent to package module
-                 * at once. This is important if we have package in more than
-                 * one architecture. If we would update one after another we
-                 * may end up with the ones doesn't matching default
-                 * architecture being removed. */
-                BufferAppendF(install_buffer,
-                              "Name=%s\nVersion=%s\nArchitecture=%s\n",
-                              package_name,
-                              update_version,
-                              update_package->arch);
-
-                /* Here we are adding latest_versions elements to different
-                 * seq. Make sure to not free those and not free latest_versions
-                 * before we are done with packages_to_install.
-                 * This is needed for later verification if package was
-                 * installed correctly. */
-                SeqAppend(packages_to_install, update_package);
-            }
-        }
-
-        char *install_formatted_list = BufferClose(install_buffer);
-
-        if (install_formatted_list)
-        {
-            /* If we have some packages to install. */
-            if (strlen(install_formatted_list) > 0)
-            {
-                Log(LOG_LEVEL_DEBUG,
-                    "Formatted list of packages to be send to package module: "
-                    "[%s]", install_formatted_list);
-                res = InstallPackageGeneric(policy_data->package_options,
-                                      PACKAGE_TYPE_REPO,
-                                      install_formatted_list, wrapper);
-
-                for (int i = 0; i < SeqLength(packages_to_install); i++)
-                {
-                    PackageInfo *to_verify = SeqAt(packages_to_install, i);
-                    PromiseResult validate =
-                            ValidateChangedPackage(policy_data, wrapper,
-                                                   to_verify,
-                                                   NEW_PACKAGE_ACTION_PRESENT);
-                    Log(LOG_LEVEL_DEBUG,
-                        "Validating package %s:%s:%s installation result: %d",
-                         to_verify->name, to_verify->version,
-                         to_verify->arch, validate);
-                    res = PromiseResultUpdate(res, validate);
-                    *verified = true;
-                }
-            }
-            free(install_formatted_list);
-        }
-
-        SeqDestroy(packages_to_install);
-        SeqDestroy(latest_versions);
-        return res;
-    }
-    /* No version or explicit version specified. */
-    else
-    {
         Log(LOG_LEVEL_VERBOSE, "Package '%s' already installed", package_name);
 
         return PROMISE_RESULT_NOOP;
     }
-    /* Just to keep compiler happy; we shouldn't reach this point. */
-    return PROMISE_RESULT_FAIL;
+
+    /* We have 'latest' version in policy. */
+
+    /* This can return more than one latest version if we have packages
+     * with different architectures installed. */
+    Seq *latest_versions = GetVersionsFromUpdates(ctx, package_info, wrapper);
+    if (latest_versions == NULL)
+    {
+        if (ChrootChanges())
+        {
+            RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_PRESENT, package_name,
+                                       package_version, package_info->arch);
+        }
+        Log(LOG_LEVEL_VERBOSE,
+            "Package '%s' is already in the latest version. "
+            "Skipping installation.", package_name);
+
+        return PROMISE_RESULT_NOOP;
+    }
+
+    PromiseResult res = PROMISE_RESULT_NOOP;
+    Buffer *install_buffer = BufferNew();
+    Seq *packages_to_install = SeqNew(1, NULL);
+
+    /* Loop through possible updates. */
+    size_t length = SeqLength(latest_versions);
+    for (size_t i = 0; i < length; i++)
+    {
+        PackageInfo *update_package = SeqAt(latest_versions, i);
+
+        /* We can have multiple packages with different architectures
+         * in updates available but we are interested only in updating
+         * package with specific architecture. */
+        if (package_info->arch &&
+            !StringEqual(package_info->arch, update_package->arch))
+        {
+            Log(LOG_LEVEL_DEBUG,
+                "Skipping update check of package '%s' as updates"
+                "architecure doesn't match specified in policy: %s != %s.",
+                package_name, package_info->arch,
+                update_package->arch);
+            continue;
+        }
+
+        const char *const update_version = update_package->version;
+
+        Log(LOG_LEVEL_DEBUG,
+            "Checking for package '%s' version '%s' in available updates",
+            package_name, update_version);
+
+        /* Just in case some package managers will report highest possible
+         * version in updates list instead of removing entry if package is
+         * already in the latest version. */
+        const int update_in_cache = IsPackageInCache(
+            ctx, wrapper, package_name, update_version, update_package->arch);
+        if (update_in_cache == 1)
+        {
+            if (ChrootChanges())
+            {
+                RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_PRESENT, package_name,
+                                           update_version, update_package->arch);
+            }
+            Log(LOG_LEVEL_VERBOSE,
+                "Package version from updates matches one installed. "
+                "Skipping package installation.");
+            res = PromiseResultUpdate(res, PROMISE_RESULT_NOOP);
+            continue;
+        }
+        else if (update_in_cache == -1)
+        {
+            Log(LOG_LEVEL_INFO,
+                "Skipping package installation due to error with checking "
+                "packages cache.");
+            res = PromiseResultUpdate(res, PROMISE_RESULT_FAIL);
+            continue;
+        }
+        else
+        {
+            PromiseResult result = PROMISE_RESULT_FAIL;
+            if (MakingChanges(ctx, pp, attr, &result, "install repo type package: %s",
+                              package_name))
+            {
+                if (ChrootChanges())
+                {
+                    RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_INSTALL, package_name,
+                                               update_version, update_package->arch);
+                }
+                else
+                {
+                    /* Append package data to buffer. At the end all packages
+                     * data that need to be updated will be sent to package module
+                     * at once. This is important if we have package in more than
+                     * one architecture. If we would update one after another we
+                     * may end up with the ones doesn't matching default
+                     * architecture being removed. */
+                    BufferAppendF(install_buffer,
+                                  "Name=%s\nVersion=%s\nArchitecture=%s\n",
+                                  package_name,
+                                  update_version,
+                                  update_package->arch);
+
+                    /* Here we are adding latest_versions elements to different
+                     * seq. Make sure to not free those and not free latest_versions
+                     * before we are done with packages_to_install.
+                     * This is needed for later verification if package was
+                     * installed correctly. */
+                    SeqAppend(packages_to_install, update_package);
+                }
+            }
+            else
+            {
+                res = PromiseResultUpdate(res, result);
+                continue;
+            }
+        }
+    }
+
+    char *install_formatted_list = BufferClose(install_buffer);
+
+    if (install_formatted_list)
+    {
+        /* If we have some packages to install. */
+        if (strlen(install_formatted_list) > 0)
+        {
+            Log(LOG_LEVEL_DEBUG,
+                "Formatted list of packages to be send to package module: "
+                "[%s]", install_formatted_list);
+            res = InstallPackageGeneric(policy_data->package_options,
+                                        PACKAGE_TYPE_REPO,
+                                        install_formatted_list, wrapper);
+
+            for (size_t i = 0; i < SeqLength(packages_to_install); i++)
+            {
+                PackageInfo *to_verify = SeqAt(packages_to_install, i);
+                PromiseResult validate =
+                    ValidateChangedPackage(policy_data, wrapper,
+                                           to_verify,
+                                           NEW_PACKAGE_ACTION_PRESENT);
+                Log(LOG_LEVEL_DEBUG,
+                    "Validating package %s:%s:%s installation result: %d",
+                    to_verify->name, to_verify->version,
+                    to_verify->arch, validate);
+                res = PromiseResultUpdate(res, validate);
+                *verified = true;
+            }
+        }
+        free(install_formatted_list);
+    }
+
+    SeqDestroy(packages_to_install);
+    SeqDestroy(latest_versions);
+    return res;
 }
 
-PromiseResult RepoInstallPackage(EvalContext *ctx,
-                                 PackageInfo *package_info,
-                                 const NewPackages *policy_data,
-                                 const PackageModuleWrapper *wrapper,
-                                 int is_in_cache,
-                                 enum cfopaction action)
+static PromiseResult RepoInstallPackage(EvalContext *ctx,
+                                        const Promise *pp,
+                                        const Attributes *attr,
+                                        const PackageInfo *package_info,
+                                        const PackageModuleWrapper *wrapper,
+                                        int is_in_cache)
 {
+    assert(attr != NULL);
+
+    const NewPackages *policy_data = &(attr->new_packages);
+
     bool verified = false;
-    PromiseResult res = RepoInstall(ctx, package_info, policy_data, wrapper,
-                                    is_in_cache, action, &verified);
+    PromiseResult res = RepoInstall(ctx, pp, attr, package_info, wrapper,
+                                    is_in_cache, &verified);
 
     if (res == PROMISE_RESULT_CHANGE && !verified)
     {
@@ -1351,11 +1397,16 @@ static bool CheckPolicyAndPackageInfoMatch(const NewPackages *packages_policy,
 }
 
 PromiseResult HandlePresentPromiseAction(EvalContext *ctx,
-                                         const char *package_name,
-                                         const NewPackages *policy_data,
-                                         const PackageModuleWrapper *wrapper,
-                                         enum cfopaction action)
+                                         const Promise *pp,
+                                         const Attributes *attr,
+                                         const PackageModuleWrapper *wrapper)
 {
+    assert(pp != NULL);
+    assert(attr != NULL);
+
+    const char *package_name = pp->promiser;
+    const NewPackages *policy_data = &(attr->new_packages);
+
     Log(LOG_LEVEL_DEBUG, "Starting evaluating present action promise.");
 
     /* Figure out what kind of package we are having. */
@@ -1421,24 +1472,20 @@ PromiseResult HandlePresentPromiseAction(EvalContext *ctx,
 
         if (is_in_cache == -1)
         {
-            Log(LOG_LEVEL_INFO, "Some error occurred while looking for package "
-                    "'%s' in cache.", package_name);
+            Log(LOG_LEVEL_ERR, "Some error occurred while looking for package '%s' in cache.",
+                package_name);
             return PROMISE_RESULT_FAIL;
         }
 
         switch (package_info->type)
         {
             case PACKAGE_TYPE_FILE:
-                result = FileInstallPackage(package_name, package_info,
-                                            policy_data,
-                                            wrapper,
-                                            is_in_cache,
-                                            action);
+                result = FileInstallPackage(ctx, pp, attr, package_name, package_info,
+                                            wrapper, is_in_cache);
                 break;
             case PACKAGE_TYPE_REPO:
-                result = RepoInstallPackage(ctx, package_info, policy_data,
-                                            wrapper,
-                                            is_in_cache, action);
+                result = RepoInstallPackage(ctx, pp, attr, package_info,
+                                            wrapper, is_in_cache);
                 break;
             default:
                 /* We shouldn't end up here. If we are having unsupported
@@ -1462,11 +1509,16 @@ PromiseResult HandlePresentPromiseAction(EvalContext *ctx,
 
 
 PromiseResult HandleAbsentPromiseAction(EvalContext *ctx,
-                                        char *package_name,
-                                        const NewPackages *policy_data,
-                                        const PackageModuleWrapper *wrapper,
-                                        enum cfopaction action)
+                                        const Promise *pp,
+                                        const Attributes *attr,
+                                        const PackageModuleWrapper *wrapper)
 {
+    assert(pp != NULL);
+    assert(attr != NULL);
+
+    const char *package_name = pp->promiser;
+    const NewPackages *policy_data = &(attr->new_packages);
+
     /* Check if we are not having 'latest' version. */
     if (policy_data->package_version &&
             StringEqual(policy_data->package_version, "latest"))
@@ -1484,14 +1536,16 @@ PromiseResult HandleAbsentPromiseAction(EvalContext *ctx,
     {
         /* Remove package(s) */
         PromiseResult res;
-
-        if (action == cfa_warn || DONTDO)
-        {
-            Log(LOG_LEVEL_VERBOSE, "Need to remove package: %s", package_name);
-            res = PROMISE_RESULT_WARN;
-        }
-        else
+        if (MakingChanges(ctx, pp, attr, &res, "remove package '%s'", package_name))
         {
+            if (ChrootChanges())
+            {
+                /* TODO: simulate removal */
+                RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_REMOVE, package_name,
+                                           policy_data->package_version,
+                                           policy_data->package_architecture);
+                return PROMISE_RESULT_CHANGE;
+            }
             res = RemovePackage(package_name,
                     policy_data->package_options, policy_data->package_version,
                     policy_data->package_architecture, wrapper);
@@ -1499,11 +1553,17 @@ PromiseResult HandleAbsentPromiseAction(EvalContext *ctx,
             if (res == PROMISE_RESULT_CHANGE)
             {
                 /* Check if package was removed. */
-                return ValidateChangedPackage(policy_data, wrapper,
-                        &((PackageInfo){.name = package_name,
-                                        .version = policy_data->package_version,
-                                        .arch = policy_data->package_architecture}),
-                        NEW_PACKAGE_ACTION_ABSENT);
+
+                /* package_name is actually pp->promiser which is 'const char *'
+                 * so we need to type-cast it. It's safe because pkg_info is
+                 * passed as const *PackageInfo. */
+                const PackageInfo pkg_info = {
+                    .name = (char *) package_name,
+                    .version = policy_data->package_version,
+                    .arch = policy_data->package_architecture
+                };
+                return ValidateChangedPackage(policy_data, wrapper, &(pkg_info),
+                                              NEW_PACKAGE_ACTION_ABSENT);
             }
         }
 
@@ -1511,7 +1571,7 @@ PromiseResult HandleAbsentPromiseAction(EvalContext *ctx,
     }
     else if (is_in_cache == -1)
     {
-        Log(LOG_LEVEL_INFO, "Error occurred while checking package '%s' "
+        Log(LOG_LEVEL_ERR, "Error occurred while checking package '%s' "
             "existence in cache.", package_name);
         return PROMISE_RESULT_FAIL;
     }
@@ -1519,7 +1579,12 @@ PromiseResult HandleAbsentPromiseAction(EvalContext *ctx,
     {
         /* Package is not in cache which means it is already removed. */
         Log(LOG_LEVEL_DEBUG, "Package '%s' not installed. Skipping removing.",
-                package_name);
+            package_name);
+        if (ChrootChanges())
+        {
+            RecordPkgOperationInChroot(CHROOT_PKG_OPERATION_ABSENT, package_name, policy_data->package_version,
+                                       policy_data->package_architecture);
+        }
         return PROMISE_RESULT_NOOP;
     }
 }
@@ -1553,12 +1618,12 @@ bool UpdateSinglePackageModuleCache(EvalContext *ctx,
     }
 
     Bundle bundle = {.name = "package_cache"};
-    PromiseType promie_type = {.name = "package_cache",
-                               .parent_bundle = &bundle};
+    BundleSection section = {.promise_type = "package_cache",
+                             .parent_bundle = &bundle};
     Promise pp = {.promiser = "package_cache",
-                  .parent_promise_type = &promie_type};
+                  .parent_section = &section};
 
-    CfLock cache_updates_lock;
+    CfLock cache_updates_lock = {NULL, NULL, false};
     char cache_updates_lock_name[CF_BUFSIZE];
     int ifelapsed_time = -1;
 
