@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -48,7 +48,7 @@ bool VerifyCommandRetcode(EvalContext *ctx, int retcode, const Attributes *a, co
         // inform constraint is only for commands promises,
         // a->inform is actually false for other promise types, so
         // checking the promise type here is important:
-        if (StringEqual("commands", pp->parent_promise_type->name) && (!a->inform))
+        if (StringEqual("commands", PromiseGetPromiseType(pp)) && (!a->inform))
         {
             // for commands promises which don't make changes to the system,
             // you can use this to make the log messages verbose:
@@ -59,15 +59,16 @@ bool VerifyCommandRetcode(EvalContext *ctx, int retcode, const Attributes *a, co
         if (RlistKeyIn(a->classes.retcode_kept, retcodeStr))
         {
             cfPS(ctx, info_or_verbose, PROMISE_RESULT_NOOP, pp, a,
-                 "Command related to promiser '%s' returned code defined as promise kept %d", pp->promiser,
+                 "Command related to promiser '%s' returned code '%d' defined as promise kept", pp->promiser,
                  retcode);
             matched = true;
         }
 
         if (RlistKeyIn(a->classes.retcode_repaired, retcodeStr))
         {
+
             cfPS(ctx, info_or_verbose, PROMISE_RESULT_CHANGE, pp, a,
-                 "Command related to promiser '%s' returned code defined as promise repaired %d", pp->promiser,
+                 "Command related to promiser '%s' returned code '%d' defined as promise repaired", pp->promiser,
                  retcode);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_CHANGE);
             matched = true;
@@ -76,7 +77,7 @@ bool VerifyCommandRetcode(EvalContext *ctx, int retcode, const Attributes *a, co
         if (RlistKeyIn(a->classes.retcode_failed, retcodeStr))
         {
             cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a,
-                 "Command related to promiser '%s' returned code defined as promise failed %d", pp->promiser,
+                 "Command related to promiser '%s' returned code '%d' defined as promise failed", pp->promiser,
                  retcode);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
             result_retcode = false;
@@ -86,7 +87,7 @@ bool VerifyCommandRetcode(EvalContext *ctx, int retcode, const Attributes *a, co
         if (!matched)
         {
             cfPS(ctx, info_or_verbose, PROMISE_RESULT_FAIL, pp, a,
-                 "Command related to promiser '%s' returned code not defined as promise kept, not kept or repaired; setting to failed: %d",
+                 "Command related to promiser '%s' returned code '%d' not defined as promise kept, not kept or repaired; setting to failed",
                  pp->promiser, retcode);
             *result = PromiseResultUpdate(*result, PROMISE_RESULT_FAIL);
             result_retcode = false;
