@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -35,9 +35,11 @@
 #include <files_lib.h>
 #include <files_operators.h>
 #include <hash.h>
+#include <files_copy.h>
 #include <files_edit.h>
 #include <files_editxml.h>
 #include <files_editline.h>
+#include <files_links.h>
 #include <files_properties.h>
 #include <files_select.h>
 #include <item_lib.h>
@@ -56,9 +58,12 @@
 #include <mustache.h>
 #include <known_dirs.h>
 #include <evalfunction.h>
+#include <changes_chroot.h>     /* PrepareChangesChroot(), RecordFileChangedInChroot() */
 
 static PromiseResult FindFilePromiserObjects(EvalContext *ctx, const Promise *pp);
 static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promise *pp);
+static PromiseResult WriteContentFromString(EvalContext *ctx, const char *path, const Attributes *attr,
+                                            const Promise *pp);
 
 /*****************************************************************************/
 
@@ -192,7 +197,8 @@ static bool AttrHasNoAction(const Attributes *attr)
     if (!(attr->transformer || attr->haverename || attr->havedelete ||
           attr->havecopy || attr->create || attr->touch || attr->havelink ||
           attr->haveperms || attr->havechange || attr->acl.acl_entries ||
-          attr->haveedit || attr->haveeditline || attr->haveeditxml))
+          attr->haveedit || attr->haveeditline || attr->haveeditxml ||
+          (attr->content != NULL)))
     {
         return true;
     }
@@ -301,6 +307,8 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     PromiseResult result = PROMISE_RESULT_NOOP;
 
+    char *chrooted_path = NULL;
+
     /* if template_data was specified, it must have been resolved to a data
      * container by now */
     /* check this early to prevent creation of the file below in case of failure */
@@ -314,7 +322,19 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         goto exit;
     }
 
-    if (lstat(path, &oslb) == -1)       /* Careful if the object is a link */
+    if (ChrootChanges())
+    {
+        PrepareChangesChroot(path);
+    }
+
+    const char *changes_path = path;
+    if (ChrootChanges())
+    {
+        chrooted_path = xstrdup(ToChangesChroot(path));
+        changes_path = chrooted_path;
+    }
+
+    if (lstat(changes_path, &oslb) == -1)       /* Careful if the object is a link */
     {
         if ((a.create) || (a.touch))
         {
@@ -324,7 +344,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
             }
             else
             {
-                exists = (lstat(path, &oslb) != -1);
+                exists = (lstat(changes_path, &oslb) != -1);
             }
         }
 
@@ -334,7 +354,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
     {
         if ((a.create) || (a.touch))
         {
-            cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "File '%s' exists as promised", path);
+            RecordNoChange(ctx, pp, &a, "File '%s' exists as promised", path);
         }
         exists = true;
         link = true;
@@ -342,18 +362,18 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     if ((a.havedelete) && (!exists))
     {
-        cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "File '%s' does not exist as promised", path);
+        RecordNoChange(ctx, pp, &a, "File '%s' does not exist as promised", path);
         goto exit;
     }
 
     if (!a.havedepthsearch)     /* if the search is trivial, make sure that we are in the parent dir of the leaf */
     {
-        char basedir[CF_BUFSIZE];
-
         Log(LOG_LEVEL_DEBUG, "Direct file reference '%s', no search implied", path);
-        snprintf(basedir, sizeof(basedir), "%s", path);
 
-        if (strcmp(ReadLastNode(basedir), ".") == 0)
+        char basedir[CF_BUFSIZE];
+        strlcpy(basedir, path, sizeof(basedir));
+
+        if (StringEqual(ReadLastNode(basedir), "."))
         {
             // Handle /.  notation for deletion of directories
             ChopLastNode(basedir);
@@ -361,9 +381,10 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         }
 
         ChopLastNode(basedir);
-        if (safe_chdir(basedir))
+        if (safe_chdir(ToChangesPath(basedir)) != 0)
         {
-            char msg[CF_BUFSIZE];
+            /* TODO: PROMISE_RESULT_FAIL?!?!?!?! */
+            char msg[sizeof(basedir) + 36 + 100]; // 36 for fmt string 100 for error string
             snprintf(msg, sizeof(msg), "Failed to chdir into '%s'. (chdir: '%s')",
                      basedir, GetErrorStr());
             if (errno == ENOLINK)
@@ -385,10 +406,10 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         && (a.haveselect && !SelectLeaf(ctx, path, &oslb, &(a.select)))
         && !(a.havedepthsearch && S_ISDIR(oslb.st_mode)))
     {
-        goto exit;
+        goto skip;
     }
 
-    if (stat(path, &osb) == -1)
+    if (stat(changes_path, &osb) == -1)
     {
         if ((a.create) || (a.touch))
         {
@@ -412,9 +433,10 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         {
             if (a.havedepthsearch)
             {
+                /* TODO: PROMISE_RESULT_DENIED */
                 Log(LOG_LEVEL_DEBUG,
                     "depth_search (recursion) is promised for a base object '%s' that is not a directory",
-                      path);
+                    path);
                 goto exit;
             }
         }
@@ -424,10 +446,19 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     if (a.link.link_children)
     {
-        if (stat(a.link.source, &dsb) != -1)
+        const char *changes_link_source = a.link.source;
+        if (ChrootChanges())
+        {
+            /* Make sure the link source is in the changes chroot. */
+            PrepareChangesChroot(a.link.source);
+            changes_link_source = ToChangesChroot(a.link.source);
+        }
+
+        if (stat(changes_link_source, &dsb) != -1)
         {
             if (!S_ISDIR(dsb.st_mode))
             {
+                /* TODO: PROMISE_RESULT_FAIL */
                 Log(LOG_LEVEL_ERR, "Cannot promise to link the children of '%s' as it is not a directory!",
                       a.link.source);
                 goto exit;
@@ -443,7 +474,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         ) ||
         ((exists || link) && a.havedelete))
     {
-        lstat(path, &oslb);     /* if doesn't exist have to stat again anyway */
+        lstat(changes_path, &oslb);     /* if doesn't exist have to stat again anyway */
 
         DepthSearch(ctx, path, &oslb, 0, &a, pp, oslb.st_dev, &result);
 
@@ -464,7 +495,7 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
             /* unless child nodes were repaired, set a promise kept class */
             if (result == PROMISE_RESULT_NOOP)
             {
-                cfPS(ctx, LOG_LEVEL_VERBOSE, PROMISE_RESULT_NOOP, pp, &a, "Basedir '%s' not promising anything", path);
+                Log(LOG_LEVEL_VERBOSE, "Basedir '%s' not promising anything", path);
             }
         }
     }
@@ -487,7 +518,27 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         result = PromiseResultUpdate(result, ScheduleLinkOperation(ctx, path, a.link.source, &a, pp));
     }
 
-/* Phase 3 - content editing */
+/* Phase 3a - direct content */
+
+    if (a.content)
+    {
+        if (a.haveedit)
+        {
+            // Disallow edits on top of content creation
+            RecordFailure(ctx, pp, &a, "A file promise with content attribute cannot have edit operations");
+            result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
+            goto exit;
+        }
+        Log(LOG_LEVEL_VERBOSE, "Replacing '%s' with content '%s'",
+            path, a.content);
+
+        PromiseResult render_result = WriteContentFromString(ctx, path, &a, pp);
+        result = PromiseResultUpdate(result, render_result);
+
+        goto exit;
+    }
+
+/* Phase 3b - content editing */
 
     if (a.haveedit)
     {
@@ -497,14 +548,14 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
         }
         else
         {
-            cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Promised to edit '%s', but file does not exist", path);
+            RecordFailure(ctx, pp, &a, "Promised to edit '%s', but file does not exist", path);
             result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
         }
     }
 
 // Once more in case a file has been created as a result of editing or copying
 
-    exists = (stat(path, &osb) != -1);
+    exists = (stat(changes_path, &osb) != -1);
 
     if (exists && (S_ISREG(osb.st_mode))
         && (!a.haveselect || SelectLeaf(ctx, path, &osb, &(a.select))))
@@ -514,18 +565,54 @@ static PromiseResult VerifyFilePromise(EvalContext *ctx, char *path, const Promi
 
     if (!exists && a.havechange)
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Promised to monitor '%s' for changes, but file does not exist", path);
+        RecordFailure(ctx, pp, &a, "Promised to monitor '%s' for changes, but file does not exist", path);
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
 exit:
-
+    free(chrooted_path);
     if (AttrHasNoAction(&a))
     {
-        Log(LOG_LEVEL_INFO,
+        Log(LOG_LEVEL_WARNING,
             "No action was requested for file '%s'. Maybe a typo in the policy?", path);
     }
 
+    switch(result)
+    {
+    case PROMISE_RESULT_NOOP:
+        cfPS(ctx, LOG_LEVEL_VERBOSE, result, pp, &a,
+             "No changes done for the files promise '%s'", pp->promiser);
+
+        if (EVAL_MODE == EVAL_MODE_SIMULATE_MANIFEST_FULL)
+        {
+            RecordFileEvaluatedInChroot(path);
+        }
+
+        break;
+    case PROMISE_RESULT_CHANGE:
+        cfPS(ctx, LOG_LEVEL_VERBOSE, result, pp, &a,
+             "files promise '%s' repaired", pp->promiser);
+
+        if (ChrootChanges() && !a.haverename)
+        {
+            /* Record that file was changed in the changes chroot so that we can
+             * later show the diff or manifest. Renames are reported
+             * separately. */
+            RecordFileChangedInChroot(path);
+        }
+
+        break;
+    case PROMISE_RESULT_WARN:
+        cfPS(ctx, LOG_LEVEL_WARNING, result, pp, &a,
+             "Warnings encountered when actuating files promise '%s'", pp->promiser);
+        break;
+    default:
+        cfPS(ctx, LOG_LEVEL_ERR, result, pp, &a,
+             "Errors encountered when actuating files promise '%s'", pp->promiser);
+        break;
+    }
+
+skip:
     YieldCurrentLock(thislock);
 
     ClearExpandedAttributes(&a);
@@ -536,6 +623,78 @@ exit:
 
 /*****************************************************************************/
 
+static PromiseResult WriteContentFromString(EvalContext *ctx, const char *path, const Attributes *attr,
+                                            const Promise *pp)
+{
+    assert(path != NULL);
+    assert(attr != NULL);
+
+    const char *changes_path = path;
+    if (ChrootChanges())
+    {
+        changes_path = ToChangesChroot(path);
+    }
+
+    PromiseResult result = PROMISE_RESULT_NOOP;
+
+    if (attr->content == NULL)
+    {
+        RecordFailure(ctx, pp, attr, "'content' not set for promiser: '%s'", path);
+        return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
+    }
+
+    unsigned char existing_content_digest[EVP_MAX_MD_SIZE + 1] = { 0 };
+    if (access(changes_path, R_OK) == 0)
+    {
+        HashFile(changes_path, existing_content_digest, CF_DEFAULT_DIGEST,
+                 FileNewLineMode(changes_path) == NewLineMode_Native);
+    }
+
+    size_t bytes_to_write = strlen(attr->content);
+    unsigned char promised_content_digest[EVP_MAX_MD_SIZE + 1] = { 0 };
+    HashString(attr->content, strlen(attr->content),
+               promised_content_digest, CF_DEFAULT_DIGEST);
+
+    if (!HashesMatch(existing_content_digest, promised_content_digest, CF_DEFAULT_DIGEST))
+    {
+        if (!MakingChanges(ctx, pp, attr, &result,
+                          "update content of '%s' with content '%s'",
+                           path, attr->content))
+        {
+            return result;
+        }
+
+        FILE *f = safe_fopen(changes_path, "w");
+        if (f == NULL)
+        {
+            RecordFailure(ctx, pp, attr, "Cannot open file '%s' for writing", path);
+            return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
+        }
+
+        Writer *w = FileWriter(f);
+        if (WriterWriteLen(w, attr->content, bytes_to_write) == bytes_to_write )
+        {
+            RecordChange(ctx, pp, attr,
+                         "Updated content of '%s' with content '%s'",
+                         path, attr->content);
+
+            result = PromiseResultUpdate(result, PROMISE_RESULT_CHANGE);
+        }
+        else
+        {
+            RecordFailure(ctx, pp, attr,
+                          "Failed to update content of '%s' with content '%s'",
+                          path, attr->content);
+            result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
+        }
+        WriterClose(w);
+    }
+
+    return result;
+}
+
+/*****************************************************************************/
+
 static PromiseResult RenderTemplateCFEngine(EvalContext *ctx, const Promise *pp,
                                             const Rlist *bundle_args, const Attributes *attr,
                                             EditContext *edcontext)
@@ -601,77 +760,66 @@ static bool SaveBufferCallback(const char *dest_filename, void *param, NewLineMo
     return true;
 }
 
-
 static PromiseResult RenderTemplateMustache(EvalContext *ctx, const Promise *pp, const Attributes *attr,
                                             EditContext *edcontext, const char *template)
 {
     assert(attr != NULL);
-    Attributes a = *attr; // TODO: Try to remove this copy
     PromiseResult result = PROMISE_RESULT_NOOP;
 
+    const JsonElement *template_data = attr->template_data;
     JsonElement *destroy_this = NULL;
 
-    if (a.template_data == NULL)
+    if (template_data == NULL)
     {
-        a.template_data = DefaultTemplateData(ctx, NULL);
-        destroy_this = a.template_data;
+        destroy_this = DefaultTemplateData(ctx, NULL);
+        template_data = destroy_this;
     }
 
     unsigned char existing_output_digest[EVP_MAX_MD_SIZE + 1] = { 0 };
-    if (access(pp->promiser, R_OK) == 0)
+    if (access(edcontext->changes_filename, R_OK) == 0)
     {
-        HashFile(pp->promiser, existing_output_digest, CF_DEFAULT_DIGEST,
+        HashFile(edcontext->changes_filename, existing_output_digest, CF_DEFAULT_DIGEST,
                  edcontext->new_line_mode == NewLineMode_Native);
     }
 
     Buffer *output_buffer = BufferNew();
 
     char *message;
-    if ( strcmp("inline_mustache", a.template_method) == 0)
+    if (strcmp("inline_mustache", attr->template_method) == 0)
     {
         message = xstrdup("inline");
     }
     else
     {
-        message = xstrdup(a.edit_template);
+        message = xstrdup(attr->edit_template);
     }
 
-    if (MustacheRender(output_buffer, template, a.template_data))
+    if (MustacheRender(output_buffer, template, template_data))
     {
         unsigned char rendered_output_digest[EVP_MAX_MD_SIZE + 1] = { 0 };
         HashString(BufferData(output_buffer), BufferSize(output_buffer), rendered_output_digest, CF_DEFAULT_DIGEST);
         if (!HashesMatch(existing_output_digest, rendered_output_digest, CF_DEFAULT_DIGEST))
         {
-            if (a.transaction.action == cfa_warn || DONTDO)
+            if (MakingChanges(ctx, pp, attr, &result,
+                              "update rendering of '%s' from mustache template '%s'",
+                              edcontext->filename, message))
             {
-                cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, &a,
-                     "Need to update rendering of '%s' from mustache template '%s' but policy is dry-run",
-                     pp->promiser, message);
-                result = PromiseResultUpdate(result, PROMISE_RESULT_WARN);
-            }
-            else
-            {
-                if (SaveAsFile(SaveBufferCallback, output_buffer, edcontext->filename, &a, edcontext->new_line_mode))
+                if (SaveAsFile(SaveBufferCallback, output_buffer, edcontext->changes_filename, attr, edcontext->new_line_mode))
                 {
-                    cfPS(ctx, LOG_LEVEL_INFO, PROMISE_RESULT_CHANGE, pp, &a, "Updated rendering of '%s' from mustache template '%s'",
-                         pp->promiser, message);
+                    RecordChange(ctx, pp, attr,
+                                 "Updated rendering of '%s' from mustache template '%s'",
+                                 edcontext->filename, message);
                     result = PromiseResultUpdate(result, PROMISE_RESULT_CHANGE);
-
-                    edcontext->num_rewrites++;
                 }
                 else
                 {
-                    cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Failed to update rendering of '%s' from mustache template '%s'",
-                         pp->promiser, message);
+                    RecordFailure(ctx, pp, attr,
+                                  "Failed to update rendering of '%s' from mustache template '%s'",
+                                  edcontext->filename, message);
                     result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
                 }
             }
         }
-
-        BufferDestroy(output_buffer);
-        free(message);
-        JsonDestroy(destroy_this);
-        return result;
     }
     else
     {
@@ -681,12 +829,14 @@ static PromiseResult RenderTemplateMustache(EvalContext *ctx, const Promise *pp,
          */
         char *tmpl = StringEqual(attr->template_method, "mustache")
                    ? attr->edit_template : attr->edit_template_string;
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, &a, "Error rendering mustache template '%s'", tmpl);
+        RecordFailure(ctx, pp, attr, "Error rendering mustache template '%s'",
+                      tmpl);
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
-        BufferDestroy(output_buffer);
-        JsonDestroy(destroy_this);
-        return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
+    BufferDestroy(output_buffer);
+    JsonDestroy(destroy_this);
+    free(message);
+    return result;
 }
 
 static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Promise *pp, const Attributes *a,
@@ -697,7 +847,7 @@ static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Prom
 
     if (!FileCanOpen(a->edit_template, "r"))
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Template file '%s' could not be opened for reading", a->edit_template);
+        RecordFailure(ctx, pp, a, "Template file '%s' could not be opened for reading", a->edit_template);
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
@@ -711,7 +861,7 @@ static PromiseResult RenderTemplateMustacheFromFile(EvalContext *ctx, const Prom
     }
     if (template_writer == NULL)
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "Could not read template file '%s'", a->edit_template);
+        RecordFailure(ctx, pp, a, "Could not read template file '%s'", a->edit_template);
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
@@ -730,7 +880,7 @@ static PromiseResult RenderTemplateMustacheFromString(EvalContext *ctx, const Pr
     {
         PromiseResult result = PROMISE_RESULT_NOOP;
 
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "'edit_template_string' not set for promiser: '%s'", pp->promiser);
+        RecordFailure(ctx, pp, a, "'edit_template_string' not set for promiser: '%s'", pp->promiser);
         return PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
     }
 
@@ -755,11 +905,12 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, const Attr
     }
 
     EditContext *edcontext = NewEditContext(filename, a);
+    StartLoggingIntoBuffer(LOG_LEVEL_INFO, LOG_LEVEL_INFO);
 
     PromiseResult result = PROMISE_RESULT_NOOP;
     if (edcontext == NULL)
     {
-        cfPS(ctx, LOG_LEVEL_ERR, PROMISE_RESULT_FAIL, pp, a, "File '%s' was marked for editing but could not be opened", filename);
+        RecordFailure(ctx, pp, a, "File '%s' was marked for editing but could not be opened", filename);
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
         goto exit;
     }
@@ -873,6 +1024,14 @@ PromiseResult ScheduleEditOperation(EvalContext *ctx, char *filename, const Attr
 exit:
     FinishEditContext(ctx, edcontext, a, pp, &result);
     YieldCurrentLock(thislock);
+    if (result == PROMISE_RESULT_CHANGE)
+    {
+        CommitLogBuffer();
+    }
+    else
+    {
+        DiscardLogBuffer();
+    }
     return result;
 }
 
@@ -886,8 +1045,46 @@ PromiseResult FindAndVerifyFilesPromises(EvalContext *ctx, const Promise *pp)
 
 /*****************************************************************************/
 
+static DefineClasses GetExpandedClassDefinitionConstraints(const EvalContext *ctx, const Promise *pp)
+{
+    const char *namespace = PromiseGetBundle(pp)->ns;
+    const char *scope = PromiseGetBundle(pp)->name;
+
+    /* Get the unexpanded classes. */
+    DefineClasses c = GetClassDefinitionConstraints(ctx, pp);
+
+    /* Expand the classes just like GetExpandedAttributes() does.
+     * NOTE: The original Rlists are owned by the promise, the new ones need to be
+     *       RlistDestroy()-ed.*/
+    c.change = ExpandList(ctx, namespace, scope, c.change, true);
+    c.failure = ExpandList(ctx, namespace, scope, c.failure, true);
+    c.denied = ExpandList(ctx, namespace, scope, c.denied, true);
+    c.timeout = ExpandList(ctx, namespace, scope, c.timeout, true);
+    c.kept = ExpandList(ctx, namespace, scope, c.kept, true);
+    c.del_change = ExpandList(ctx, namespace, scope, c.del_change, true);
+    c.del_kept = ExpandList(ctx, namespace, scope, c.del_kept, true);
+    c.del_notkept = ExpandList(ctx, namespace, scope, c.del_notkept, true);
+
+    return c;
+}
+
+static void ClearExpandedClassDefinitionConstraints(DefineClasses *c)
+{
+    assert(c != NULL);
+    RlistDestroy(c->change);
+    RlistDestroy(c->failure);
+    RlistDestroy(c->denied);
+    RlistDestroy(c->timeout);
+    RlistDestroy(c->kept);
+    RlistDestroy(c->del_change);
+    RlistDestroy(c->del_kept);
+    RlistDestroy(c->del_notkept);
+}
+
 static PromiseResult FindFilePromiserObjects(EvalContext *ctx, const Promise *pp)
 {
+    assert(pp != NULL);
+
     char *val = PromiseGetConstraintAsRval(pp, "pathtype", RVAL_TYPE_SCALAR);
     int literal = (PromiseGetConstraintAsBoolean(ctx, "copy_from", pp)) || ((val != NULL) && (strcmp(val, "literal") == 0));
 
@@ -902,6 +1099,17 @@ static PromiseResult FindFilePromiserObjects(EvalContext *ctx, const Promise *pp
     else                        // Default is to expand regex paths
     {
         result = PromiseResultUpdate(result, LocateFilePromiserGroup(ctx, pp->promiser, pp, VerifyFilePromise));
+
+        /* Now set the outcome classes for the pp->promiser itself (not the expanded paths). */
+        if (result != PROMISE_RESULT_SKIPPED)
+        {
+            EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "promiser", pp->promiser, CF_DATA_TYPE_STRING, "source=promise");
+            Attributes a = ZeroAttributes;
+            a.classes = GetExpandedClassDefinitionConstraints(ctx, pp);
+            SetPromiseOutcomeClasses(ctx, result, &(a.classes));
+            ClearExpandedClassDefinitionConstraints(&(a.classes));
+            EvalContextVariableRemoveSpecial(ctx, SPECIAL_SCOPE_THIS, "promiser");
+        }
     }
 
     return result;
