@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -25,6 +25,7 @@
 #include <verify_methods.h>
 
 #include <actuator.h>
+#include <audit.h> // FatalError()
 #include <eval_context.h>
 #include <vars.h>
 #include <expand.h>
@@ -159,11 +160,12 @@ PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, const Attributes *
             Variable *var;
             while ((var = VariableTableIteratorNext(iter)))
             {
-                if (!var->ref->num_indices)
+                const VarRef *var_ref = VariableGetRef(var);
+                if (!var_ref->num_indices)
                 {
                     continue;
                 }
-                EvalContextVariableRemove(ctx, var->ref);
+                EvalContextVariableRemove(ctx, var_ref);
             }
             VariableTableIteratorDestroy(iter);
 
@@ -226,6 +228,10 @@ PromiseResult VerifyMethod(EvalContext *ctx, const Rval call, const Attributes *
              "A method attempted to use a bundle '%s' that was apparently not defined",
              BufferData(method_name));
         result = PromiseResultUpdate(result, PROMISE_RESULT_FAIL);
+        if (!EvalContextGetConfig(ctx)->ignore_missing_bundles)
+        {
+            FatalError(ctx, "Aborting due to missing bundle '%s'", BufferData(method_name));
+        }
     }
 
     YieldCurrentLock(thislock);
@@ -248,16 +254,18 @@ static void GetReturnValue(EvalContext *ctx, const Bundle *callee, const Promise
         Variable *result_var = NULL;
         while ((result_var = VariableTableIteratorNext(iter)))
         {
-            assert(result_var->ref->num_indices == 1);
-            if (result_var->ref->num_indices != 1)
+            const VarRef *result_var_ref = VariableGetRef(result_var);
+            assert(result_var_ref->num_indices == 1);
+            if (result_var_ref->num_indices != 1)
             {
                 continue;
             }
 
             VarRef *new_ref = VarRefParseFromBundle(result, PromiseGetBundle(caller));
-            VarRefAddIndex(new_ref, result_var->ref->indices[0]);
+            VarRefAddIndex(new_ref, result_var_ref->indices[0]);
 
-            EvalContextVariablePut(ctx, new_ref, result_var->rval.item, result_var->type, "source=bundle");
+            Rval result_var_rval = VariableGetRval(result_var, true);
+            EvalContextVariablePut(ctx, new_ref, result_var_rval.item, VariableGetType(result_var), "source=bundle");
 
             VarRefDestroy(new_ref);
         }
