@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -27,8 +27,6 @@
 
 #include <cf3.defs.h>
 
-PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp,
-                                          const Attributes *a, char **promise_log_msg,
-                                          LogLevel *log_lvl);
+PromiseResult HandleNewPackagePromiseType(EvalContext *ctx, const Promise *pp, const Attributes *a);
 
 #endif
