@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -461,7 +461,7 @@ static PromiseResult VerifyMountPromise(EvalContext *ctx, char *name, const Attr
     {
         if (!a->mount.unmount)
         {
-            if (!MakeParentDirectory(dir, a->move_obstructions))
+            if (!MakeParentDirectory(dir, a->move_obstructions, NULL))
             {
                 // Could not create parent directory, assume this is okay,
                 // verbose logging in MakeParentDirectory()
@@ -492,7 +492,7 @@ static PromiseResult VerifyMountPromise(EvalContext *ctx, char *name, const Attr
             }
         }
 
-        if (changes)
+        if (changes > 0)
         {
             CF_MOUNTALL = true;
         }
