@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -24,6 +24,20 @@
 
 #include <cf-execd.h>
 
+#ifndef __MINGW32__
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/un.h>
+#include <files_lib.h>
+#include <cf-execd-runagent.h>
+#include <files_names.h>        /* ChopLastNode */
+
+#ifndef AF_LOCAL
+#define AF_LOCAL AF_UNIX
+#endif
+
+#endif
+
 #include <cf-execd-runner.h>
 #include <item_lib.h>
 #include <known_dirs.h>
@@ -43,17 +57,27 @@
 #include <repair.h>
 #include <dbm_api.h>            /* CheckDBRepairFlagFile() */
 #include <string_lib.h>
+#include <acl_tools.h>          /* AllowAccessForUsers() */
 
 #include <cf-windows-functions.h>
 
 #define CF_EXEC_IFELAPSED 0
 #define CF_EXEC_EXPIREAFTER 1
 
+#define CF_EXECD_RUNAGENT_SOCKET_NAME "runagent.socket"
+
+/* The listen() queue doesn't need to be long, new connections are accepted
+ * quickly and handed over to forked child processes so a pile up means some
+ * serious problem and it's better to just throw such connections away. */
+#define CF_EXECD_RUNAGENT_SOCKET_LISTEN_QUEUE 5
+
 static bool PERFORM_DB_CHECK = false;
 static int NO_FORK = false; /* GLOBAL_A */
 static int ONCE = false; /* GLOBAL_A */
 static int WINSERVICE = true; /* GLOBAL_A */
 
+static char *RUNAGENT_SOCKET_DIR = NULL;
+
 static pthread_attr_t threads_attrs; /* GLOBAL_T, initialized by pthread_attr_init */
 
 /*******************************************************************/
@@ -64,10 +88,14 @@ void ThisAgentInit(void);
 static bool ScheduleRun(EvalContext *ctx, Policy **policy, GenericAgentConfig *config,
                         ExecdConfig **execd_config, ExecConfig **exec_config);
 #ifndef __MINGW32__
+static pid_t LocalExecInFork(const ExecConfig *config);
 static void Apoptosis(void);
+static inline bool GetRunagentSocketInfo(struct sockaddr_un *sock_info);
+static inline bool SetRunagentSocketACLs(char *sock_path, StringSet *allow_users);
+#else
+static bool LocalExecInThread(const ExecConfig *config);
 #endif
 
-static bool LocalExecInThread(const ExecConfig *config);
 
 /*******************************************************************/
 /* Command line options                                            */
@@ -103,7 +131,10 @@ static const struct option OPTIONS[] =
     {"ld-library-path", required_argument, 0, 'L'},
     {"color", optional_argument, 0, 'C'},
     {"timestamp", no_argument, 0, 'l'},
+    /* Only long option for the rest */
+    {"ignore-preferred-augments", no_argument, 0, 0},
     {"skip-db-check", optional_argument, 0, 0 },
+    {"with-runagent-socket", required_argument, 0, 0},
     {NULL, 0, 0, '\0'}
 };
 
@@ -127,7 +158,9 @@ static const char *const HINTS[] =
     "Set the internal value of LD_LIBRARY_PATH for child processes",
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Log timestamps on each line of log output",
+    "Ignore def_preferred.json file in favor of def.json",
     "Do not run database integrity checks and repairs at startup",
+    "Specify the directory for the socket for runagent requests or 'no' to disable the socket",
     NULL
 };
 
@@ -145,7 +178,10 @@ int main(int argc, char *argv[])
     EvalContext *ctx = EvalContextNew();
     GenericAgentConfigApply(ctx, config);
 
-    GenericAgentDiscoverContext(ctx, config);
+    const char *program_invocation_name = argv[0];
+    const char *last_dir_sep = strrchr(program_invocation_name, FILE_SEPARATOR);
+    const char *program_name = (last_dir_sep != NULL ? last_dir_sep + 1 : program_invocation_name);
+    GenericAgentDiscoverContext(ctx, config, program_name);
 
     Policy *policy = SelectAndLoadPolicy(config, ctx, false, false);
 
@@ -176,6 +212,8 @@ int main(int argc, char *argv[])
     GenericAgentFinalize(ctx, config);
     ExecConfigDestroy(exec_config);
     ExecdConfigDestroy(execd_config);
+    free(RUNAGENT_SOCKET_DIR);
+    CallCleanupFunctions();
     return 0;
 }
 
@@ -255,7 +293,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'n':
-            DONTDO = true;
+            EVAL_MODE = EVAL_MODE_DRY_RUN;
             config->ignore_locks = true;
             break;
 
@@ -325,17 +363,21 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
         case 0:
         {
             const char *const option_name = OPTIONS[longopt_idx].name;
-            if (StringSafeEqual(option_name, "skip-db-check"))
+            if (StringEqual(option_name, "ignore-preferred-augments"))
+            {
+                config->ignore_preferred_augments = true;
+            }
+            else if (StringEqual(option_name, "skip-db-check"))
             {
                 if (optarg == NULL)
                 {
                     PERFORM_DB_CHECK = false; // Skip (no arg), check = false
                 }
-                else if (StringSafeEqual_IgnoreCase(optarg, "yes"))
+                else if (StringEqual_IgnoreCase(optarg, "yes"))
                 {
                     PERFORM_DB_CHECK = false; // Skip = yes, check = false
                 }
-                else if (StringSafeEqual_IgnoreCase(optarg, "no"))
+                else if (StringEqual_IgnoreCase(optarg, "no"))
                 {
                     PERFORM_DB_CHECK = true; // Skip = no, check = true
                 }
@@ -347,6 +389,12 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                     DoCleanupAndExit(EXIT_FAILURE);
                 }
             }
+            else if (StringEqual(option_name, "with-runagent-socket"))
+            {
+                assert(optarg != NULL); /* required_argument */
+                RUNAGENT_SOCKET_DIR = xstrdup(optarg);
+            }
+
             break;
         }
         default:
@@ -378,20 +426,310 @@ void ThisAgentInit(void)
 
 /*****************************************************************************/
 
-// msg should include exactly one reference to unsigned int.
-static unsigned int MaybeSleepLog(LogLevel level, const char *msg, unsigned int seconds)
+
+#ifndef __MINGW32__
+
+static inline bool UsingRunagentSocket()
+{
+    /* No runagent socket dir specified (use the default) or a directory
+     * specified ("no" disables the functionality). */
+    return ((RUNAGENT_SOCKET_DIR == NULL) || (!StringEqual_IgnoreCase(RUNAGENT_SOCKET_DIR, "no")));
+}
+
+/**
+ * Sleep for the given number of seconds while handling requests from sockets.
+ *
+ * @return Whether to terminate (skip any further actions) or not.
+ */
+static bool HandleRequestsOrSleep(time_t seconds, const char *reason,
+                                  int runagent_socket, const char *local_run_command)
+{
+    if (IsPendingTermination())
+    {
+        return true;
+    }
+
+    Log(LOG_LEVEL_VERBOSE, "Sleeping for %s %ju seconds", reason, (intmax_t) seconds);
+
+    if (runagent_socket >= 0)
+    {
+        time_t sleep_started = time(NULL);
+        struct timeval remaining = {seconds, 0};
+        while (remaining.tv_sec != 0)
+        {
+            fd_set rfds;
+            FD_ZERO(&rfds);
+            FD_SET(runagent_socket, &rfds);
+
+            int ret = select(runagent_socket + 1, &rfds, NULL, NULL, &remaining);
+            if ((ret == -1) && (errno != EINTR))
+            {
+                /* unexpected error */
+                Log(LOG_LEVEL_ERR, "Failed to sleep for %s using select(): %s",
+                    reason, GetErrorStr());
+            }
+            else if (ret == 0)
+            {
+                /* timeout -- slept for the specified time */
+                remaining.tv_sec = 0;
+            }
+            else
+            {
+                /* runagent_socket ready or signal received (EINTR) */
+
+                // We are sleeping above, so make sure a terminating signal did not
+                // arrive during that time.
+                if (IsPendingTermination())
+                {
+                    return true;
+                }
+
+                if (ret > 0)
+                {
+                    assert(FD_ISSET(runagent_socket, &rfds));
+                    int data_socket = accept(runagent_socket, NULL, NULL);
+                    pid_t pid = fork();
+                    if (pid == 0)
+                    {
+                        /* child */
+                        signal(SIGPIPE, SIG_DFL);
+                        HandleRunagentRequest(data_socket, local_run_command);
+                        _exit(EXIT_SUCCESS);
+                    }
+                    else if (pid == -1)
+                    {
+                        /* error */
+                        Log(LOG_LEVEL_ERR, "Failed to fork runagent request handler: %s",
+                            GetErrorStr());
+                    }
+                    /* parent: nothing more to do, go back to sleep */
+                }
+
+                remaining.tv_sec = MAX(0, seconds - (time(NULL) - sleep_started));
+            }
+        }
+    }
+    else
+    {
+        sleep(seconds);
+    }
+
+    // We are sleeping above, so make sure a terminating signal did not
+    // arrive during that time.
+    if (IsPendingTermination())
+    {
+        return true;
+    }
+
+    return false;
+}
+
+static void CFExecdMainLoop(EvalContext *ctx, Policy **policy, GenericAgentConfig *config,
+                            ExecdConfig **execd_config, ExecConfig **exec_config,
+                            int runagent_socket)
+{
+    bool terminate = false;
+    while (!IsPendingTermination())
+    {
+        /* reap child processes (if any) */
+        while (waitpid(-1, NULL, WNOHANG) > 0)
+        {
+            Log(LOG_LEVEL_DEBUG, "Reaped child process");
+        }
+
+        if (ScheduleRun(ctx, policy, config, execd_config, exec_config))
+        {
+            terminate = HandleRequestsOrSleep((*execd_config)->splay_time, "splay time",
+                                              runagent_socket, (*execd_config)->local_run_command);
+            if (terminate)
+            {
+                break;
+            }
+            pid_t child_pid = LocalExecInFork(*exec_config);
+            if (child_pid < 0)
+            {
+                Log(LOG_LEVEL_INFO,
+                    "Unable to run agent in a fork, falling back to blocking execution");
+                LocalExec(*exec_config);
+            }
+        }
+        /* 1 Minute resolution is enough */
+        terminate = HandleRequestsOrSleep(CFPULSETIME, "pulse time", runagent_socket,
+                                          (*execd_config)->local_run_command);
+        if (terminate)
+        {
+            break;
+        }
+    }
+
+    /* Remove the runagent socket (if any). */
+    if (UsingRunagentSocket())
+    {
+        struct sockaddr_un sock_info;
+        if (GetRunagentSocketInfo(&sock_info))
+        {
+            unlink(sock_info.sun_path);
+        }
+    }
+}
+
+static inline bool GetRunagentSocketInfo(struct sockaddr_un *sock_info)
+{
+    assert(sock_info != NULL);
+
+    memset(sock_info, 0, sizeof(*sock_info));
+
+    /* This can easily fail if GetStateDir() returns some long path,
+     * 'sock_info.sun_path' is limited to 140 characters or even
+     * fewer. "/var/cfengine/state" is fine, crazy long temporary state
+     * directories used in the tests are too long. */
+    int ret;
+    if (RUNAGENT_SOCKET_DIR == NULL)
+    {
+        ret = snprintf(sock_info->sun_path, sizeof(sock_info->sun_path) - 1,
+                       "%s/cf-execd.sockets/"CF_EXECD_RUNAGENT_SOCKET_NAME, GetStateDir());
+    }
+    else
+    {
+        ret = snprintf(sock_info->sun_path, sizeof(sock_info->sun_path) - 1,
+                       "%s/"CF_EXECD_RUNAGENT_SOCKET_NAME, RUNAGENT_SOCKET_DIR);
+    }
+    return ((ret > 0) && ((size_t) ret <= (sizeof(sock_info->sun_path) - 1)));
+}
+
+static inline bool SetRunagentSocketACLs(char *sock_path, StringSet *allow_users)
+{
+    /* Allow access to the socket (rw) */
+    bool success = AllowAccessForUsers(sock_path, allow_users, true, false);
+
+    /* Need to ensure access to the parent folder too (rx) */
+    if (success)
+    {
+        ChopLastNode(sock_path);
+        success = AllowAccessForUsers(sock_path, allow_users, false, true);
+    }
+    return success;
+}
+
+static int SetupRunagentSocket(const ExecdConfig *execd_config)
+{
+    assert(execd_config != NULL);
+
+    int runagent_socket = -1;
+
+    struct sockaddr_un sock_info;
+    if (GetRunagentSocketInfo(&sock_info))
+    {
+        sock_info.sun_family = AF_LOCAL;
+
+        bool created;
+        MakeParentDirectory(sock_info.sun_path, true, &created);
+
+        /* Make sure the permissions are correct if the directory was created
+         * (note: this code doesn't run on Windows). */
+        if (created)
+        {
+            char *last_slash = strrchr(sock_info.sun_path, '/');
+            *last_slash = '\0';
+            chmod(sock_info.sun_path, (mode_t) 0750);
+            *last_slash = '/';
+        }
+
+        /* Remove potential left-overs from old processes. */
+        unlink(sock_info.sun_path);
+
+        runagent_socket = socket(AF_LOCAL, SOCK_STREAM, 0);
+        assert(runagent_socket >= 0);
+    }
+    if (runagent_socket < 0)
+    {
+        Log(LOG_LEVEL_ERR, "Failed to create socket for runagent requests");
+    }
+    else
+    {
+        int ret = bind(runagent_socket, (const struct sockaddr *) &sock_info, sizeof(sock_info));
+        assert(ret == 0);
+        if (ret == -1)
+        {
+            Log(LOG_LEVEL_ERR, "Failed to bind the runagent socket: %s", GetErrorStr());
+            close(runagent_socket);
+            runagent_socket = -1;
+        }
+        else
+        {
+            ret = listen(runagent_socket, CF_EXECD_RUNAGENT_SOCKET_LISTEN_QUEUE);
+            assert(ret == 0);
+            if (ret == -1)
+            {
+                Log(LOG_LEVEL_ERR, "Failed to listen on runagent socket: %s", GetErrorStr());
+                close(runagent_socket);
+                runagent_socket = -1;
+            }
+        }
+        if (StringSetSize(execd_config->runagent_allow_users) > 0)
+        {
+            bool success = SetRunagentSocketACLs(sock_info.sun_path,
+                                                 execd_config->runagent_allow_users);
+            if (!success)
+            {
+                Log(LOG_LEVEL_ERR,
+                    "Failed to allow runagent_socket_allow_users users access the runagent socket");
+                /* keep going anyway */
+            }
+        }
+    }
+    return runagent_socket;
+}
+
+#else  /* ! __MINGW32__ */
+
+/**
+ * Sleep if not pending termination and log a message.
+ *
+ * @note: #msg_format should include exactly one "%u".
+ */
+static inline unsigned int MaybeSleepLog(LogLevel level, const char *msg_format, unsigned int seconds)
 {
     if (IsPendingTermination())
     {
         return seconds;
     }
 
-    Log(level, msg, seconds);
+    Log(level, msg_format, seconds);
 
     return sleep(seconds);
 }
 
-/*****************************************************************************/
+static void CFExecdMainLoop(EvalContext *ctx, Policy **policy, GenericAgentConfig *config,
+                            ExecdConfig **execd_config, ExecConfig **exec_config,
+                            ARG_UNUSED int runagent_socket)
+{
+    while (!IsPendingTermination())
+    {
+        if (ScheduleRun(ctx, policy, config, execd_config, exec_config))
+        {
+            MaybeSleepLog(LOG_LEVEL_VERBOSE,
+                          "Sleeping for splaytime %u seconds",
+                          (*execd_config)->splay_time);
+
+            // We are sleeping above, so make sure a terminating signal did not
+            // arrive during that time.
+            if (IsPendingTermination())
+            {
+                break;
+            }
+            if (!LocalExecInThread(*exec_config))
+            {
+                Log(LOG_LEVEL_INFO,
+                    "Unable to run agent in thread, falling back to blocking execution");
+                LocalExec(*exec_config);
+            }
+        }
+        /* 1 Minute resolution is enough */
+        MaybeSleepLog(LOG_LEVEL_VERBOSE, "Sleeping for pulse time %u seconds...", CFPULSETIME);
+    }
+}
+#endif  /* ! __MINGW32__ */
 
 /* Might be called back from NovaWin_StartExecService */
 void StartServer(EvalContext *ctx, Policy *policy, GenericAgentConfig *config, ExecdConfig **execd_config, ExecConfig **exec_config)
@@ -441,6 +779,15 @@ void StartServer(EvalContext *ctx, Policy *policy, GenericAgentConfig *config, E
 
     umask(077);
 
+    int runagent_socket = -1;
+
+#ifndef __MINGW32__
+    if (UsingRunagentSocket())
+    {
+        runagent_socket = SetupRunagentSocket(*execd_config);
+    }
+#endif
+
     if (ONCE)
     {
         LocalExec(*exec_config);
@@ -448,66 +795,40 @@ void StartServer(EvalContext *ctx, Policy *policy, GenericAgentConfig *config, E
     }
     else
     {
-        while (!IsPendingTermination())
-        {
-            if (ScheduleRun(ctx, &policy, config, execd_config, exec_config))
-            {
-                MaybeSleepLog(LOG_LEVEL_VERBOSE,
-                              "Sleeping for splaytime %u seconds",
-                              (*execd_config)->splay_time);
-
-                // We are sleeping both above and inside ScheduleRun(), so make
-                // sure a terminating signal did not arrive during that time.
-                if (IsPendingTermination())
-                {
-                    break;
-                }
-
-                if (!LocalExecInThread(*exec_config))
-                {
-                    Log(LOG_LEVEL_INFO,
-                        "Unable to run agent in thread, falling back to blocking execution");
-                    LocalExec(*exec_config);
-                }
-            }
-        }
+        CFExecdMainLoop(ctx, &policy, config, execd_config, exec_config, runagent_socket);
     }
     PolicyDestroy(policy);
 }
 
 /*****************************************************************************/
 
-static void Thread_AllSignalsBlock(void)
-{
 #ifndef __MINGW32__
-    sigset_t sigmask;
-    sigfillset(&sigmask);
-    int ret = pthread_sigmask(SIG_SETMASK, &sigmask, NULL);
-    if (ret != 0)
+static pid_t LocalExecInFork(const ExecConfig *config)
+{
+    Log(LOG_LEVEL_VERBOSE, "Forking for exec_command execution");
+
+    pid_t pid = fork();
+    if (pid == -1)
     {
-        Log(LOG_LEVEL_ERR,
-            "Unable to block signals in child thread,"
-            " killing cf-execd might fail (pthread_sigmask: %s)",
+        Log(LOG_LEVEL_ERR, "Failed to fork for exec_command execution: %s",
             GetErrorStr());
+        return -1;
     }
-#endif
-}
-
-static void Thread_AllSignalsUnblock(void)
-{
-#ifndef __MINGW32__
-    sigset_t sigmask;
-    sigemptyset(&sigmask);
-    int ret = pthread_sigmask(SIG_SETMASK, &sigmask, NULL);
-    if (ret != 0)
+    else if (pid == 0)
     {
-        Log(LOG_LEVEL_ERR,
-            "Unable to unblock signals again (pthread_sigmask: %s)",
-            GetErrorStr());
+        /* child */
+        LocalExec(config);
+        Log(LOG_LEVEL_VERBOSE, "Finished exec_command execution, terminating the forked process");
+        _exit(0);
+    }
+    else
+    {
+        /* parent */
+        return pid;
     }
-#endif
 }
 
+#else
 static void *LocalExecThread(void *param)
 {
     ExecConfig *config = (ExecConfig *) param;
@@ -524,16 +845,7 @@ static bool LocalExecInThread(const ExecConfig *config)
     pthread_t tid;
 
     Log(LOG_LEVEL_VERBOSE, "Spawning thread for exec_command execution");
-
-    /* ENT-3147: Block all signals so that child thread inherits it and
-     * signals get only delivered to the main thread. Unblock all signals
-     * right after thread has been spawned. */
-    Thread_AllSignalsBlock();
-
     int ret = pthread_create(&tid, &threads_attrs, LocalExecThread, thread_config);
-
-    Thread_AllSignalsUnblock();
-
     if (ret != 0)
     {
         ExecConfigDestroy(thread_config);
@@ -544,6 +856,7 @@ static bool LocalExecInThread(const ExecConfig *config)
 
     return true;
 }
+#endif  /* ! __MINGW32__ */
 
 #ifndef __MINGW32__
 
@@ -646,9 +959,6 @@ static Reload CheckNewPromises(GenericAgentConfig *config)
 static bool ScheduleRun(EvalContext *ctx, Policy **policy, GenericAgentConfig *config,
                         ExecdConfig **execd_config, ExecConfig **exec_config)
 {
-    /* 1 Minute resolution is enough */
-    MaybeSleepLog(LOG_LEVEL_VERBOSE, "Sleeping for pulse time %u seconds...", CFPULSETIME);
-
     /*
      * FIXME: this logic duplicates the one from cf-serverd.c. Unify ASAP.
      */
@@ -670,7 +980,7 @@ static bool ScheduleRun(EvalContext *ctx, Policy **policy, GenericAgentConfig *c
         UpdateLastPolicyUpdateTime(ctx);
 
         DetectEnvironment(ctx);
-        GenericAgentDiscoverContext(ctx, config);
+        GenericAgentDiscoverContext(ctx, config, NULL);
 
         EvalContextClassPutHard(ctx, CF_AGENTTYPES[AGENT_TYPE_EXECUTOR], "cfe_internal,source=agent");
 
@@ -679,6 +989,16 @@ static bool ScheduleRun(EvalContext *ctx, Policy **policy, GenericAgentConfig *c
 
         GenericAgentConfigSetBundleSequence(config, NULL);
 
+#ifndef __MINGW32__
+        /* Take over the runagent_socket_allow_users set for comparison. */
+        StringSet *old_runagent_allow_users = NULL;
+        if (UsingRunagentSocket())
+        {
+            old_runagent_allow_users = (*execd_config)->runagent_allow_users;
+            (*execd_config)->runagent_allow_users = NULL;
+        }
+#endif
+
         *policy = LoadPolicy(ctx, config);
         ExecConfigDestroy(*exec_config);
         ExecdConfigDestroy(*execd_config);
@@ -686,6 +1006,35 @@ static bool ScheduleRun(EvalContext *ctx, Policy **policy, GenericAgentConfig *c
         *exec_config = ExecConfigNew(!ONCE, ctx, *policy);
         *execd_config = ExecdConfigNew(ctx, *policy);
 
+#ifndef __MINGW32__
+        if (UsingRunagentSocket())
+        {
+            /* Check if the old list and the new one differ. */
+            if (!StringSetIsEqual(old_runagent_allow_users,
+                                  (*execd_config)->runagent_allow_users))
+            {
+                struct sockaddr_un sock_info;
+                if (GetRunagentSocketInfo(&sock_info))
+                {
+                    bool success = SetRunagentSocketACLs(sock_info.sun_path,
+                                                         (*execd_config)->runagent_allow_users);
+                    if (!success)
+                    {
+                        Log(LOG_LEVEL_ERR,
+                            "Failed to allow new runagent_socket_allow_users users access the runagent socket"
+                            " (on policy reload)");
+                        /* keep going anyway */
+                    }
+                }
+                else
+                {
+                    Log(LOG_LEVEL_ERR, "Failed to get runagent.socket path");
+                }
+            }
+            StringSetDestroy(old_runagent_allow_users);
+        }
+#endif
+
         SetFacility((*execd_config)->log_facility);
     }
     else
