@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -28,6 +28,8 @@
 #include <eval_context.h>
 #include <conversion.h>
 #include <string_lib.h>
+#include <mod_common.h>         /* CFS_CONTROLBODY, SERVER_CONTROL_CFRUNCOMMAND */
+#include <expand.h>             /* ExpandScalar() */
 
 /* Get a number in the interval [0, 1) for a calculation of a pseudo-random delay */
 static double GetSplay(void)
@@ -56,6 +58,7 @@ ExecdConfig *ExecdConfigNew(const EvalContext *ctx, const Policy *policy)
     StringSetAdd(execd_config->schedule, xstrdup("Min55"));
     execd_config->splay_time = 0;
     execd_config->log_facility = xstrdup("LOG_USER");
+    execd_config->runagent_allow_users = StringSetNew();
 
     Seq *constraints = ControlBodyConstraints(policy, AGENT_TYPE_EXECUTOR);
     if (constraints)
@@ -81,18 +84,18 @@ ExecdConfig *ExecdConfigNew(const EvalContext *ctx, const Policy *policy)
                                  cp->lval);
             }
 
-            if (strcmp(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_EXECUTORFACILITY].lval) == 0)
+            if (StringEqual(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_EXECUTORFACILITY].lval))
             {
                 free(execd_config->log_facility);
                 execd_config->log_facility = xstrdup(value);
                 Log(LOG_LEVEL_DEBUG, "executorfacility '%s'", execd_config->log_facility);
             }
-            else if (strcmp(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_SPLAYTIME].lval) == 0)
+            else if (StringEqual(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_SPLAYTIME].lval))
             {
                 int time = IntFromString(value);
                 execd_config->splay_time = (int) (time * SECONDS_PER_MINUTE * GetSplay());
             }
-            else if (strcmp(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_SCHEDULE].lval) == 0)
+            else if (StringEqual(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_SCHEDULE].lval))
             {
                 Log(LOG_LEVEL_DEBUG, "Loading user-defined schedule...");
                 StringSetClear(execd_config->schedule);
@@ -103,6 +106,40 @@ ExecdConfig *ExecdConfigNew(const EvalContext *ctx, const Policy *policy)
                     Log(LOG_LEVEL_DEBUG, "Adding '%s'", RlistScalarValue(rp));
                 }
             }
+            else if (StringEqual(cp->lval, CFEX_CONTROLBODY[EXEC_CONTROL_RUNAGENT_ALLOW_USERS].lval))
+            {
+                assert(t == CF_DATA_TYPE_STRING_LIST); /* should be enforced by parser */
+                for (const Rlist *rp = value; rp; rp = rp->next)
+                {
+                    Log(LOG_LEVEL_DEBUG, "User '%s' allowed to use the runagent.socket", RlistScalarValue(rp));
+                    StringSetAdd(execd_config->runagent_allow_users, xstrdup(RlistScalarValue(rp)));
+                }
+            }
+        }
+    }
+
+    /* We need to pull the 'cfruncommand' value from 'body control server' for
+     * local run-agent requests. */
+    constraints = ControlBodyConstraints(policy, AGENT_TYPE_SERVER);
+    if (constraints != NULL)
+    {
+        const size_t length = SeqLength(constraints);
+        for (size_t i = 0; i < length; i++)
+        {
+            Constraint *cp = SeqAt(constraints, i);
+
+            if (!IsDefinedClass(ctx, cp->classes))
+            {
+                continue;
+            }
+            if (StringEqual(cp->lval, CFS_CONTROLBODY[SERVER_CONTROL_CFRUNCOMMAND].lval))
+            {
+                assert(cp->rval.type == RVAL_TYPE_SCALAR);
+                execd_config->local_run_command = ExpandScalar(ctx, NULL, NULL,
+                                                               RvalScalarValue(cp->rval),
+                                                               NULL);
+                break;
+            }
         }
     }
 
@@ -111,10 +148,12 @@ ExecdConfig *ExecdConfigNew(const EvalContext *ctx, const Policy *policy)
 
 void ExecdConfigDestroy(ExecdConfig *execd_config)
 {
-    if (execd_config)
+    if (execd_config != NULL)
     {
         free(execd_config->log_facility);
         StringSetDestroy(execd_config->schedule);
+        StringSetDestroy(execd_config->runagent_allow_users);
+        free(execd_config->local_run_command);
     }
     free(execd_config);
 }
