@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -35,6 +35,8 @@ typedef struct ExecdConfig
     StringSet *schedule;
     int splay_time;
     char *log_facility;
+    StringSet *runagent_allow_users;
+    char *local_run_command;
 } ExecdConfig;
 
 ExecdConfig *ExecdConfigNew(const EvalContext *ctx, const Policy *policy);
