@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -90,6 +90,8 @@ static const struct option OPTIONS[] =
     {"tcpdump", no_argument, 0, 'T'},
     {"color", optional_argument, 0, 'C'},
     {"timestamp", no_argument, 0, 'l'},
+    /* Only long option for the rest */
+    {"ignore-preferred-augments", no_argument, 0, 0},
     {NULL, 0, 0, '\0'}
 };
 
@@ -110,6 +112,7 @@ static const char *const HINTS[] =
     "Interface with tcpdump if available to collect data about network",
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Log timestamps on each line of log output",
+    "Ignore def_preferred.json file in favor of def.json",
     NULL
 };
 
@@ -149,8 +152,9 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
     int c;
     GenericAgentConfig *config = GenericAgentConfigNewDefault(AGENT_TYPE_MONITOR, GetTTYInteractive());
 
+    int longopt_idx;
     while ((c = getopt_long(argc, argv, "dvnIf:g:VSxHTKMFhC::l",
-                            OPTIONS, NULL)) != -1)
+                            OPTIONS, &longopt_idx)) != -1)
     {
         switch (c)
         {
@@ -236,6 +240,17 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             LoggingEnableTimestamps(true);
             break;
 
+        /* long options only */
+        case 0:
+        {
+            const char *const option_name = OPTIONS[longopt_idx].name;
+            if (StringEqual(option_name, "ignore-preferred-augments"))
+            {
+                config->ignore_preferred_augments = true;
+            }
+            break;
+        }
+
         default:
         {
             Writer *w = FileWriter(stdout);
