@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -199,7 +199,7 @@ void SetMeasurementPromises(Item ** classlist)
     {
         if (stored != NULL)
         {
-            if (sizeof(entry) < vsize)
+            if (sizeof(entry) < (size_t) vsize)
             {
                 Log(LOG_LEVEL_ERR, "Invalid entry in measurements database. Expected size: %zu, actual size: %d", sizeof(entry), vsize);
                 continue;
