@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -90,6 +90,7 @@ static const struct option OPTIONS[] =
     {"tag-release", required_argument, 0, 'T'},
     {"timestamp", no_argument, 0, 'l'},
     /* Only long option for the rest */
+    {"ignore-preferred-augments", no_argument, 0, 0},
     {"log-modules", required_argument, 0, 0},
     {NULL, 0, 0, '\0'}
 };
@@ -119,6 +120,7 @@ static const char *const HINTS[] =
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Tag a directory with promises.cf with cf_promises_validated and cf_promises_release_id",
     "Log timestamps on each line of log output",
+    "Ignore def_preferred.json file in favor of def.json",
     "Enable even more detailed debug logging for specific areas of the implementation. Use together with '-d'. Use --log-modules=help for a list of available modules",
     NULL
 };
@@ -249,7 +251,6 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
     extern char *optarg;
     int c;
     GenericAgentConfig *config = GenericAgentConfigNewDefault(AGENT_TYPE_COMMON, GetTTYInteractive());
-    config->tag_release_dir = NULL;
 
     int longopt_idx;
     while ((c = getopt_long(argc, argv, "dvnIw:f:g:D:N:VSrxMb:i:p:s:cg:hW:C::T:l",
@@ -283,9 +284,13 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'w':
+        {
             Log(LOG_LEVEL_INFO, "Setting workdir to '%s'", optarg);
-            putenv_wrapper(StringConcatenate(2, "CFENGINE_TEST_OVERRIDE_WORKDIR=", optarg));
+            char *str = StringConcatenate(2, "CFENGINE_TEST_OVERRIDE_WORKDIR=", optarg);
+            putenv_wrapper(str);
+            free(str);
             break;
+        }
 
         case 'c':
             config->check_runnable = true;
@@ -405,7 +410,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'n':
-            DONTDO = true;
+            EVAL_MODE = EVAL_MODE_DRY_RUN;
             config->ignore_locks = true;
             break;
 
@@ -474,8 +479,13 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
 
         /* long options only */
         case 0:
-
-            if (strcmp(OPTIONS[longopt_idx].name, "log-modules") == 0)
+        {
+            const char *const option_name = OPTIONS[longopt_idx].name;
+            if (StringEqual(option_name, "ignore-preferred-augments"))
+            {
+                config->ignore_preferred_augments = true;
+            }
+            else if (StringEqual(option_name, "log-modules"))
             {
                 bool ret = LogEnableModulesFromString(optarg);
                 if (!ret)
@@ -484,7 +494,7 @@ GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
             }
             break;
-
+        }
         default:
         {
             Writer *w = FileWriter(stdout);
