@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -114,6 +114,7 @@ static const struct option OPTIONS[] =
     {"color", optional_argument, 0, 'C'},
     {"timestamp", no_argument, 0, 'l'},
     /* Only long option for the rest */
+    {"ignore-preferred-augments", no_argument, 0, 0},
     {"log-modules", required_argument, 0, 0},
     {"remote-bundles", required_argument, 0, 0},
     {NULL, 0, 0, '\0'}
@@ -139,6 +140,7 @@ static const char *const HINTS[] =
     "Connection timeout, seconds",
     "Enable colorized output. Possible values: 'always', 'auto', 'never'. If option is used, the default value is 'auto'",
     "Log timestamps on each line of log output",
+    "Ignore def_preferred.json file in favor of def.json",
     "Enable even more detailed debug logging for specific areas of the implementation. Use together with '-d'. Use --log-modules=help for a list of available modules",
     "Bundles to execute on the remote agent",
     NULL
@@ -392,7 +394,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
             break;
 
         case 'n':
-            DONTDO = true;
+            EVAL_MODE = EVAL_MODE_DRY_RUN;
             config->ignore_locks = true;
             break;
 
@@ -446,8 +448,13 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
 
         /* long options only */
         case 0:
-
-            if (strcmp(OPTIONS[longopt_idx].name, "log-modules") == 0)
+        {
+            const char *const option_name = OPTIONS[longopt_idx].name;
+            if (StringEqual(option_name, "ignore-preferred-augments"))
+            {
+                config->ignore_preferred_augments = true;
+            }
+            else if (strcmp(OPTIONS[longopt_idx].name, "log-modules") == 0)
             {
                 bool ret = LogEnableModulesFromString(optarg);
                 if (!ret)
@@ -467,7 +474,7 @@ static GenericAgentConfig *CheckOpts(int argc, char **argv)
                 }
             }
             break;
-
+        }
         default:
         {
             Writer *w = FileWriter(stdout);
@@ -695,9 +702,15 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy)
 
             if (strcmp(cp->lval, CFR_CONTROLBODY[RUNAGENT_CONTROL_OUTPUT_DIRECTORY].lval) == 0)
             {
-                if (IsAbsPath(value))
+                if (strlen(value) >= sizeof(OUTPUT_DIRECTORY))
                 {
-                    strlcpy(OUTPUT_DIRECTORY, value, CF_BUFSIZE);
+                    Log(LOG_LEVEL_ERR,
+                        "Could not set output direcory to '%s' - too long path",
+                        (const char *) value);
+                }
+                else if (IsAbsPath(value))
+                {
+                    strlcpy(OUTPUT_DIRECTORY, value, sizeof(OUTPUT_DIRECTORY));
                     Log(LOG_LEVEL_VERBOSE, "Setting output direcory to '%s'", OUTPUT_DIRECTORY);
                 }
                 continue;
@@ -857,16 +870,16 @@ static int HailExec(AgentConnection *conn, char *peer)
 
 static FILE *NewStream(char *name)
 {
-    char filename[CF_BUFSIZE];
+    char *filename;
 
     if (OUTPUT_DIRECTORY[0] != '\0')
     {
-        snprintf(filename, CF_BUFSIZE, "%s/%s_runagent.out", OUTPUT_DIRECTORY, name);
+        xasprintf(&filename, "%s/%s_runagent.out", OUTPUT_DIRECTORY, name);
     }
     else
     {
-        snprintf(filename, CF_BUFSIZE, "%s%coutputs%c%s_runagent.out",
-                 GetWorkDir(), FILE_SEPARATOR, FILE_SEPARATOR, name);
+        xasprintf(&filename, "%s%coutputs%c%s_runagent.out",
+                  GetWorkDir(), FILE_SEPARATOR, FILE_SEPARATOR, name);
     }
 
     FILE *fp;
@@ -886,5 +899,6 @@ static FILE *NewStream(char *name)
         fp = stdout;
     }
 
+    free(filename);
     return fp;
 }
