@@ -1,5 +1,5 @@
 /*
-   Copyright 2020 Northern.tech AS
+   Copyright 2021 Northern.tech AS
 
    This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -368,7 +368,7 @@ static bool RSAEncrypt(Seq *rsa_keys, const char *input_path, const char *output
     char header[MAX_HEADER_LEN + 2] = "Version: 1.0\n";
     size_t header_len = strlen(header);
     ssize_t n_written = FullWrite(fileno(output_file), header, header_len);
-    if (n_written != header_len)
+    if (n_written < 0 || (size_t) n_written != header_len)
     {
         Log(LOG_LEVEL_ERR, "Failed to write header to the output file '%s'", output_path);
         success = false;
@@ -382,7 +382,7 @@ static bool RSAEncrypt(Seq *rsa_keys, const char *input_path, const char *output
         free(key_digest);
         assert(header_len <= (MAX_HEADER_LEN + 2));
         n_written = FullWrite(fileno(output_file), header, header_len);
-        if (n_written != header_len)
+        if ((size_t) n_written != header_len)
         {
             Log(LOG_LEVEL_ERR, "Failed to write header to the output file '%s'", output_path);
             success = false;
@@ -574,6 +574,13 @@ static inline bool ParseHeader(FILE *input_file, char *key, char *value)
 
 static bool ParseHeaders(FILE *input_file, RSA *privkey, size_t *enc_key_pos, size_t *n_enc_keys)
 {
+    assert(enc_key_pos != NULL);
+    assert(n_enc_keys != NULL);
+
+    /* Make sure these are always set by this function. */
+    *enc_key_pos = 0;
+    *n_enc_keys = 0;
+
     /* Actually works for a private RSA key too because it contains the public
      * key. */
     char *key_digest = GetPubkeyDigest(privkey);
@@ -928,7 +935,7 @@ int main(int argc, char *argv[])
     input_path = argv[optind];
 
     /* Some more unexpected arguments? */
-    if (argc > (optind + offset))
+    if ((size_t) argc > (optind + offset))
     {
         Log(LOG_LEVEL_ERR, "Unexpected non-option argument: '%s'", argv[optind + 1]);
         DoCleanupAndExit(EXIT_FAILURE);
