@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -88,7 +88,7 @@ static bool access_CheckResource(const struct resource_acl *acl,
          */
 
         const char *rule = NULL;
-        for (int i = 0; i < StrList_Len(acl->admit.ips); i++)
+        for (size_t i = 0; i < StrList_Len(acl->admit.ips); i++)
         {
             if (FuzzySetMatch(StrList_At(acl->admit.ips, i), ipaddr) == 0 ||
                 /* Legacy regex matching, TODO DEPRECATE */
@@ -118,7 +118,7 @@ static bool access_CheckResource(const struct resource_acl *acl,
         /* === Legacy regex matching, slow, TODO DEPRECATE === */
         if (pos == (size_t) -1)
         {
-            for (int i = 0; i < StrList_Len(acl->admit.hostnames); i++)
+            for (size_t i = 0; i < StrList_Len(acl->admit.hostnames); i++)
             {
                 if (StringMatchFull(StrList_At(acl->admit.hostnames, i),
                                     hostname))
@@ -138,6 +138,10 @@ static bool access_CheckResource(const struct resource_acl *acl,
             access = true;
             have_match = true;
         }
+        else
+        {
+            Log(LOG_LEVEL_VERBOSE, "Hostname '%s' not admitted", hostname);
+        }
     }
     if (!access && !NULL_OR_EMPTY(key) &&
         acl->admit.keys != NULL)
@@ -190,7 +194,7 @@ static bool access_CheckResource(const struct resource_acl *acl,
         acl->deny.ips != NULL)
     {
         const char *rule = NULL;
-        for (int i = 0; i < StrList_Len(acl->deny.ips); i++)
+        for (size_t i = 0; i < StrList_Len(acl->deny.ips); i++)
         {
             if (FuzzySetMatch(StrList_At(acl->deny.ips, i), ipaddr) == 0 ||
                 /* Legacy regex matching, TODO DEPRECATE */
@@ -221,7 +225,7 @@ static bool access_CheckResource(const struct resource_acl *acl,
         /* === Legacy regex matching, slow, TODO DEPRECATE === */
         if (pos == (size_t) -1)
         {
-            for (int i = 0; i < StrList_Len(acl->deny.hostnames); i++)
+            for (size_t i = 0; i < StrList_Len(acl->deny.hostnames); i++)
             {
                 if (StringMatchFull(StrList_At(acl->deny.hostnames, i),
                                     hostname))
