@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -53,6 +53,7 @@ static const int CF_NOSIZE = -1;
 #include <cf-windows-functions.h>                  /* NovaWin_UserNameToSid */
 #include <mutex.h>                                 /* ThreadLock */
 #include <stat_cache.h>                            /* struct Stat */
+#include <unix.h>                                  /* GetUserID() */
 #include "server_access.h"
 
 
@@ -403,13 +404,13 @@ void CfGetFile(ServerFileGetState *args)
 {
     int fd;
     off_t n_read, total = 0, sendlen = 0, count = 0;
-    char sendbuffer[CF_BUFSIZE + 256], filename[CF_BUFSIZE];
+    char sendbuffer[CF_BUFSIZE + 256], filename[CF_BUFSIZE - 128];
     struct stat sb;
     int blocksize = 2048;
 
     ConnectionInfo *conn_info = args->conn->conn_info;
 
-    TranslatePath(filename, args->replyfile);
+    TranslatePath(args->replyfile, filename, sizeof(filename));
 
     stat(filename, &sb);
 
@@ -567,18 +568,19 @@ void CfEncryptGetFile(ServerFileGetState *args)
 {
     int fd, n_read, cipherlen = 0, finlen = 0;
     off_t total = 0, count = 0;
-    char sendbuffer[CF_BUFSIZE + 256], out[CF_BUFSIZE], filename[CF_BUFSIZE];
+    char filename[CF_BUFSIZE];
+    unsigned char sendbuffer[CF_BUFSIZE + 256];
+    unsigned char out[CF_BUFSIZE];
     unsigned char iv[32] =
         { 1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8, 1, 2, 3, 4, 5, 6, 7, 8 };
     int blocksize = CF_BUFSIZE - 4 * CF_INBAND_OFFSET;
-    char *key, enctype;
     struct stat sb;
     ConnectionInfo *conn_info = args->conn->conn_info;
 
-    key = args->conn->session_key;
-    enctype = args->conn->encryption_type;
+    const unsigned char *const key = args->conn->session_key;
+    const char enctype = args->conn->encryption_type;
 
-    TranslatePath(filename, args->replyfile);
+    TranslatePath(args->replyfile, filename, sizeof(filename));
 
     stat(filename, &sb);
 
@@ -671,7 +673,7 @@ void CfEncryptGetFile(ServerFileGetState *args)
 
             if (total >= savedlen)
             {
-                if (SendTransaction(conn_info, out, cipherlen + finlen, CF_DONE) == -1)
+                if (SendTransaction(conn_info, (const char *) out, cipherlen + finlen, CF_DONE) == -1)
                 {
                     Log(LOG_LEVEL_VERBOSE, "Send failed in GetFile. (send: %s)", GetErrorStr());
                     EVP_CIPHER_CTX_free(ctx);
@@ -682,7 +684,7 @@ void CfEncryptGetFile(ServerFileGetState *args)
             }
             else
             {
-                if (SendTransaction(conn_info, out, cipherlen + finlen, CF_MORE) == -1)
+                if (SendTransaction(conn_info, (const char *) out, cipherlen + finlen, CF_MORE) == -1)
                 {
                     Log(LOG_LEVEL_VERBOSE, "Send failed in GetFile. (send: %s)", GetErrorStr());
                     close(fd);
@@ -704,16 +706,16 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
 {
     Stat cfst;
     struct stat statbuf, statlinkbuf;
-    char linkbuf[CF_BUFSIZE], filename[CF_BUFSIZE];
+    char linkbuf[CF_BUFSIZE], filename[CF_BUFSIZE - 128];
     int islink = false;
 
-    TranslatePath(filename, ofilename);
+    TranslatePath(ofilename, filename, sizeof(filename));
 
     memset(&cfst, 0, sizeof(Stat));
 
     if (strlen(ReadLastNode(filename)) > CF_MAXLINKSIZE)
     {
-        snprintf(sendbuffer, CF_BUFSIZE, "BAD: Filename suspiciously long [%s]", filename);
+        snprintf(sendbuffer, CF_MSGSIZE, "BAD: Filename suspiciously long [%s]", filename);
         Log(LOG_LEVEL_ERR, "%s", sendbuffer);
         SendTransaction(conn->conn_info, sendbuffer, 0, CF_DONE);
         return -1;
@@ -721,7 +723,7 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
 
     if (lstat(filename, &statbuf) == -1)
     {
-        snprintf(sendbuffer, CF_BUFSIZE, "BAD: unable to stat file %s", filename);
+        snprintf(sendbuffer, CF_MSGSIZE, "BAD: unable to stat file %s", filename);
         Log(LOG_LEVEL_VERBOSE, "%s. (lstat: %s)", sendbuffer, GetErrorStr());
         SendTransaction(conn->conn_info, sendbuffer, 0, CF_DONE);
         return -1;
@@ -823,7 +825,7 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
         cfst.cf_makeholes = 0;
     }
 
-    memset(sendbuffer, 0, CF_BUFSIZE);
+    memset(sendbuffer, 0, CF_MSGSIZE);
 
     /* send as plain text */
 
@@ -833,7 +835,7 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
         (uintmax_t) cfst.cf_uid, (uintmax_t) cfst.cf_gid, (intmax_t) cfst.cf_size,
         (intmax_t) cfst.cf_atime, (intmax_t) cfst.cf_mtime);
 
-    snprintf(sendbuffer, CF_BUFSIZE,
+    snprintf(sendbuffer, CF_MSGSIZE,
              "OK: %d %ju %ju %ju %ju %jd %jd %jd %jd %d %d %d %jd",
              cfst.cf_type, (uintmax_t) cfst.cf_mode, (uintmax_t) cfst.cf_lmode,
              (uintmax_t) cfst.cf_uid, (uintmax_t) cfst.cf_gid,   (intmax_t) cfst.cf_size,
@@ -842,7 +844,7 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
 
     SendTransaction(conn->conn_info, sendbuffer, 0, CF_DONE);
 
-    memset(sendbuffer, 0, CF_BUFSIZE);
+    memset(sendbuffer, 0, CF_MSGSIZE);
 
     if (cfst.cf_readlink != NULL)
     {
@@ -858,13 +860,13 @@ int StatFile(ServerConnectionState *conn, char *sendbuffer, char *ofilename)
     return 0;
 }
 
-bool CompareLocalHash(const char *filename, const char digest[EVP_MAX_MD_SIZE + 1],
+bool CompareLocalHash(const char *filename, const unsigned char digest[EVP_MAX_MD_SIZE + 1],
                       char sendbuffer[CFD_FALSE_SIZE])
 {
     nt_static_assert(CFD_FALSE_SIZE == (strlen(CFD_FALSE) + 1));
     nt_static_assert(strlen(CFD_FALSE) >= strlen(CFD_TRUE));
     char translated_filename[CF_BUFSIZE] = { 0 };
-    TranslatePath(translated_filename, filename);
+    TranslatePath(filename, translated_filename, sizeof(translated_filename));
 
     unsigned char file_digest[EVP_MAX_MD_SIZE + 1] = { 0 };
     /* TODO connection might timeout if this takes long! */
@@ -894,12 +896,12 @@ void GetServerLiteral(EvalContext *ctx, ServerConnectionState *conn, char *sendb
     if (ReturnLiteralData(ctx, handle, out))
     {
         memset(sendbuffer, 0, CF_BUFSIZE);
-        snprintf(sendbuffer, CF_BUFSIZE - 1, "%s", out);
+        snprintf(sendbuffer, CF_BUFSIZE, "%s", out);
     }
     else
     {
         memset(sendbuffer, 0, CF_BUFSIZE);
-        snprintf(sendbuffer, CF_BUFSIZE - 1, "BAD: Not found");
+        snprintf(sendbuffer, CF_BUFSIZE, "BAD: Not found");
     }
 
     if (encrypted)
@@ -962,9 +964,9 @@ int CfOpenDirectory(ServerConnectionState *conn, char *sendbuffer, char *oldDirn
     Dir *dirh;
     const struct dirent *dirp;
     int offset;
-    char dirname[CF_BUFSIZE];
+    char dirname[CF_BUFSIZE - 128];
 
-    TranslatePath(dirname, oldDirname);
+    TranslatePath(oldDirname, dirname, sizeof(dirname));
 
     if (!IsAbsoluteFileName(dirname))
     {
@@ -1449,17 +1451,12 @@ void SetConnIdentity(ServerConnectionState *conn, const char *username)
     if (conn->uid == CF_UNKNOWN_OWNER)      /* skip looking up UID for root */
     {
         static pthread_mutex_t pwnam_mtx = PTHREAD_ERRORCHECK_MUTEX_INITIALIZER_NP;
-        struct passwd *pw = NULL;
 
         ThreadLock(&pwnam_mtx);
         /* TODO Redmine#7643: looking up the UID is expensive and should
          * not be needed, since today's agent machine VS hub most probably
          * do not share the accounts. */
-        pw = getpwnam(conn->username);
-        if (pw != NULL)
-        {
-            conn->uid = pw->pw_uid;
-        }
+        GetUserID(conn->username, &(conn->uid), LOG_LEVEL_VERBOSE);
         ThreadUnlock(&pwnam_mtx);
     }
 
@@ -1645,7 +1642,7 @@ bool DoExec2(const EvalContext *ctx,
     char   cmdbuf[CF_BUFSIZE] = "";
     size_t cmdbuf_len         = 0;
 
-    assert(sizeof(CFRUNCOMMAND) <= sizeof(cmdbuf));
+    nt_static_assert(sizeof(CFRUNCOMMAND) <= sizeof(cmdbuf));
 
     StrCat(cmdbuf, sizeof(cmdbuf), &cmdbuf_len, CFRUNCOMMAND, 0);
 
