@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -370,8 +370,8 @@ bool ServerIdentificationDialog(ConnectionInfo *conn_info,
     username[0] = '\0';
 
     /* Assert sscanf() is safe to use. */
-    assert(sizeof(word1) >= sizeof(input));
-    assert(sizeof(word2) >= sizeof(input));
+    nt_static_assert(sizeof(word1) >= sizeof(input));
+    nt_static_assert(sizeof(word2) >= sizeof(input));
 
     ret = sscanf(line2, "IDENTITY %[^=]=%s%n", word1, word2, &chars_read);
     while (ret >= 2)
@@ -421,7 +421,15 @@ bool ServerSendWelcome(const ServerConnectionState *conn)
     {
         ret = snprintf(&s[len], sizeof(s) - len, " %s=%s",
                            "USERNAME", conn->username);
-        if (ret >= sizeof(s) - len)
+        if (ret < 0)
+        {
+            Log(LOG_LEVEL_ERR, 
+                "Unexpected failure from snprintf (%d - %s) while "
+                "constructing OK WELCOME message (ServerSendWelcome)", 
+                errno, GetErrorStr());
+            return false;
+        }
+        else if ((size_t) ret >= sizeof(s) - len)
         {
             Log(LOG_LEVEL_NOTICE, "Sending OK WELCOME message truncated: %s", s);
             return false;
@@ -963,7 +971,7 @@ bool BusyWithNewProtocol(EvalContext *ctx, ServerConnectionState *conn)
         assert(CF_DEFAULT_DIGEST_LEN <= EVP_MAX_MD_SIZE);
         unsigned char digest[EVP_MAX_MD_SIZE + 1];
 
-        assert(CF_BUFSIZE + CF_SMALL_OFFSET + CF_DEFAULT_DIGEST_LEN
+        assert(CF_BUFSIZE + CF_SMALL_OFFSET + (size_t) CF_DEFAULT_DIGEST_LEN
                <= sizeof(recvbuffer));
         memcpy(digest, recvbuffer + strlen(recvbuffer) + CF_SMALL_OFFSET,
                CF_DEFAULT_DIGEST_LEN);
