@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -51,6 +51,7 @@
 #include <net.h>
 #include <client_code.h>
 #include <cfnet.h>
+#include <known_dirs.h> // GetWorkDir()
 
 #include "server_common.h"                         /* PreprocessRequestPath */
 #include "server_access.h"
@@ -546,6 +547,10 @@ static void KeepControlPromises(EvalContext *ctx, const Policy *policy, GenericA
         }
     }
 
+    if (config->agent_type == AGENT_TYPE_SERVER)
+    {
+        EvalContextUpdateDumpReports(ctx);
+    }
 }
 
 /*********************************************************************/
@@ -591,7 +596,7 @@ static void EvaluateBundle(EvalContext *ctx, const Bundle *bp, const char * cons
 
     for (int type = 0; seq[type] != NULL; type++)
     {
-        const PromiseType *sp = BundleGetPromiseType((Bundle *)bp, seq[type]);
+        const BundleSection *sp = BundleGetSection((Bundle *)bp, seq[type]);
 
         /* Some promise types might not be there. */
         if (!sp || SeqLength(sp->promises) == 0)
@@ -601,7 +606,7 @@ static void EvaluateBundle(EvalContext *ctx, const Bundle *bp, const char * cons
             continue;
         }
 
-        EvalContextStackPushPromiseTypeFrame(ctx, sp);
+        EvalContextStackPushBundleSectionFrame(ctx, sp);
         for (size_t ppi = 0; ppi < SeqLength(sp->promises); ppi++)
         {
             Promise *pp = SeqAt(sp->promises, ppi);
@@ -612,23 +617,23 @@ static void EvaluateBundle(EvalContext *ctx, const Bundle *bp, const char * cons
 
     /* Check if we are having some other promise types which we
      * should evaluate. THIS IS ONLY FOR BACKWARD COMPATIBILITY! */
-    for (size_t j = 0; j < SeqLength(bp->promise_types); j++)
+    for (size_t j = 0; j < SeqLength(bp->sections); j++)
     {
-        PromiseType *sp = SeqAt(bp->promise_types, j);
+        BundleSection *sp = SeqAt(bp->sections, j);
 
         /* Skipping evaluation of promise as this was evaluated in
          * loop above. */
-        if (!IsPromiseTypeNotInTypeSequence(sp->name, seq))
+        if (!IsPromiseTypeNotInTypeSequence(sp->promise_type, seq))
         {
             Log(LOG_LEVEL_DEBUG, "Skipping subsequent evaluation of "
-                    "promise type %s in bundle %s", sp->name, bp->name);
+                    "promise type %s in bundle %s", sp->promise_type, bp->name);
             continue;
         }
 
         Log(LOG_LEVEL_WARNING, "Trying to evaluate unsupported/obsolete "
-                    "promise type %s in %s bundle %s", sp->name, bp->type, bp->name);
+                    "promise type %s in %s bundle %s", sp->promise_type, bp->type, bp->name);
 
-        EvalContextStackPushPromiseTypeFrame(ctx, sp);
+        EvalContextStackPushBundleSectionFrame(ctx, sp);
         for (size_t ppi = 0; ppi < SeqLength(sp->promises); ppi++)
         {
             Promise *pp = SeqAt(sp->promises, ppi);
@@ -681,12 +686,12 @@ static PromiseResult KeepServerPromise(EvalContext *ctx, const Promise *pp, ARG_
     assert(!param);
     PromiseBanner(ctx, pp);
 
-    if (strcmp(pp->parent_promise_type->name, "vars") == 0)
+    if (strcmp(PromiseGetPromiseType(pp), "vars") == 0)
     {
         return VerifyVarPromise(ctx, pp, NULL);
     }
 
-    if (strcmp(pp->parent_promise_type->name, "classes") == 0)
+    if (strcmp(PromiseGetPromiseType(pp), "classes") == 0)
     {
         return VerifyClassPromise(ctx, pp, NULL);
     }
@@ -700,8 +705,8 @@ static PromiseResult KeepServerPromise(EvalContext *ctx, const Promise *pp, ARG_
         Log(LOG_LEVEL_WARNING,
             "ifelapsed attribute specified in action body for %s promise '%s',"
             " but %s promises do not support promise locking",
-            pp->parent_promise_type->name, pp->promiser,
-            pp->parent_promise_type->name);
+            PromiseGetPromiseType(pp), pp->promiser,
+            PromiseGetPromiseType(pp));
     }
     int expireafter = PromiseGetConstraintAsInt(ctx, "expireafter", pp);
     if (expireafter != CF_NOINT)
@@ -709,11 +714,11 @@ static PromiseResult KeepServerPromise(EvalContext *ctx, const Promise *pp, ARG_
         Log(LOG_LEVEL_WARNING,
             "expireafter attribute specified in action body for %s promise '%s',"
             " but %s promises do not support promise locking",
-            pp->parent_promise_type->name, pp->promiser,
-            pp->parent_promise_type->name);
+            PromiseGetPromiseType(pp), pp->promiser,
+            PromiseGetPromiseType(pp));
     }
 
-    if (strcmp(pp->parent_promise_type->name, "access") == 0)
+    if (strcmp(PromiseGetPromiseType(pp), "access") == 0)
     {
         const char *resource_type =
             PromiseGetConstraintAsRval(pp, "resource_type", RVAL_TYPE_SCALAR);
@@ -752,7 +757,7 @@ static PromiseResult KeepServerPromise(EvalContext *ctx, const Promise *pp, ARG_
             return PROMISE_RESULT_NOOP;
         }
     }
-    else if (strcmp(pp->parent_promise_type->name, "roles") == 0)
+    else if (strcmp(PromiseGetPromiseType(pp), "roles") == 0)
     {
         KeepServerRolePromise(ctx, pp);
         return PROMISE_RESULT_NOOP;
@@ -988,7 +993,7 @@ static void NewHostToOldACL(Auth *old, const char *host)
 
             /* Identify next fragment: */
             const char *d = strchr(dot, '.');
-            size_t len = d ? d - dot : strlen(dot);
+            size_t len = d ? (size_t) (d - dot) : strlen(dot);
 
             /* Copy fragment: */
             memcpy(dst, dot, len);
