@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -79,7 +79,7 @@ char CFENGINE_PORT_STR[16] = "5308";
 
 void DetermineCfenginePort()
 {
-    assert(sizeof(CFENGINE_PORT_STR) >= PRINTSIZE(CFENGINE_PORT));
+    nt_static_assert(sizeof(CFENGINE_PORT_STR) >= PRINTSIZE(CFENGINE_PORT));
     /* ... but we leave more space, as service names can be longer */
 
     errno = 0;
@@ -197,7 +197,7 @@ AgentConnection *ServerConnection(const char *server, const char *port,
 
     conn = NewAgentConn(server, port, flags);
 
-#if !defined(__MINGW32__)
+#if !defined(__MINGW32__) && !defined(__ANDROID__)
     signal(SIGPIPE, SIG_IGN);
 
     sigset_t signal_mask;
@@ -208,7 +208,7 @@ AgentConnection *ServerConnection(const char *server, const char *port,
     /* FIXME: username is local */
     GetCurrentUserName(conn->username, sizeof(conn->username));
 #else
-    /* Always say "root" as username from windows. */
+    /* Always say "root" if windows or termux. */
     strlcpy(conn->username, "root", sizeof(conn->username));
 #endif
 
@@ -230,7 +230,7 @@ AgentConnection *ServerConnection(const char *server, const char *port,
         return NULL;
     }
 
-    assert(sizeof(conn->remoteip) >= sizeof(txtaddr));
+    nt_static_assert(sizeof(conn->remoteip) >= sizeof(txtaddr));
     strcpy(conn->remoteip, txtaddr);
 
     ProtocolVersion protocol_version = flags.protocol_version;
@@ -668,7 +668,7 @@ static bool EncryptCopyRegularFileNet(const char *source, const char *dest, off_
 
         EVP_DecryptInit_ex(crypto_ctx, CfengineCipher(CfEnterpriseOptions()), NULL, conn->session_key, iv);
 
-        if (!EVP_DecryptUpdate(crypto_ctx, workbuf, &plainlen, buf, cipherlen))
+        if (!EVP_DecryptUpdate(crypto_ctx, (unsigned char *) workbuf, &plainlen, (unsigned char *) buf, cipherlen))
         {
             close(dd);
             free(buf);
@@ -676,7 +676,7 @@ static bool EncryptCopyRegularFileNet(const char *source, const char *dest, off_
             return false;
         }
 
-        if (!EVP_DecryptFinal_ex(crypto_ctx, workbuf + plainlen, &finlen))
+        if (!EVP_DecryptFinal_ex(crypto_ctx, (unsigned char *) workbuf + plainlen, &finlen))
         {
             close(dd);
             free(buf);
@@ -795,8 +795,8 @@ bool CopyRegularFileNet(const char *source, const char *dest, off_t size,
     Log(LOG_LEVEL_VERBOSE, "Copying remote file '%s:%s', expecting %jd bytes",
           conn->this_server, source, (intmax_t)size);
 
-    size_t n_wrote_total        = 0;
-    bool   last_write_made_hole = false;
+    int n_wrote_total = 0;
+    bool last_write_made_hole = false;
     while (n_wrote_total < size)
     {
         int toget = MIN(size - n_wrote_total, buf_size);
