@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -490,7 +490,7 @@ X509 *TLSGenerateCertFromPrivKey(RSA *privkey)
 
     ret = 0;
     ret += X509_NAME_add_entry_by_txt(name, "CN", MBSTRING_ASC,
-                                      (const char *) "a",
+                                      (const unsigned char *) "a",
                                       -1, -1, 0);
     ret += X509_set_issuer_name(x509, name);
     ret += X509_set_pubkey(x509, pkey);
@@ -1021,22 +1021,8 @@ bool TLSSetCipherList(SSL_CTX *ssl_ctx, const char *cipher_list)
         }
     }
 
-    /* The above code can leave a trailing colon in the lists because it copies
-     * the items over *with* the colons splitting them. */
-    if ((ciphers_len > 0) && (ciphers[ciphers_len - 1] == ':'))
-    {
-        ciphers[ciphers_len - 1] = '\0';
-        ciphers_len--;
-    }
-    if ((cipher_suites_len > 0) && (cipher_suites[cipher_suites_len - 1] == ':'))
-    {
-        cipher_suites[cipher_suites_len - 1] = '\0';
-        cipher_suites_len--;
-    }
-
     if (ciphers_len != 0)       /* TLS <= 1.2 ciphers */
     {
-        Log(LOG_LEVEL_VERBOSE, "Enabling ciphers '%s' for TLS 1.2 and older", ciphers);
         int ret = SSL_CTX_set_cipher_list(ssl_ctx, ciphers);
         if (ret != 1)
         {
@@ -1048,7 +1034,6 @@ bool TLSSetCipherList(SSL_CTX *ssl_ctx, const char *cipher_list)
 #ifdef HAVE_TLS_1_3
     if (cipher_suites_len != 0) /* TLS >= 1.3 ciphers */
     {
-        Log(LOG_LEVEL_VERBOSE, "Enabling cipher suites '%s' for TLS 1.3 and newer", cipher_suites);
         int ret = SSL_CTX_set_ciphersuites(ssl_ctx, cipher_suites);
         if (ret != 1)
         {
