@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -89,6 +89,7 @@ static void InitIgnoreInterfaces(void);
 static Rlist *IGNORE_INTERFACES = NULL; /* GLOBAL_E */
 
 typedef void (*ProcPostProcessFn)(void *ctx, void *json);
+typedef JsonElement * (*ProcTiebreakerFn)(JsonElement *prev_item, JsonElement *this_item);
 
 
 /*********************************************************************/
@@ -148,7 +149,7 @@ static void GetMacAddress(EvalContext *ctx, ARG_UNUSED int fd, struct ifreq *ifr
       return;
     }
 
-# if defined(SIOCGIFHWADDR) && defined(HAVE_STRUCT_IFREQ_IFR_HWADDR)
+# if defined(SIOCGIFHWADDR) && defined(HAVE_STRUCT_IFREQ_IFR_HWADDR) && !defined(__ANDROID__)
     char hw_mac[CF_MAXVARSIZE];
 
     if ((ioctl(fd, SIOCGIFHWADDR, ifr) == -1))
@@ -180,9 +181,8 @@ static void GetMacAddress(EvalContext *ctx, ARG_UNUSED int fd, struct ifreq *ifr
 
 # elif defined(HAVE_GETIFADDRS) && !defined(__sun)
     char hw_mac[CF_MAXVARSIZE];
-    char *m;
+    char *mac_pointer = NULL;
     struct ifaddrs *ifaddr, *ifa;
-    struct sockaddr_dl *sdl;
 
     if (getifaddrs(&ifaddr) == -1)
     {
@@ -197,29 +197,49 @@ static void GetMacAddress(EvalContext *ctx, ARG_UNUSED int fd, struct ifreq *ifr
     {
         if ( strcmp(ifa->ifa_name, ifp->ifr_name) == 0)
         {
+            if (ifa->ifa_addr == NULL)
+            {
+               Log(LOG_LEVEL_VERBOSE, "Interface '%s' has no address information", ifa->ifa_name);
+               continue;
+            }
+#if AF_LINK
             if (ifa->ifa_addr->sa_family == AF_LINK)
             {
-                sdl = (struct sockaddr_dl *)ifa->ifa_addr;
-                m = (char *) LLADDR(sdl);
-
-                snprintf(hw_mac, sizeof(hw_mac), "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x",
-                    (unsigned char) m[0],
-                    (unsigned char) m[1],
-                    (unsigned char) m[2],
-                    (unsigned char) m[3],
-                    (unsigned char) m[4],
-                    (unsigned char) m[5]);
-
-                EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, name, hw_mac, CF_DATA_TYPE_STRING, "source=agent");
-                if (!RlistContainsString(*hardware, hw_mac))
-                {
-                    RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
-                }
-                RlistAppend(interfaces, ifa->ifa_name, RVAL_TYPE_SCALAR);
-
-                snprintf(name, sizeof(name), "mac_%s", CanonifyName(hw_mac));
-                EvalContextClassPutHard(ctx, name, "source=agent");
+                struct sockaddr_dl *sdl = (struct sockaddr_dl *)ifa->ifa_addr;
+                mac_pointer = (char *) LLADDR(sdl);
             }
+#elif AF_PACKET
+            if (ifa->ifa_addr->sa_family == AF_PACKET)
+            {
+                struct sockaddr_ll *sll = (struct sockaddr_ll *)ifa->ifa_addr;
+                mac_pointer = (char *) sll->sll_addr;
+            }
+#else
+# error "AF_LINK or AF_PACKET must be available for GetMacAddress() currently"
+#endif
+            if (mac_pointer == NULL)
+            {
+                Log(LOG_LEVEL_VERBOSE, "Couldn't get Physical-layer address from interface '%s'", ifr->ifr_name);
+                continue;
+            }
+
+            snprintf(hw_mac, sizeof(hw_mac), "%.2x:%.2x:%.2x:%.2x:%.2x:%.2x",
+                (unsigned char) mac_pointer[0],
+                (unsigned char) mac_pointer[1],
+                (unsigned char) mac_pointer[2],
+                (unsigned char) mac_pointer[3],
+                (unsigned char) mac_pointer[4],
+                (unsigned char) mac_pointer[5]);
+
+            EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, name, hw_mac, CF_DATA_TYPE_STRING, "source=agent");
+            if (!RlistContainsString(*hardware, hw_mac))
+            {
+                RlistAppend(hardware, hw_mac, RVAL_TYPE_SCALAR);
+            }
+            RlistAppend(interfaces, ifa->ifa_name, RVAL_TYPE_SCALAR);
+
+            snprintf(name, sizeof(name), "mac_%s", CanonifyName(hw_mac));
+            EvalContextClassPutHard(ctx, name, "source=agent");
         }
 
     }
@@ -471,7 +491,7 @@ void GetInterfacesInfo(EvalContext *ctx)
 
                 /* No DNS lookup, just convert IP address to string. */
                 char txtaddr[CF_MAX_IP_LEN] = "";
-                assert(sizeof(VIPADDRESS) >= sizeof(txtaddr));
+                nt_static_assert(sizeof(VIPADDRESS) >= sizeof(txtaddr));
 
                 getnameinfo((struct sockaddr *) sin, sizeof(*sin),
                             txtaddr, sizeof(txtaddr),
@@ -485,7 +505,7 @@ void GetInterfacesInfo(EvalContext *ctx)
                     /* TODO remove, interface address can't be 0.0.0.0 and
                      * even then DNS is not a safe way to set a variable... */
                     Log(LOG_LEVEL_VERBOSE, "Cannot discover hardware IP, using DNS value");
-                    assert(sizeof(ip) >= sizeof(VIPADDRESS) + sizeof("ipv4_"));
+                    nt_static_assert(sizeof(ip) >= sizeof(VIPADDRESS) + sizeof("ipv4_"));
                     strcpy(ip, "ipv4_");
                     strcat(ip, VIPADDRESS);
                     EvalContextAddIpAddress(ctx, VIPADDRESS, NULL); // we don't know the interface
@@ -515,7 +535,7 @@ void GetInterfacesInfo(EvalContext *ctx)
                     continue;
                 }
 
-                assert(sizeof(ip) >= sizeof(txtaddr) + sizeof("ipv4_"));
+                nt_static_assert(sizeof(ip) >= sizeof(txtaddr) + sizeof("ipv4_"));
                 strcpy(ip, "ipv4_");
                 strcat(ip, txtaddr);
                 EvalContextClassPutHard(ctx, ip, "inventory,attribute_name=none,source=agent");
@@ -1022,7 +1042,7 @@ static void NetworkingRoutesPostProcessInfo(
 # endif
 }
 
-static ProcPostProcessFn NetworkingIPv6RoutesPostProcessInfo(
+static void NetworkingIPv6RoutesPostProcessInfo(
     ARG_UNUSED void *passed_ctx, ARG_LINUX_ONLY void *json)
 {
 # if defined (__linux__)
@@ -1055,10 +1075,9 @@ static ProcPostProcessFn NetworkingIPv6RoutesPostProcessInfo(
     // like we do with IPv4 routes
 
 # endif
-    return NULL;
 }
 
-static ProcPostProcessFn NetworkingIPv6AddressesPostProcessInfo(ARG_UNUSED void *passed_ctx, void *json)
+static void NetworkingIPv6AddressesPostProcessInfo(ARG_UNUSED void *passed_ctx, void *json)
 {
     JsonElement *entry = json;
 
@@ -1067,7 +1086,50 @@ static ProcPostProcessFn NetworkingIPv6AddressesPostProcessInfo(ARG_UNUSED void
     JsonExtractParsedNumber(entry, "raw_device_number", "device_number", true, false);
     JsonExtractParsedNumber(entry, "raw_prefix_length", "prefix_length", true, false);
     JsonExtractParsedNumber(entry, "raw_scope", "scope", true, false);
-    return NULL;
+}
+
+static unsigned RankIPv6Address(const char *address)
+{
+    unsigned long first_word = 0;
+    char *end;
+
+    if (address == NULL)
+    {
+        return 0;
+    }
+
+    first_word = strtoul(address, &end, 16);
+
+    if (*end != ':')
+    {
+        return 0;  // invalid IPv6 address?
+    }
+
+    if ((first_word & 0xffc0) == 0xfe80)
+    {
+        // link-local (fe80:://10)
+
+        return 1;
+    }
+    else
+    {
+        return 2;
+    }
+}
+
+static JsonElement *NetworkingIPv6AddressesTiebreaker(JsonElement *prev_item, JsonElement *this_item)
+{
+    const char *prev_addr = JsonObjectGetAsString(prev_item, "address");
+    const char *this_addr = JsonObjectGetAsString(this_item, "address");
+
+    if (RankIPv6Address(this_addr) >= RankIPv6Address(prev_addr))
+    {
+        return this_item;
+    }
+    else
+    {
+        return prev_item;
+    }
 }
 
 /*******************************************************************/
@@ -1096,7 +1158,7 @@ static const char* GetPortStateString(ARG_LINUX_ONLY int state)
 
 // used in evalfunction.c but defined here so
 // JsonRewriteParsedIPAddress() etc. can stay local
-ProcPostProcessFn NetworkingPortsPostProcessInfo(ARG_UNUSED void *passed_ctx, void *json)
+void NetworkingPortsPostProcessInfo(ARG_UNUSED void *passed_ctx, void *json)
 {
     JsonElement *conn = json;
 
@@ -1113,8 +1175,6 @@ ProcPostProcessFn NetworkingPortsPostProcessInfo(ARG_UNUSED void *passed_ctx, vo
             JsonObjectAppendString(conn, "state", GetPortStateString(num_state));
         }
     }
-
-    return NULL;
 }
 
 /*******************************************************************/
@@ -1181,7 +1241,7 @@ static JsonElement* GetNetworkingStatsInfo(const char *filename)
 // always returns the parsed data. If the key is not NULL, also
 // creates a sys.KEY variable.
 
-JsonElement* GetProcFileInfo(EvalContext *ctx, const char* filename, const char* key, const char* extracted_key, ProcPostProcessFn post, const char* regex)
+JsonElement* GetProcFileInfo(EvalContext *ctx, const char* filename, const char* key, const char* extracted_key, ProcPostProcessFn post, ProcTiebreakerFn tiebreak, const char* regex)
 {
     JsonElement *info = NULL;
     bool extract_key_mode = (extracted_key != NULL);
@@ -1219,14 +1279,39 @@ JsonElement* GetProcFileInfo(EvalContext *ctx, const char* filename, const char*
 
                     if (extract_key_mode)
                     {
-                        if (JsonObjectGetAsString(item, extracted_key) == NULL)
+                        const char *extracted_key_value = JsonObjectGetAsString(item, extracted_key);
+
+                        if (extracted_key_value == NULL)
                         {
                             Log(LOG_LEVEL_ERR, "While parsing %s, looked to extract key %s but couldn't find it in line %s", filename, extracted_key, line);
                         }
                         else
                         {
-                            Log(LOG_LEVEL_DEBUG, "While parsing %s, got key %s from line %s", filename, JsonObjectGetAsString(item, extracted_key), line);
-                            JsonObjectAppendElement(info, JsonObjectGetAsString(item, extracted_key), item);
+                            JsonElement *prev_item = JsonObjectGet(info, extracted_key_value);
+
+                            Log(LOG_LEVEL_DEBUG, "While parsing %s, got key %s from line %s", filename, extracted_key_value, line);
+
+                            if (prev_item != NULL && tiebreak != NULL)
+                            {
+                                JsonElement *winner = (*tiebreak)(prev_item, item);
+
+                                if (winner == prev_item)
+                                {
+                                    Log(LOG_LEVEL_DEBUG, "Multiple entries for key %s, preferring previous value", extracted_key_value);
+
+                                    JsonDestroy(item);
+                                    item = NULL;
+                                }
+                                else
+                                {
+                                    Log(LOG_LEVEL_DEBUG, "Multiple entries for key %s, preferring new value", extracted_key_value);
+                                }
+                            }
+
+                            if (item != NULL)
+                            {
+                                JsonObjectAppendElement(info, extracted_key_value, item);
+                            }
                         }
                     }
                     else
@@ -1275,7 +1360,7 @@ void GetNetworkingInfo(EvalContext *ctx)
     }
 
     BufferPrintf(pbuf, "%s/proc/net/route", procdir_root);
-    JsonElement *routes = GetProcFileInfo(ctx, BufferData(pbuf),  NULL, NULL, (ProcPostProcessFn) &NetworkingRoutesPostProcessInfo,
+    JsonElement *routes = GetProcFileInfo(ctx, BufferData(pbuf),  NULL, NULL, &NetworkingRoutesPostProcessInfo, NULL,
                     // format: Iface	Destination	Gateway 	Flags	RefCnt	Use	Metric	Mask		MTU	Window	IRTT
                     //         eth0	00000000	0102A8C0	0003	0	0	1024	00000000	0	0	0
                     "^(?<interface>\\S+)\\t(?<raw_dest>[[:xdigit:]]+)\\t(?<raw_gw>[[:xdigit:]]+)\\t(?<raw_flags>[[:xdigit:]]+)\\t(?<refcnt>\\d+)\\t(?<use>\\d+)\\t(?<metric>[[:xdigit:]]+)\\t(?<raw_mask>[[:xdigit:]]+)\\t(?<mtu>\\d+)\\t(?<window>\\d+)\\t(?<irtt>[[:xdigit:]]+)");
@@ -1323,7 +1408,7 @@ void GetNetworkingInfo(EvalContext *ctx)
     JsonElement *inet6 = JsonObjectCreate(3);
 
     BufferPrintf(pbuf, "%s/proc/net/snmp6", procdir_root);
-    JsonElement *inet6_stats = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, NULL,
+    JsonElement *inet6_stats = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, NULL, NULL,
                                                "^\\s*(?<key>\\S+)\\s+(?<value>\\d+)");
 
     if (inet6_stats != NULL)
@@ -1349,7 +1434,7 @@ void GetNetworkingInfo(EvalContext *ctx)
     }
 
     BufferPrintf(pbuf, "%s/proc/net/ipv6_route", procdir_root);
-    JsonElement *inet6_routes = GetProcFileInfo(ctx, BufferData(pbuf),  NULL, NULL, (ProcPostProcessFn) &NetworkingIPv6RoutesPostProcessInfo,
+    JsonElement *inet6_routes = GetProcFileInfo(ctx, BufferData(pbuf),  NULL, NULL, &NetworkingIPv6RoutesPostProcessInfo, NULL,
                     // format: dest                    dest_prefix source                source_prefix next_hop                         metric   refcnt   use      flags        interface
                     //         fe800000000000000000000000000000 40 00000000000000000000000000000000 00 00000000000000000000000000000000 00000100 00000000 00000000 00000001     eth0
                     "^(?<raw_dest>[[:xdigit:]]+)\\s+(?<dest_prefix>[[:xdigit:]]+)\\s+"
@@ -1364,7 +1449,7 @@ void GetNetworkingInfo(EvalContext *ctx)
     }
 
     BufferPrintf(pbuf, "%s/proc/net/if_inet6", procdir_root);
-    JsonElement *inet6_addresses = GetProcFileInfo(ctx, BufferData(pbuf),  NULL, "interface", (ProcPostProcessFn) &NetworkingIPv6AddressesPostProcessInfo,
+    JsonElement *inet6_addresses = GetProcFileInfo(ctx, BufferData(pbuf),  NULL, "interface", &NetworkingIPv6AddressesPostProcessInfo, &NetworkingIPv6AddressesTiebreaker,
                     // format: address device_number prefix_length scope flags interface_name
                     // 00000000000000000000000000000001 01 80 10 80       lo
                     // fe80000000000000004249fffebdd7b4 04 40 20 80  docker0
@@ -1388,7 +1473,7 @@ void GetNetworkingInfo(EvalContext *ctx)
 
     BufferPrintf(pbuf, "%s/proc/net/dev", procdir_root);
     JsonElement *interfaces_data =
-    GetProcFileInfo(ctx, BufferData(pbuf), "interfaces_data", "device", NULL,
+    GetProcFileInfo(ctx, BufferData(pbuf), "interfaces_data", "device", NULL, NULL,
                     "^\\s*(?<device>[^:]+)\\s*:\\s*"
                     // All of the below are just decimal digits separated by spaces
                     "(?<receive_bytes>\\d+)\\s+"
@@ -1421,32 +1506,33 @@ JsonElement* GetNetworkingConnections(EvalContext *ctx)
     Buffer *pbuf = BufferNew();
 
     BufferPrintf(pbuf, "%s/proc/net/tcp", procdir_root);
-    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, (ProcPostProcessFn) &NetworkingPortsPostProcessInfo, ports_regex);
+    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, &NetworkingPortsPostProcessInfo, NULL, ports_regex);
     if (data != NULL)
     {
         JsonObjectAppendElement(json, "tcp", data);
     }
 
     BufferPrintf(pbuf, "%s/proc/net/tcp6", procdir_root);
-    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, (ProcPostProcessFn) &NetworkingPortsPostProcessInfo, ports_regex);
+    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, &NetworkingPortsPostProcessInfo, NULL, ports_regex);
     if (data != NULL)
     {
         JsonObjectAppendElement(json, "tcp6", data);
     }
 
     BufferPrintf(pbuf, "%s/proc/net/udp", procdir_root);
-    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, (ProcPostProcessFn) &NetworkingPortsPostProcessInfo, ports_regex);
+    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, &NetworkingPortsPostProcessInfo, NULL, ports_regex);
     if (data != NULL)
     {
         JsonObjectAppendElement(json, "udp", data);
     }
 
     BufferPrintf(pbuf, "%s/proc/net/udp6", procdir_root);
-    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, (ProcPostProcessFn) &NetworkingPortsPostProcessInfo, ports_regex);
+    data = GetProcFileInfo(ctx, BufferData(pbuf), NULL, NULL, &NetworkingPortsPostProcessInfo, NULL, ports_regex);
     if (data != NULL)
     {
         JsonObjectAppendElement(json, "udp6", data);
     }
+    BufferDestroy(pbuf);
 
     if (JsonLength(json) < 1)
     {
