@@ -1,5 +1,5 @@
 /*
-  Copyright 2020 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -425,7 +425,7 @@ char *MapName(char *s)
     if (strlcpy(s, ret, MAX_FILENAME) >= MAX_FILENAME)
     {
         FatalError(ctx, "Expanded path (%s) is longer than MAX_FILENAME ("
-                   TOSTRING(MAX_FILENAME) ") characters",
+                   TO_STRING(MAX_FILENAME) ") characters",
                    ret);
     }
     free(ret);
@@ -562,8 +562,15 @@ int safe_open_create_perms(
 #ifdef __MINGW32__
     // Windows gets off easy. No symlinks there.
     return open(pathname, flags, create_perms);
-#else // !__MINGW32__
-
+#elif defined(__ANDROID__)
+    // if effective user is not root then don't try to open
+    // all paths from '/' up, might not have permissions.
+    uid_t p_euid = geteuid();
+    if (p_euid != 0)
+    {
+        return open(pathname, flags, create_perms);
+    }
+#else // !__MINGW32__ and !__ANDROID__
     const size_t path_bufsize = strlen(pathname) + 1;
     char path[path_bufsize];
     const size_t res_len = StringCopy(pathname, path, path_bufsize);
@@ -1483,6 +1490,46 @@ ssize_t CfReadLine(char **buff, size_t *size, FILE *fp)
     return b;
 }
 
+ssize_t CfReadLines(char **buff, size_t *size, FILE *fp, Seq *lines)
+{
+    assert(size != NULL);
+    assert(fp != NULL);
+    assert(lines != NULL);
+    assert((buff != NULL) || *size == 0);
+
+    ssize_t appended = 0;
+
+    ssize_t ret;
+    bool free_buff = (buff == NULL);
+    while (!feof(fp))
+    {
+        assert((buff != NULL) || *size == 0);
+        ret = CfReadLine(buff, size, fp);
+        if (ret == -1)
+        {
+            if (!feof(fp))
+            {
+                if (free_buff)
+                {
+                    free(*buff);
+                }
+                return -1;
+            }
+        }
+        else
+        {
+            SeqAppend(lines, xstrdup(*buff));
+            appended++;
+        }
+    }
+    if (free_buff)
+    {
+        free(*buff);
+    }
+
+    return appended;
+}
+
 StringSet* GlobFileList(const char *pattern)
 {
     StringSet *set = StringSetNew();
@@ -1510,7 +1557,7 @@ StringSet* GlobFileList(const char *pattern)
 
         if (glob(expanded, globflags, NULL, &globbuf) == 0)
         {
-            for (int i = 0; i < globbuf.gl_pathc; i++)
+            for (size_t i = 0; i < globbuf.gl_pathc; i++)
             {
                 StringSetAdd(set, xstrdup(globbuf.gl_pathv[i]));
             }
