@@ -1,5 +1,5 @@
 /*
-  Copyright 2020 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -29,11 +29,15 @@
 
 #include <alloc.h>
 #include <sequence.h>
+#include <stdint.h>
+#include <inttypes.h>
 #include <string_lib.h>
 #include <misc_lib.h>
 #include <file_lib.h>
 #include <printsize.h>
+#ifdef WITH_PCRE
 #include <regex.h>
+#endif
 #include <buffer.h>
 
 static const int SPACES_PER_INDENT = 2;
@@ -742,7 +746,7 @@ bool JsonPrimitiveGetAsBool(const JsonElement *const primitive)
     assert(primitive->type == JSON_ELEMENT_TYPE_PRIMITIVE);
     assert(primitive->primitive.type == JSON_PRIMITIVE_TYPE_BOOL);
 
-    return StringSafeEqual(JSON_TRUE, primitive->primitive.value);
+    return StringEqual(JSON_TRUE, primitive->primitive.value);
 }
 
 long JsonPrimitiveGetAsInteger(const JsonElement *const primitive)
@@ -754,6 +758,34 @@ long JsonPrimitiveGetAsInteger(const JsonElement *const primitive)
     return StringToLongExitOnError(primitive->primitive.value);
 }
 
+
+int JsonPrimitiveGetAsInt64(const JsonElement *primitive, int64_t *value_out)
+{
+    assert(primitive != NULL);
+    assert(primitive->type == JSON_ELEMENT_TYPE_PRIMITIVE);
+    assert(primitive->primitive.type == JSON_PRIMITIVE_TYPE_INTEGER);
+
+    return StringToInt64(primitive->primitive.value, value_out);
+}
+
+int64_t JsonPrimitiveGetAsInt64DefaultOnError(const JsonElement *primitive, int64_t default_return)
+{
+    assert(primitive != NULL);
+    assert(primitive->type == JSON_ELEMENT_TYPE_PRIMITIVE);
+    assert(primitive->primitive.type == JSON_PRIMITIVE_TYPE_INTEGER);
+
+    return StringToInt64DefaultOnError(primitive->primitive.value, default_return);
+}
+
+int64_t JsonPrimitiveGetAsInt64ExitOnError(const JsonElement *primitive)
+{
+    assert(primitive != NULL);
+    assert(primitive->type == JSON_ELEMENT_TYPE_PRIMITIVE);
+    assert(primitive->primitive.type == JSON_PRIMITIVE_TYPE_INTEGER);
+
+    return StringToInt64ExitOnError(primitive->primitive.value);
+}
+
 double JsonPrimitiveGetAsReal(const JsonElement *const primitive)
 {
     assert(primitive != NULL);
@@ -853,7 +885,7 @@ JsonElement *JsonObjectCreate(const size_t initialCapacity)
         JSON_CONTAINER_TYPE_OBJECT, NULL, initialCapacity);
 }
 
-static void JsonEncodeStringWriter(
+void JsonEncodeStringWriter(
     const char *const unescaped_string, Writer *const writer)
 {
     assert(unescaped_string != NULL);
@@ -909,9 +941,8 @@ static void JsonDecodeStringWriter(
 
     for (const char *c = escaped_string; *c != '\0'; c++)
     {
-        switch (*c)
+        if (*c == '\\')
         {
-        case '\\':
             switch (c[1])
             {
             case '\"':
@@ -943,10 +974,10 @@ static void JsonDecodeStringWriter(
                 WriterWriteChar(w, *c);
                 break;
             }
-            break;
-        default:
+        }
+        else
+        {
             WriterWriteChar(w, *c);
-            break;
         }
     }
 }
@@ -1037,6 +1068,13 @@ void JsonObjectAppendInteger(
     JsonObjectAppendElement(object, key, child);
 }
 
+void JsonObjectAppendInteger64(
+    JsonElement *const object, const char *const key, const int64_t value)
+{
+    JsonElement *child = JsonIntegerCreate64(value);
+    JsonObjectAppendElement(object, key, child);
+}
+
 void JsonObjectAppendBool(
     JsonElement *const object, const char *const key, const _Bool value)
 {
@@ -1192,6 +1230,24 @@ const char *JsonObjectGetAsString(
     return NULL;
 }
 
+bool JsonObjectGetAsBool(
+    const JsonElement *const object, const char *key)
+{
+    assert(object != NULL);
+    assert(JsonGetType(object) == JSON_TYPE_OBJECT);
+    assert(key != NULL);
+
+    JsonElement *childPrimitive = JsonObjectGet(object, key);
+
+    if (childPrimitive != NULL)
+    {
+        assert(JsonGetType(childPrimitive) == JSON_TYPE_BOOL);
+        return StringEqual(childPrimitive->primitive.value, "true");
+    }
+
+    return false;
+}
+
 JsonElement *JsonObjectGetAsObject(
     JsonElement *const object, const char *const key)
 {
@@ -1315,6 +1371,24 @@ void JsonArrayAppendElement(
     SeqAppend(array->container.children, element);
 }
 
+void JsonArrayExtend(JsonElement *a, JsonElement *b)
+{
+    assert(a != NULL);
+    assert(a->type == JSON_ELEMENT_TYPE_CONTAINER);
+    assert(a->container.type == JSON_CONTAINER_TYPE_ARRAY);
+    assert(b != NULL);
+    assert(b->type == JSON_ELEMENT_TYPE_CONTAINER);
+    assert(b->container.type == JSON_CONTAINER_TYPE_ARRAY);
+
+    SeqAppendSeq(a->container.children, b->container.children);
+    SeqSoftDestroy(b->container.children);
+    if (b->propertyName != NULL)
+    {
+        free(b->propertyName);
+    }
+    free(b);
+}
+
 void JsonArrayRemoveRange(
     JsonElement *const array, const size_t start, const size_t end)
 {
@@ -1423,6 +1497,14 @@ JsonElement *JsonIntegerCreate(const int value)
     return JsonElementCreatePrimitive(JSON_PRIMITIVE_TYPE_INTEGER, buffer);
 }
 
+JsonElement *JsonIntegerCreate64(const int64_t value)
+{
+    char *buffer;
+    xasprintf(&buffer, "%" PRIi64, value);
+
+    return JsonElementCreatePrimitive(JSON_PRIMITIVE_TYPE_INTEGER, buffer);
+}
+
 JsonElement *JsonRealCreate(double value)
 {
     if (isnan(value) || !isfinite(value))
@@ -1492,21 +1574,19 @@ static void JsonPrimitiveWrite(
 
     const char *const value = primitiveElement->primitive.value;
 
-    switch (primitiveElement->primitive.type)
+    if (primitiveElement->primitive.type == JSON_PRIMITIVE_TYPE_STRING)
     {
-    case JSON_PRIMITIVE_TYPE_STRING:
         PrintIndent(writer, indent_level);
         {
             char *encoded = JsonEncodeString(value);
             WriterWriteF(writer, "\"%s\"", encoded);
             free(encoded);
         }
-        break;
-
-    default:
+    }
+    else
+    {
         PrintIndent(writer, indent_level);
         WriterWrite(writer, value);
-        break;
     }
 }
 
@@ -1939,6 +2019,7 @@ const char *JsonParseErrorToString(const JsonParseError error)
         [JSON_PARSE_ERROR_NO_LIBYAML] =
             "CFEngine was not built with libyaml support",
         [JSON_PARSE_ERROR_LIBYAML_FAILURE] = "libyaml internal failure",
+        [JSON_PARSE_ERROR_NO_SUCH_FILE] = "No such file or directory",
         [JSON_PARSE_ERROR_NO_DATA] = "No data"};
 
     return parse_errors[error];
@@ -1974,7 +2055,9 @@ static JsonParseError JsonParseAsString(
             switch (**data)
             {
             case '\\':
+                break;
             case '"':
+                break;
             case '/':
                 break;
 
@@ -2007,7 +2090,7 @@ static JsonParseError JsonParseAsString(
                 WriterWriteChar(writer, '\\');
                 break;
             }
-            /* Deliberate fall-through */
+            /* fall through */
         default:
             WriterWriteChar(writer, **data);
             break;
@@ -2154,9 +2237,7 @@ static JsonParseError JsonParseAsPrimitive(
     assert(data != NULL);
     assert(*data != NULL);
 
-    switch (**data)
-    {
-    case '"':
+    if (**data == '"')
     {
         char *value = NULL;
         const JsonParseError err = JsonParseAsString(data, &value);
@@ -2167,10 +2248,10 @@ static JsonParseError JsonParseAsPrimitive(
         *json_out = JsonElementCreatePrimitive(
             JSON_PRIMITIVE_TYPE_STRING, JsonDecodeString(value));
         free(value);
-    }
         return JSON_PARSE_OK;
-
-    default:
+    }
+    else
+    {
         if (**data == '-' || **data == '0' || IsDigit(**data))
         {
             const JsonParseError err = JsonParseAsNumber(data, json_out);
@@ -2674,7 +2755,7 @@ JsonParseError JsonParseAnyFile(
     Writer *contents = FileRead(path, size_max, &truncated);
     if (contents == NULL)
     {
-        return JSON_PARSE_ERROR_NO_DATA;
+        return JSON_PARSE_ERROR_NO_SUCH_FILE;
     }
     else if (truncated)
     {
@@ -2706,9 +2787,70 @@ JsonParseError JsonParseFile(
     return JsonParseAnyFile(path, size_max, json_out, false);
 }
 
+bool JsonWalk(JsonElement *element,
+              JsonElementVisitor object_visitor,
+              JsonElementVisitor array_visitor,
+              JsonElementVisitor primitive_visitor,
+              void *data)
+{
+    assert(element != NULL);
+
+    if (element->type == JSON_ELEMENT_TYPE_PRIMITIVE)
+    {
+        if (primitive_visitor != NULL)
+        {
+            return primitive_visitor(element, data);
+        }
+        else
+        {
+            return true;
+        }
+    }
+
+    assert(element->type == JSON_ELEMENT_TYPE_CONTAINER);
+    if (element->container.type == JSON_CONTAINER_TYPE_ARRAY)
+    {
+        if (array_visitor != NULL)
+        {
+            bool keep_going = array_visitor(element, data);
+            if (!keep_going)
+            {
+                return false;
+            }
+        }
+    }
+    else
+    {
+        assert(element->container.type == JSON_CONTAINER_TYPE_OBJECT);
+        if (object_visitor != NULL)
+        {
+            bool keep_going = object_visitor(element, data);
+            if (!keep_going)
+            {
+                return false;
+            }
+        }
+    }
+
+    bool keep_going = true;
+    JsonIterator iter = JsonIteratorInit(element);
+    while (keep_going && JsonIteratorHasMore(&iter))
+    {
+        JsonElement *child = JsonIteratorNextValue(&iter);
+        keep_going = JsonWalk(child, object_visitor, array_visitor, primitive_visitor, data);
+    }
+    return keep_going;
+}
+
+bool JsonErrorVisitor(ARG_UNUSED JsonElement *element, ARG_UNUSED void *data)
+{
+    return false;
+}
+
 /*******************************************************************/
 
 #ifdef WITH_PCRE
+#include <json-pcre.h>
 
 // returns NULL on any failure
 // takes either a pre-compiled pattern OR a regex (one of the two shouldn't be
@@ -2740,7 +2882,7 @@ JsonElement *StringCaptureData(
 
     JsonElement *json = JsonObjectCreate(length / 2);
 
-    for (int i = 1; i < length; i += 2)
+    for (size_t i = 1; i < length; i += 2)
     {
         Buffer *key = SeqAt(s, i - 1);
         Buffer *value = SeqAt(s, i);
