@@ -1,5 +1,5 @@
 /*
-  Copyright 2020 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -25,8 +25,8 @@
 #ifndef CFENGINE_JSON_H
 #define CFENGINE_JSON_H
 
-#include <regex.h>
 #include <writer.h>
+#include <inttypes.h> // int64_t
 
 /**
   @brief JSON data-structure.
@@ -113,6 +113,7 @@ typedef enum
     JSON_PARSE_ERROR_INVALID_START,
     JSON_PARSE_ERROR_NO_LIBYAML,
     JSON_PARSE_ERROR_LIBYAML_FAILURE,
+    JSON_PARSE_ERROR_NO_SUCH_FILE,
     JSON_PARSE_ERROR_NO_DATA,
     JSON_PARSE_ERROR_TRUNCATED,
 
@@ -185,9 +186,8 @@ const char *JsonElementGetPropertyName(const JsonElement *element);
 
 const char *JsonGetPropertyAsString(const JsonElement *element);
 
-JsonElement *StringCaptureData(
-    pcre *pattern, const char *regex, const char *data);
-
+#define NULL_JSON(json) ((json == NULL) || (JsonGetType(json) == JSON_TYPE_NULL))
+#define JSON_NOT_NULL(json) ((json != NULL) && (JsonGetType(json) != JSON_TYPE_NULL))
 
 //////////////////////////////////////////////////////////////////////////////
 // JSON Primitives
@@ -199,10 +199,14 @@ const char *JsonPrimitiveGetAsString(const JsonElement *primitive);
 char *JsonPrimitiveToString(const JsonElement *primitive);
 bool JsonPrimitiveGetAsBool(const JsonElement *primitive);
 long JsonPrimitiveGetAsInteger(const JsonElement *primitive);
+int JsonPrimitiveGetAsInt64(const JsonElement *primitive, int64_t *value_out);
+int64_t JsonPrimitiveGetAsInt64DefaultOnError(const JsonElement *primitive, int64_t default_return);
+int64_t JsonPrimitiveGetAsInt64ExitOnError(const JsonElement *primitive);
 double JsonPrimitiveGetAsReal(const JsonElement *primitive);
 
 JsonElement *JsonStringCreate(const char *value);
 JsonElement *JsonIntegerCreate(int value);
+JsonElement *JsonIntegerCreate64(int64_t value);
 JsonElement *JsonRealCreate(double value);
 JsonElement *JsonBoolCreate(bool value);
 JsonElement *JsonNullCreate();
@@ -254,6 +258,14 @@ void JsonObjectAppendString(
   */
 void JsonObjectAppendInteger(JsonElement *object, const char *key, int value);
 
+/**
+  @brief Append a 64-bit integer field to an object.
+  @param object [in] The JSON object parent.
+  @param key [in] the key of the field.
+  @param value [in] The value of the field.
+  */
+void JsonObjectAppendInteger64(JsonElement *object, const char *key, int64_t value);
+
 /**
   @brief Append an real number field to an object.
   @param object [in] The JSON object parent.
@@ -311,6 +323,15 @@ void JsonObjectAppendElement(
   */
 const char *JsonObjectGetAsString(const JsonElement *object, const char *key);
 
+
+/**
+  @brief Get the value of a field in an object, as a boolean.
+  @param object [in] The JSON object parent.
+  @param key [in] the key of the field.
+  @returns The boolean value for the key or false if key is not available
+  */
+bool JsonObjectGetAsBool(const JsonElement *const object, const char *key);
+
 /**
   @brief Get the value of a field in an object, as an object.
   @param object [in] The JSON object parent.
@@ -406,6 +427,15 @@ void JsonArrayAppendObject(JsonElement *array, JsonElement *object);
   */
 void JsonArrayAppendElement(JsonElement *array, JsonElement *element);
 
+/**
+  * @brief Move elements from JSON array `b` to JSON array `a`.
+  * @param a [in] The JSON array to move elements to.
+  * @param b [in] The JSON array to move elements from.
+  * @note JSON array `a` takes ownership of elements in JSON array `b`.
+  *       JSON array `b` is freed from memory.
+  */
+void JsonArrayExtend(JsonElement *a, JsonElement *b);
+
 /**
   @brief Remove an inclusive range from a JSON array.
   @see SequenceRemoveRange
@@ -458,6 +488,48 @@ JsonPrimitiveType JsonIteratorCurrentPrimitiveType(const JsonIterator *iter);
 bool JsonIteratorHasMore(const JsonIterator *iter);
 
 
+/**
+ * @param element current element being visited
+ * @param data    arbitrary data passed to JsonWalk()
+ * @return        whether the recursive walk should continue or not
+ */
+typedef bool JsonElementVisitor(JsonElement *element, void *data);
+
+/**
+ * Recursively walk over the JSON element.
+ *
+ * @param element           JSON element to start with
+ * @param object_visitor    function to call on child objects
+ * @param array_visitor     function to call on child arrays
+ * @param primitive_visitor function to call on child primitives
+ * @param data              arbitrary data passed to visitor functions
+ * @return                  whether the recursive walk finished or was stopped (see #JsonElementVisitor)
+ *
+ * The function starts with the given JSON element #element and recursively
+ * visits its child elements (if any), calling respective visitor functions on
+ * each of the child elements.
+ *
+ * @note Each parent element is visited before its child elements.
+ * @note Every element in the given JSON is visited unless one of the visitor functions returns
+ *       #false.
+ * @note Every element is visited at most once.
+ * @warning Modifications of the visited elements must be done with extreme caution and good
+ *          understanding of the implementation of the #JsonElement and #JsonIterator internals.
+ */
+bool JsonWalk(JsonElement *element,
+              JsonElementVisitor object_visitor,
+              JsonElementVisitor array_visitor,
+              JsonElementVisitor primitive_visitor,
+              void *data);
+
+/**
+ * Visitor that just stops the walk on any element.
+ *
+ * Can be used as one of the visitor functions for JsonWalk() to detect
+ * undesired child elements.
+ */
+bool JsonErrorVisitor(JsonElement *element, void *data);
+
 //////////////////////////////////////////////////////////////////////////////
 // JSON Parsing
 //////////////////////////////////////////////////////////////////////////////
@@ -536,4 +608,6 @@ void JsonWrite(
 
 void JsonWriteCompact(Writer *w, const JsonElement *element);
 
+void JsonEncodeStringWriter(const char *const unescaped_string, Writer *const writer);
+
 #endif
