@@ -1,5 +1,5 @@
 /*
-  Copyright 2020 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -27,10 +27,9 @@
 #include <string_lib.h>
 #include <misc_lib.h>
 #include <cleanup.h>
+#include <sequence.h>
 
-#ifdef __MINGW32__
 #include <definitions.h>        /* CF_BUFSIZE */
-#endif
 
 char VPREFIX[1024] = ""; /* GLOBAL_C */
 
@@ -38,10 +37,22 @@ static char AgentType[80] = "generic";
 static bool TIMESTAMPS = false;
 
 static LogLevel global_level = LOG_LEVEL_NOTICE; /* GLOBAL_X */
+static LogLevel global_system_log_level = LOG_LEVEL_NOTHING; /* default value that means not set */
 
 static pthread_once_t log_context_init_once = PTHREAD_ONCE_INIT; /* GLOBAL_T */
 static pthread_key_t log_context_key; /* GLOBAL_T, initialized by pthread_key_create */
 
+static Seq *log_buffer = NULL;
+static bool logging_into_buffer = false;
+static LogLevel min_log_into_buffer_level = LOG_LEVEL_NOTHING;
+static LogLevel max_log_into_buffer_level = LOG_LEVEL_NOTHING;
+
+typedef struct LogEntry_
+{
+    LogLevel level;
+    char *msg;
+} LogEntry;
+
 static void LoggingInitializeOnce(void)
 {
     if (pthread_key_create(&log_context_key, &free) != 0)
@@ -62,7 +73,9 @@ LoggingContext *GetCurrentThreadContext(void)
     if (lctx == NULL)
     {
         lctx = xcalloc(1, sizeof(LoggingContext));
-        lctx->log_level = global_level;
+        lctx->log_level = (global_system_log_level != LOG_LEVEL_NOTHING ?
+                           global_system_log_level :
+                           global_level);
         lctx->report_level = global_level;
         pthread_setspecific(log_context_key, lctx);
     }
@@ -143,31 +156,31 @@ LogLevel LogLevelFromString(const char *const level)
     {
         return LOG_LEVEL_NOTHING;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "CRITICAL", len))
+    if (StringEqualN_IgnoreCase(level, "CRITICAL", len))
     {
         return LOG_LEVEL_CRIT;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "errors", len))
+    if (StringEqualN_IgnoreCase(level, "errors", len))
     {
         return LOG_LEVEL_ERR;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "warnings", len))
+    if (StringEqualN_IgnoreCase(level, "warnings", len))
     {
         return LOG_LEVEL_WARNING;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "notices", len))
+    if (StringEqualN_IgnoreCase(level, "notices", len))
     {
         return LOG_LEVEL_NOTICE;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "information", len))
+    if (StringEqualN_IgnoreCase(level, "information", len))
     {
         return LOG_LEVEL_INFO;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "verbose", len))
+    if (StringEqualN_IgnoreCase(level, "verbose", len))
     {
         return LOG_LEVEL_VERBOSE;
     }
-    if (StringSafeEqualN_IgnoreCase(level, "debug", len))
+    if (StringEqualN_IgnoreCase(level, "debug", len))
     {
         return LOG_LEVEL_DEBUG;
     }
@@ -321,7 +334,14 @@ bool WouldLog(LogLevel level)
     return (log_to_console || log_to_syslog || force_hook);
 }
 
-void VLog(LogLevel level, const char *fmt, va_list ap)
+/**
+ * #no_format determines whether #fmt_msg is interpreted as a format string and
+ * combined with #ap to create the real log message (%false) or as a log message
+ * dirrectly (%true) in which case #ap is ignored.
+ *
+ * @see LogNoFormat()
+ */
+static void VLogNoFormat(LogLevel level, const char *fmt_msg, va_list ap, bool no_format)
 {
     LoggingContext *lctx = GetCurrentThreadContext();
 
@@ -338,9 +358,37 @@ void VLog(LogLevel level, const char *fmt, va_list ap)
         return;                            /* early return - save resources */
     }
 
-    char *msg = StringVFormat(fmt, ap);
+    char *msg;
+    if (no_format)
+    {
+        msg = xstrdup(fmt_msg);
+    }
+    else
+    {
+        msg = StringVFormat(fmt_msg, ap);
+    }
     char *hooked_msg = NULL;
 
+    if (logging_into_buffer &&
+        (level >= min_log_into_buffer_level) && (level <= max_log_into_buffer_level))
+    {
+        assert(log_buffer != NULL);
+
+        if (log_buffer == NULL)
+        {
+            /* Should never happen. */
+            Log(LOG_LEVEL_ERR,
+                "Attempt to log a message to an unitialized log buffer, discarding the message");
+        }
+
+        LogEntry *entry = xmalloc(sizeof(LogEntry));
+        entry->level = level;
+        entry->msg = msg;
+
+        SeqAppend(log_buffer, entry);
+        return;
+    }
+
     /* Remove ending EOLN. */
     for (char *sp = msg; *sp != '\0'; sp++)
     {
@@ -376,28 +424,36 @@ void VLog(LogLevel level, const char *fmt, va_list ap)
     free(msg);
 }
 
-/* TODO create libutils/defs.h. */
-#define CF_MAX_BUFSIZE 16384
+void VLog(LogLevel level, const char *fmt, va_list ap)
+{
+    VLogNoFormat(level, fmt, ap, false);
+}
 
 /**
  * @brief Logs binary data in #buf, with unprintable bytes translated to '.'.
  *        Message is prefixed with #prefix.
- * @param #buflen must be no more than CF_MAX_BUFSIZE
+ * @param #buflen must be no more than CF_BUFSIZE
  */
 void LogRaw(LogLevel level, const char *prefix, const void *buf, size_t buflen)
 {
-    const unsigned char *src = buf;
-    unsigned char dst[buflen+1];
-    assert(sizeof(dst) <= CF_MAX_BUFSIZE);
+    if (buflen > CF_BUFSIZE)
+    {
+        debug_abort_if_reached();
+        buflen = CF_BUFSIZE;
+    }
 
     LoggingContext *lctx = GetCurrentThreadContext();
     if (level <= lctx->report_level || level <= lctx->log_level)
     {
+        const unsigned char *src = buf;
+        unsigned char dst[CF_BUFSIZE+1];
+        assert(buflen < sizeof(dst));
         size_t i;
         for (i = 0; i < buflen; i++)
         {
             dst[i] = isprint(src[i]) ? src[i] : '.';
         }
+        assert(i < sizeof(dst));
         dst[i] = '\0';
 
         /* And Log the translated buffer, which is now a valid string. */
@@ -413,7 +469,28 @@ void Log(LogLevel level, const char *fmt, ...)
     va_end(ap);
 }
 
-
+/**
+ * The same as above, but for logging messages without formatting sequences
+ * ("%s", "%d",...). Or more precisely, for *safe* logging of messages that may
+ * contain such sequences (for example Log(LOG_LEVEL_ERR, "%s") can potentially
+ * log some secrets).
+ *
+ * It doesn't have the FUNC_ATTR_PRINTF restriction that causes a compilation
+ * warning/error if #msg is not a constant string.
+ *
+ * @note This is for special cases where #msg was already printf-like formatted,
+ *       the variadic arguments are ignored, they are here just so that
+ *       'va_list ap' can be constructed and passed to VLogNoFormat().
+ *
+ * @see CommitLogBuffer()
+ */
+static void LogNoFormat(LogLevel level, const char *msg, ...)
+{
+    va_list ap;
+    va_start(ap, msg);
+    VLogNoFormat(level, msg, ap, true);
+    va_end(ap);
+}
 
 static bool module_is_enabled[LOG_MOD_MAX];
 static const char *log_modules[LOG_MOD_MAX] =
@@ -542,8 +619,8 @@ void LogDebug(enum LogModule mod, const char *fmt, ...)
 {
     assert(mod < LOG_MOD_MAX);
 
-    /* Did we forget any entry in log_modules? Should be a static assert. */
-    assert(sizeof(log_modules) / sizeof(log_modules[0]) == LOG_MOD_MAX);
+    /* Did we forget any entry in log_modules? */
+    nt_static_assert(sizeof(log_modules) / sizeof(log_modules[0]) == LOG_MOD_MAX);
 
     if (LogModuleEnabled(mod))
     {
@@ -560,7 +637,14 @@ void LogDebug(enum LogModule mod, const char *fmt, ...)
 void LogSetGlobalLevel(LogLevel level)
 {
     global_level = level;
-    LoggingPrivSetLevels(level, level);
+    if (global_system_log_level == LOG_LEVEL_NOTHING)
+    {
+        LoggingPrivSetLevels(level, level);
+    }
+    else
+    {
+        LoggingPrivSetLevels(global_system_log_level, level);
+    }
 }
 
 void LogSetGlobalLevelArgOrExit(const char *const arg)
@@ -582,6 +666,26 @@ LogLevel LogGetGlobalLevel(void)
     return global_level;
 }
 
+void LogSetGlobalSystemLogLevel(LogLevel level)
+{
+    /* LOG_LEVEL_NOTHING means "unset" (see LogUnsetGlobalSystemLogLevel()) */
+    assert(level != LOG_LEVEL_NOTHING);
+
+    global_system_log_level = level;
+    LoggingPrivSetLevels(level, global_level);
+}
+
+void LogUnsetGlobalSystemLogLevel(void)
+{
+    global_system_log_level = LOG_LEVEL_NOTHING;
+    LoggingPrivSetLevels(global_level, global_level);
+}
+
+LogLevel LogGetGlobalSystemLogLevel(void)
+{
+    return global_system_log_level;
+}
+
 void LoggingSetColor(bool enabled)
 {
     LoggingContext *lctx = GetCurrentThreadContext();
@@ -633,3 +737,61 @@ const char *byte_unit(long bytes)
     }
     return "bytes";
 }
+
+void LogEntryDestroy(LogEntry *entry)
+{
+    if (entry != NULL)
+    {
+        free(entry->msg);
+        free(entry);
+    }
+}
+
+void StartLoggingIntoBuffer(LogLevel min_level, LogLevel max_level)
+{
+    assert((log_buffer == NULL) && (!logging_into_buffer));
+
+    if (log_buffer != NULL)
+    {
+        /* Should never happen. */
+        Log(LOG_LEVEL_ERR, "Re-initializing log buffer without prior commit, discarding messages");
+        DiscardLogBuffer();
+    }
+
+    log_buffer = SeqNew(16, LogEntryDestroy);
+    logging_into_buffer = true;
+    min_log_into_buffer_level = min_level;
+    max_log_into_buffer_level = max_level;
+}
+
+void DiscardLogBuffer()
+{
+    SeqDestroy(log_buffer);
+    log_buffer = NULL;
+    logging_into_buffer = false;
+}
+
+void CommitLogBuffer()
+{
+    assert(logging_into_buffer);
+    assert(log_buffer != NULL);
+
+    if (log_buffer == NULL)
+    {
+        /* Should never happen. */
+        Log(LOG_LEVEL_ERR, "Attempt to commit an unitialized log buffer");
+    }
+
+    /* Disable now so that LogNonConstant() below doesn't append the message
+     * into the buffer instaed of logging it. */
+    logging_into_buffer = false;
+
+    const size_t n_entries = SeqLength(log_buffer);
+    for (size_t i = 0; i < n_entries; i++)
+    {
+        LogEntry *entry = SeqAt(log_buffer, i);
+        LogNoFormat(entry->level, entry->msg);
+    }
+
+    DiscardLogBuffer();
+}
