@@ -1,5 +1,5 @@
 /*
-  Copyright 2020 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -448,11 +448,19 @@ static inline uint32_t ByteSwap32(uint32_t le32uint)
     return be32uint;
 }
 # ifdef WORDS_BIGENDIAN
-#  define le32toh(x) ByteSwap32(x)
-#  define htole32(x) ByteSwap32(x)
+#  ifndef le32toh
+#   define le32toh(x) ByteSwap32(x)
+#  endif
+#  ifndef htole32
+#   define htole32(x) ByteSwap32(x)
+#  endif
 # else
-#  define le32toh(x) (x)
-#  define htole32(x) (x)
+#  ifndef le32toh
+#   define le32toh(x) (x)
+#  endif
+#  ifndef htole32
+#   define htole32(x) (x)
+#  endif
 # endif
 #endif // !HAVE_DECL_LE32TOH
 
@@ -468,10 +476,10 @@ int pthread_attr_setstacksize(pthread_attr_t *attr, size_t stacksize);
 # include <sched.h>
 #endif
 
-#ifdef HAVE_ATTR_XATTR_H
-# include <attr/xattr.h>
-#elif defined(HAVE_SYS_XATTR_H)
+#if defined(HAVE_SYS_XATTR_H)
 # include <sys/xattr.h>
+#elif defined(HAVE_ATTR_XATTR_H)
+# include <attr/xattr.h>
 #endif
 
 #ifndef MIN
@@ -552,20 +560,18 @@ int getnetgrent(char **host, char **user, char **domain);
 
 #if !HAVE_DECL_SETNETGRENT
 #if SETNETGRENT_RETURNS_INT
-int
+int setnetgrent(const char *netgroup);
 #else
-void
+void setnetgrent(const char *netgroup);
 #endif
-setnetgrent(const char *netgroup);
 #endif
 
 #if !HAVE_DECL_ENDNETGRENT
 #if ENDNETGRENT_RETURNS_INT
-int
+int endnetgrent(void);
 #else
-void
+void endnetgrent(void);
 #endif
-endnetgrent(void);
 #endif
 
 #if !HAVE_DECL_STRSTR
