@@ -1,5 +1,5 @@
 /*
-  Copyright 2020 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -31,7 +31,8 @@
 #include <logging.h>
 #include <cleanup.h>
 #include <definitions.h> // CF_BUFSIZE
-
+#include <condition_macros.h> // nt_static_assert()
+#include <printsize.h>
 
 char *StringVFormat(const char *fmt, va_list ap)
 {
@@ -61,7 +62,6 @@ size_t StringCopy(const char *const from, char *const to, const size_t buf_size)
     assert(from != NULL);
     assert(to != NULL);
     assert(from != to);
-    assert(buf_size >= 0);
 
     memset(to, 0, buf_size);
     strncpy(to, from, buf_size);
@@ -230,7 +230,14 @@ int StringSafeCompare(const char *const a, const char *const b)
     }
     if (a != NULL && b != NULL)
     {
-        return strcmp(a, b);
+        // Adding this as strcmp gives difference of the buffer values in aarch64
+        // Whereas it gives 1, 0 or -1 in other platforms accordingly.
+        int compare_result = strcmp(a, b);
+        if (compare_result != 0)
+        {
+            compare_result = compare_result / abs(compare_result);
+        }
+        return compare_result;
     }
 
     // Weird edge cases where one is NULL:
@@ -252,12 +259,12 @@ int StringSafeCompareN(const char *const a, const char *const b, const size_t n)
     return NullCompare(a, b);
 }
 
-bool StringSafeEqual(const char *const a, const char *const b)
+bool StringEqual(const char *const a, const char *const b)
 {
     return (StringSafeCompare(a, b) == 0);
 }
 
-bool StringSafeEqualN(const char *const a, const char *const b, const size_t n)
+bool StringEqualN(const char *const a, const char *const b, const size_t n)
 {
     return (StringSafeCompareN(a, b, n) == 0);
 }
@@ -292,19 +299,19 @@ int StringSafeCompareN_IgnoreCase(const char *const a, const char *const b, cons
     return NullCompare(a, b);
 }
 
-bool StringSafeEqual_IgnoreCase(const char *const a, const char *const b)
+bool StringEqual_IgnoreCase(const char *const a, const char *const b)
 {
     return (StringSafeCompare_IgnoreCase(a, b) == 0);
 }
 
-bool StringSafeEqualN_IgnoreCase(const char *const a, const char *const b, const size_t n)
+bool StringEqualN_IgnoreCase(const char *const a, const char *const b, const size_t n)
 {
     return (StringSafeCompareN_IgnoreCase(a, b, n) == 0);
 }
 
-bool StringSafeEqual_untyped(const void *a, const void *b)
+bool StringEqual_untyped(const void *a, const void *b)
 {
-    return StringSafeEqual(a, b);
+    return StringEqual(a, b);
 }
 
 /*********************************************************************/
@@ -389,7 +396,7 @@ char *StringConcatenate(size_t count, const char *first, ...)
 
 char *StringSubstring(const char *source, size_t source_len, int start, int len)
 {
-    size_t end = -1;
+    ssize_t end = -1;
 
     if (len == 0)
     {
@@ -483,15 +490,14 @@ bool EmptyString(const char *s)
  */
 int StringToLong(const char *str, long *value_out)
 {
+    nt_static_assert(ERANGE != 0);
     assert(str != NULL);
     assert(value_out != NULL);
-    assert(ERANGE != 0);
 
     char *endptr = NULL;
-    long val;
 
     errno = 0;
-    val = strtol(str, &endptr, 10);
+    const long val = strtol(str, &endptr, 10);
 
     if ((errno == ERANGE && (val == LONG_MAX || val == LONG_MIN)))
     {
@@ -523,7 +529,43 @@ int StringToLong(const char *str, long *value_out)
 }
 
 /**
- * @brief Log StringToLong conversion error with an identifier for debugging
+ * @brief Convert a string representing a real number to int, discarding the decimal part.
+ *
+ * Attempts to find the first '.' character in order to copy the integer part to a
+ * temporary array before passing it to StringToLong. Should the input string be
+ * an entire integer by itself no copying occurs. The integer part must NOT exceed 12 digits.
+ *
+ * @param[in] str String matching the regex \d+(\.\d+)?, cannot be NULL
+ * @param[out] value_out Where to store result on success, cannot be NULL
+ * @return 0 on success, -84 if the integer part is too long or other error code otherwise (see StringToLong source code)
+ */
+int StringDecimalToLong(const char *str, long *value_out)
+{
+    assert(str != NULL);
+    assert(value_out != NULL);
+
+    size_t len_of_int_part = strcspn(str, ".");
+
+    if (len_of_int_part > PRINTSIZE(INT_MAX))
+    {
+        return -84;  // Integer part too large to allocate buffer
+    }
+
+    if (len_of_int_part == 0 || str[len_of_int_part] == '\0')
+    {
+        return StringToLong(str, value_out);
+    }
+
+    char int_part[len_of_int_part + 1];
+    strncpy(int_part, str, len_of_int_part);
+    int_part[len_of_int_part] = '\0';
+
+    return StringToLong(int_part, value_out);
+}
+
+/**
+ * @brief Log StringToLong & StringToUlong & StringDecimalToLong conversion error
+ *        with an identifier for debugging.
  */
 void LogStringToLongError(const char *str_attempted, const char *id, int error_code)
 {
@@ -543,6 +585,9 @@ void LogStringToLongError(const char *str_attempted, const char *id, int error_c
         case -83:
             error_str = "Not terminated";
             break;
+        case -84:
+            error_str = "Integer part too large";
+            break;
     }
     Log(LOG_LEVEL_ERR, "Conversion error (%d - %s) on '%s' (%s)", error_code, error_str, str_attempted, id);
 }
@@ -593,6 +638,220 @@ long StringToLongExitOnError(const char *str)
     return result;
 }
 
+/**
+ * @brief Converts a string of numerals in base 10 to a unsigned long 
+ *        integer.
+ *
+ * Result is stored in *value_out, return value should be checked.
+ * On error *value_out is unmodified and an error code is returned.
+ * Leading spaces in input string are skipped.
+ * String numeral must be terminated by NULL byte or space (isspace).
+ *
+ * @see StringToUlongExitOnError()
+ * @see StringToUlongDefaultOnError()
+ * @param[in] str String with numerals to convert, cannot be NULL
+ * @param[out] value_out Where to store result on success, cannot be NULL
+ * @return 0 on success, error code otherwise (see source code)
+ */
+int StringToUlong(const char *str, unsigned long *value_out)
+{
+    nt_static_assert(ERANGE != 0);
+    assert(str != NULL);
+    assert(value_out != NULL);
+
+    char *endptr = NULL;
+
+    errno = 0;
+    const unsigned long val = strtoul(str, &endptr, 10);
+
+    if ((errno == ERANGE && val == ULONG_MAX))
+    {
+        return ERANGE; // Overflow
+    }
+
+    // Negative numbers should cause underflow
+    if (val > 0) // "-0" is ok
+    {
+        const char *ch = str;
+        while (ch < endptr && isspace(*ch)) ch++;
+        if (*ch == '-')
+        {
+            return ERANGE; // Underflow
+        }
+    }
+
+    if (endptr == str)
+    {
+        return -81; // No digits found
+    }
+
+    if (endptr == NULL)
+    {
+        return -82; // endpointer not set by strtol
+    }
+
+    if (*endptr != '\0' && !isspace(*endptr))
+    {
+        return -83; // string not properly terminated
+    }
+
+    if (errno != 0)
+    {
+        return errno; // Unknown error
+    }
+
+    *value_out = val;
+    return 0;
+}
+
+/**
+ * @brief Converts a string of numerals in base 10 to a unsigned long 
+ *        integer, uses a default value if errors occur.
+ *
+ * @see StringToUlong()
+ * @param[in] str String with numerals to convert, cannot be NULL
+ * @param[in] default_return Value to return on error
+ * @return Result of conversion (or default_return in case of error)
+ */
+unsigned long StringToUlongDefaultOnError(const char *str, unsigned long default_return)
+{
+    assert(str != NULL);
+    unsigned long result = 0;
+    int return_code = StringToUlong(str, &result);
+    if (return_code != 0)
+    {
+        // Do not log anything because this can be used frequently
+        return default_return;
+    }
+    return result;
+}
+
+/**
+ * @brief Converts a string of numerals in base 10 to a unsigned long 
+ *        integer, exits on error.
+ *
+ * Only use this function in contexts/components where it is acceptable
+ * to immediately exit when something goes wrong.
+ *
+ * @warning This function can exit the process based on string contents
+ * @see StringToUlong()
+ * @param[in] str String with numerals to convert, cannot be NULL
+ * @return Result of conversion
+ */
+unsigned long StringToUlongExitOnError(const char *str)
+{
+    assert(str != NULL);
+    unsigned long result;
+    int return_code = StringToUlong(str, &result);
+    if (return_code != 0)
+    {
+        LogStringToLongError(str, "StringToUlongExitOnError", return_code);
+        DoCleanupAndExit(EXIT_FAILURE);
+    }
+    return result;
+}
+
+/**
+ * @brief Converts a string of numerals in base 10 to 64-bit signed int
+ *
+ * Result is stored in *value_out, return value should be checked.
+ * On error *value_out is unmodified and false is returned.
+ *
+ * @see StringToLong()
+ * @param[in] str String with numerals to convert, cannot be NULL
+ * @param[out] value_out Where to store result on success, cannot be NULL
+ * @return 0 on success, error code otherwise (see source code)
+ */
+int StringToInt64(const char *str, int64_t *value_out)
+{
+    nt_static_assert(sizeof(int64_t) == sizeof(intmax_t));
+    nt_static_assert(ERANGE != 0);
+    assert(str != NULL);
+    assert(value_out != NULL);
+
+    char *endptr = NULL;
+    int64_t val;
+
+    errno = 0;
+    val = strtoimax(str, &endptr, 10);
+
+    if ((errno == ERANGE && (val == INTMAX_MAX || val == INTMAX_MIN)))
+    {
+        return ERANGE; // Overflow or underflow
+    }
+
+    if (endptr == str)
+    {
+        return -81; // No digits found
+    }
+
+    if (endptr == NULL)
+    {
+        return -82; // endpointer not set by strtol
+    }
+
+    if (*endptr != '\0' && !isspace(*endptr))
+    {
+        return -83; // string not properly terminated
+    }
+
+    if (errno != 0)
+    {
+        return errno; // Unknown error
+    }
+
+    *value_out = val;
+    return 0;
+}
+
+/**
+ * @brief Convert a string to int64_t, exits if parsing fails
+ *
+ * Only use this function in contexts/components where it is acceptable
+ * to immediately exit when something goes wrong.
+ *
+ * @warning This function can exit the process based on string contents
+ * @see StringToInt64()
+ * @see StringToLongExitOnError()
+ * @param[in] str String with numerals to convert, cannot be NULL
+ * @return Result of conversion
+ */
+int64_t StringToInt64ExitOnError(const char *str)
+{
+    assert(str != NULL);
+
+    int64_t result;
+    const int error_code = StringToInt64(str, &result);
+    if (error_code != 0)
+    {
+        LogStringToLongError(str, "StringToInt64ExitOnError", error_code);
+        DoCleanupAndExit(EXIT_FAILURE);
+    }
+    return result;
+}
+
+/**
+ * @brief Converts a string to int64_t, with a default value in case of errors
+ *
+ * @see StringToInt64()
+ * @see StringToLongDefaultOnError()
+ * @param[in] str String with numerals to convert, cannot be NULL
+ * @param[in] default_return Value to return on error
+ * @return Result of conversion
+ */
+int64_t StringToInt64DefaultOnError(const char *str, int64_t default_return)
+{
+    assert(str != NULL);
+
+    int64_t result;
+    const int error_code = StringToInt64(str, &result);
+    if (error_code != 0)
+    {
+        return default_return;
+    }
+    return result;
+}
+
 /**
  * @brief Convert a string of numerals to a long integer (deprecated).
  *
@@ -734,7 +993,7 @@ int CountChar(const char *string, char sep)
     return count;
 }
 
-void ReplaceChar(char *in, char *out, int outSz, char from, char to)
+void ReplaceChar(const char *in, char *out, int outSz, char from, char to)
 /* Replaces all occurrences of 'from' to 'to' in preallocated
  * string 'out'. */
 {
@@ -765,9 +1024,27 @@ void ReplaceChar(char *in, char *out, int outSz, char from, char to)
  */
 ssize_t StringReplace(char *buf, size_t buf_size,
                       const char *find, const char *replace)
+{
+    return StringReplaceN(buf, buf_size, find, replace, buf_size);
+}
+
+
+/**
+ * Replace the first at most #n occurrences of #find in #buf with #replace.
+ *
+ * @return the length of #buf or -1 in case of overflow, or 0 if no replace
+ *         took place.
+ */
+ssize_t StringReplaceN(char *buf, size_t buf_size,
+                       const char *find, const char *replace, size_t n)
 {
     assert(find[0] != '\0');
 
+    if (n == 0)
+    {
+        return 0;
+    }
+
     char *p = strstr(buf, find);
     if (p == NULL)
     {
@@ -800,7 +1077,8 @@ ssize_t StringReplace(char *buf, size_t buf_size,
 
         buf_idx = buf_newidx + find_len;
         p = strstr(&buf[buf_idx], find);
-    } while (p != NULL);
+        n--;
+    } while ((p != NULL) && (n > 0));
 
     /* Copy leftover plus terminating '\0'. */
     size_t leftover_len = buf_len - buf_idx;
@@ -1182,10 +1460,10 @@ size_t TrimCSVLineCRLFStrict(char *const data)
     return length;
 }
 
-void StringCloseHole(char *s, const int start, const int end)
+void StringCloseHole(char *s, const size_t start, const size_t end)
 {
     assert(s != NULL);
-    assert(0 <= start && start <= end && end <= strlen(s));
+    assert(start <= end && end <= strlen(s));
     assert((end - start) <= strlen(s));
 
     if (end > start)
@@ -1284,7 +1562,7 @@ void *memcchr(const void *buf, int c, size_t buf_size)
  * @param[out] obuf   : the output buffer
  * @retval    true if string was capped, false if not
  */
-bool StringNotMatchingSetCapped(const char *isp, int limit,
+bool StringNotMatchingSetCapped(const char *isp, size_t limit,
                       const char *exclude, char *obuf)
 {
     size_t l = strcspn(isp, exclude);
@@ -1305,7 +1583,7 @@ bool StringNotMatchingSetCapped(const char *isp, int limit,
 
 bool StringAppend(char *dst, const char *src, size_t n)
 {
-    int i, j;
+    size_t i, j;
     n--;
     for (i = 0; i < n && dst[i]; i++)
     {
@@ -1500,7 +1778,7 @@ bool StringMatchesOption(
     }
     else if (length == 2)
     {
-        return StringSafeEqual(supplied, shortopt);
+        return StringEqual(supplied, shortopt);
     }
-    return StringSafeEqualN_IgnoreCase(supplied, longopt, length);
+    return StringEqualN_IgnoreCase(supplied, longopt, length);
 }
