@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -25,6 +25,19 @@
 #ifndef CFENGINE_ACL_TOOLS_H
 #define CFENGINE_ACL_TOOLS_H
 
-bool CopyACLs(const char *src, const char *dst);
+bool CopyACLs(const char *src, const char *dst, bool *change);
+
+/**
+ * Allow access to the given file ONLY for the given users using ACLs. Existing
+ * 'user:...' ACL entries are replaced by new entries allowing access for the
+ * given users.
+ *
+ * @param path  file to set the ACLs on
+ * @param users users to allow access for
+ * @param allow_writes whether to allow write access (read access is always allowed)
+ * @param allow_execute whether to allow execute access (read access is always allowed)
+ * @return Whether the change of ACLs was successful or not
+ */
+bool AllowAccessForUsers(const char *path, StringSet *users, bool allow_writes, bool allow_execute);
 
 #endif // CFENGINE_ACL_TOOLS_H
