@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -49,18 +49,19 @@ PromiseResult PromiseResultUpdate(PromiseResult prior, PromiseResult evidence)
         case PROMISE_RESULT_NOOP:
         case PROMISE_RESULT_SKIPPED:
             return prior;
+        default:
+            ProgrammingError("Unexpected promise result");
         }
-
     case PROMISE_RESULT_SKIPPED:
         return evidence;
 
     case PROMISE_RESULT_NOOP:
-        switch (evidence)
+        if (evidence == PROMISE_RESULT_SKIPPED)
         {
-        case PROMISE_RESULT_SKIPPED:
             return prior;
-
-        default:
+        }
+        else
+        {
             return evidence;
         }
 
