@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -31,6 +31,7 @@
 #include <chflags.h>
 #include <audit.h>
 #include <protocol.h> // ParseProtocolVersionPolicy()
+#include <string_lib.h>         /* StringEqual() */
 
 #define CF_DEFINECLASSES "classes"
 #define CF_TRANSACTION   "action"
@@ -53,6 +54,7 @@ Attributes GetFilesAttributes(const EvalContext *ctx, const Promise *pp)
     attr.haveselect = PromiseGetConstraintAsBoolean(ctx, "file_select", pp);
     attr.haverename = PromiseGetConstraintAsBoolean(ctx, "rename", pp);
     attr.havedelete = PromiseGetConstraintAsBoolean(ctx, "delete", pp);
+    attr.content = PromiseGetConstraintAsRval(pp, "content", RVAL_TYPE_SCALAR);
     attr.haveperms = PromiseGetConstraintAsBoolean(ctx, "perms", pp);
     attr.havechange = PromiseGetConstraintAsBoolean(ctx, "changes", pp);
     attr.havecopy = PromiseGetConstraintAsBoolean(ctx, "copy_from", pp);
@@ -481,15 +483,15 @@ Acl GetAclConstraints(const EvalContext *ctx, const Promise *pp)
 static FilePerms GetPermissionConstraints(const EvalContext *ctx, const Promise *pp)
 {
     FilePerms p;
-    char *value;
+    char *mode_value;
     Rlist *list;
 
-    value = PromiseGetConstraintAsRval(pp, "mode", RVAL_TYPE_SCALAR);
+    mode_value = PromiseGetConstraintAsRval(pp, "mode", RVAL_TYPE_SCALAR);
 
     p.plus = CF_SAMEMODE;
     p.minus = CF_SAMEMODE;
 
-    if (!ParseModeString(value, &p.plus, &p.minus))
+    if (!ParseModeString(mode_value, &p.plus, &p.minus))
     {
         Log(LOG_LEVEL_ERR, "Problem validating a mode string");
         PromiseRef(LOG_LEVEL_ERR, pp);
@@ -510,14 +512,8 @@ static FilePerms GetPermissionConstraints(const EvalContext *ctx, const Promise
     p.groups = Rlist2GidList((Rlist *) PromiseGetConstraintAsRval(pp, "groups", RVAL_TYPE_LIST), pp);
 
     p.findertype = PromiseGetConstraintAsRval(pp, "findertype", RVAL_TYPE_SCALAR);
-    p.rxdirs = PromiseGetConstraintAsBoolean(ctx, "rxdirs", pp);
-
-// The default should be true
-
-    if (!PromiseGetConstraintAsRval(pp, "rxdirs", RVAL_TYPE_SCALAR))
-    {
-        p.rxdirs = true;
-    }
+    p.rxdirs = PromiseGetConstraintAsBooleanWithDefault(ctx, "rxdirs", pp,
+                                                        true, (mode_value != NULL));
 
     return p;
 }
@@ -683,28 +679,28 @@ TransactionContext GetTransactionConstraints(const EvalContext *ctx, const Promi
 
     /* Warn if promise locking was used with a promise that doesn't support it.
      * XXX: EvalContextGetPass() takes 'EvalContext *' instead of 'const EvalContext *'*/
-    if ((strcmp("access", pp->parent_promise_type->name) == 0 ||
-         strcmp("classes", pp->parent_promise_type->name) == 0 ||
-         strcmp("defaults", pp->parent_promise_type->name) == 0 ||
-         strcmp("meta", pp->parent_promise_type->name) == 0 ||
-         strcmp("roles", pp->parent_promise_type->name) == 0 ||
-         strcmp("vars", pp->parent_promise_type->name) == 0))
+    if ((strcmp("access", PromiseGetPromiseType(pp)) == 0 ||
+         strcmp("classes", PromiseGetPromiseType(pp)) == 0 ||
+         strcmp("defaults", PromiseGetPromiseType(pp)) == 0 ||
+         strcmp("meta", PromiseGetPromiseType(pp)) == 0 ||
+         strcmp("roles", PromiseGetPromiseType(pp)) == 0 ||
+         strcmp("vars", PromiseGetPromiseType(pp)) == 0))
     {
         if (t.ifelapsed != CF_NOINT)
         {
             Log(LOG_LEVEL_WARNING,
                 "ifelapsed attribute specified in action body for %s promise '%s',"
                 " but %s promises do not support promise locking",
-                pp->parent_promise_type->name, pp->promiser,
-                pp->parent_promise_type->name);
+                PromiseGetPromiseType(pp), pp->promiser,
+                PromiseGetPromiseType(pp));
         }
         if (t.expireafter != CF_NOINT)
         {
             Log(LOG_LEVEL_WARNING,
                 "expireafter attribute specified in action body for %s promise '%s',"
                 " but %s promises do not support promise locking",
-                pp->parent_promise_type->name, pp->promiser,
-                pp->parent_promise_type->name);
+                PromiseGetPromiseType(pp), pp->promiser,
+                PromiseGetPromiseType(pp));
         }
     }
 
@@ -739,6 +735,28 @@ TransactionContext GetTransactionConstraints(const EvalContext *ctx, const Promi
 
 /*******************************************************************/
 
+bool IsClassesBodyConstraint(const char *constraint)
+{
+    return (
+        StringEqual(constraint, "classes") ||
+        StringEqual(constraint, "classes_name") ||
+        StringEqual(constraint, "scope") ||
+        StringEqual(constraint, "promise_repaired") ||
+        StringEqual(constraint, "repair_failed") ||
+        StringEqual(constraint, "repair_denied") ||
+        StringEqual(constraint, "repair_timeout") ||
+        StringEqual(constraint, "promise_kept") ||
+        StringEqual(constraint, "cancel_repaired") ||
+        StringEqual(constraint, "cancel_kept") ||
+        StringEqual(constraint, "cancel_notkept") ||
+        StringEqual(constraint, "kept_returncodes") ||
+        StringEqual(constraint, "repaired_returncodes") ||
+        StringEqual(constraint, "failed_returncodes") ||
+        StringEqual(constraint, "persist_time") ||
+        StringEqual(constraint, "timer_policy")
+        );
+}
+
 DefineClasses GetClassDefinitionConstraints(const EvalContext *ctx, const Promise *pp)
 {
     DefineClasses c;
