@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -357,7 +357,7 @@ typedef struct Policy_ Policy;
 typedef struct Bundle_ Bundle;
 typedef struct Body_ Body;
 typedef struct Promise_ Promise;
-typedef struct PromiseType_ PromiseType;
+typedef struct BundleSection_ BundleSection;
 typedef struct FnCall_ FnCall;
 
 /*************************************************************************/
@@ -427,6 +427,7 @@ typedef enum
     COMMON_CONTROL_SITE_CLASSES,
     COMMON_CONTROL_SYSLOG_HOST,
     COMMON_CONTROL_SYSLOG_PORT,
+    COMMON_CONTROL_SYSTEM_LOG_LEVEL,
     COMMON_CONTROL_FIPS_MODE,
     COMMON_CONTROL_BWLIMIT,
     COMMON_CONTROL_CACHE_SYSTEM_FUNCTIONS,
@@ -506,6 +507,7 @@ typedef enum
     EXEC_CONTROL_EXECUTORFACILITY,
     EXEC_CONTROL_EXECCOMMAND,
     EXEC_CONTROL_AGENT_EXPIREAFTER,
+    EXEC_CONTROL_RUNAGENT_ALLOW_USERS,
     EXEC_CONTROL_NONE
 } ExecControl;
 
@@ -551,7 +553,6 @@ typedef enum
 #define CF_HIGHINIT 999999L
 #define CF_LOWINIT -999999L
 
-#define CF_SIGNALRANGE "hup,int,trap,kill,pipe,cont,abrt,stop,quit,term,child,usr1,usr2,bus,segv"
 #define CF_BOOL      "true,false,yes,no,on,off"
 #define CF_LINKRANGE "symlink,hardlink,relative,absolute"
 #define CF_TIMERANGE "0,2147483647" /* i.e. "0,0x7fffffff" */
@@ -749,25 +750,6 @@ typedef enum
     FILE_CHANGE_REPORT_ALL
 } FileChangeReport;
 
-enum signalnames
-{
-    cfa_hup,
-    cfa_int,
-    cfa_trap,
-    cfa_kill,
-    cfa_pipe,
-    cfa_cont,
-    cfa_abrt,
-    cfa_stop,
-    cfa_quit,
-    cfa_term,
-    cfa_child,
-    cfa_usr1,
-    cfa_usr2,
-    cfa_bus,
-    cfa_segv
-};
-
 typedef enum
 {
     PACKAGE_ACTION_ADD,
@@ -1537,6 +1519,7 @@ typedef struct
     FilePerms perms;
     FileCopy copy;
     FileDelete delete;
+    char *content;
     FileRename rename;
     FileChange change;
     FileLink link;
@@ -1628,6 +1611,7 @@ typedef struct
     .perms = {0},\
     .copy = {0},\
     .delete = {0},\
+    .content = NULL,\
     .rename = {0},\
     .change = {0},\
     .link = {0},\
