@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -44,11 +44,24 @@ extern struct utsname VSYSNAME;
 extern char VIPADDRESS[CF_MAX_IP_LEN];
 extern char VPREFIX[1024];
 
-extern char VDOMAIN[CF_MAXVARSIZE];
-extern char VFQNAME[];
-extern char VUQNAME[];
+extern char VFQNAME[CF_MAXVARSIZE];
+extern char VDOMAIN[CF_MAXVARSIZE / 2];
+extern char VUQNAME[CF_MAXVARSIZE / 2];
+
+typedef enum EvalMode {
+    EVAL_MODE_NORMAL = 0,                 /* needs to be 'false' to work for DONTDO below */
+    EVAL_MODE_DRY_RUN = 1,
+    EVAL_MODE_SIMULATE_DIFF = 2,
+    EVAL_MODE_SIMULATE_MANIFEST = 3,
+    EVAL_MODE_SIMULATE_MANIFEST_FULL = 4,
+} EvalMode;
+
+extern EvalMode EVAL_MODE;
+
+/* This is only for backwards compatibility with code that uses
+ * 'if (!DONTDO)'. */
+#define DONTDO ((bool) EVAL_MODE)
 
-extern bool DONTDO;
 extern bool MINUSF;
 
 extern int EDITFILESIZE;
