@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -48,7 +48,7 @@ TYPED_MAP_DECLARE(Class, char *, Class *)
 
 TYPED_MAP_DEFINE(Class, char *, Class *,
                  StringHash_untyped,
-                 StringSafeEqual_untyped,
+                 StringEqual_untyped,
                  free,
                  ClassDestroy_untyped)
 
@@ -66,14 +66,13 @@ struct ClassTableIterator_
 };
 
 
-/**
- * @param #tags is a comma separated string of words.
- *              Both "" or NULL are equivalent.
- */
 static void ClassInit(Class *cls,
                       const char *ns, const char *name,
-                      bool is_soft, ContextScope scope, const char *tags)
+                      bool is_soft, ContextScope scope, StringSet *tags,
+                      const char *comment)
 {
+    assert(cls != NULL);
+
     if (ns == NULL || strcmp(ns, "default") == 0)
     {
         cls->ns = NULL;
@@ -88,21 +87,22 @@ static void ClassInit(Class *cls,
 
     cls->is_soft = is_soft;
     cls->scope = scope;
-
-    cls->tags = StringSetFromString(tags, ',');
+    cls->tags = (tags ? tags : StringSetNew());
     if (!is_soft && !StringSetContains(cls->tags, "hardclass"))
     {
         StringSetAdd(cls->tags, xstrdup("hardclass"));
     }
+    cls->comment = SafeStringDuplicate(comment);
 }
 
 static void ClassDestroySoft(Class *cls)
 {
-    if (cls)
+    if (cls != NULL)
     {
         free(cls->ns);
         free(cls->name);
         StringSetDestroy(cls->tags);
+        free(cls->comment);
     }
 }
 
@@ -135,7 +135,7 @@ void ClassTableDestroy(ClassTable *table)
 
 bool ClassTablePut(ClassTable *table,
                    const char *ns, const char *name,
-                   bool is_soft, ContextScope scope, const char *tags)
+                   bool is_soft, ContextScope scope, StringSet *tags, const char *comment)
 {
     assert(name);
     assert(is_soft || (!ns || strcmp("default", ns) == 0)); // hard classes should have default namespace
@@ -147,7 +147,7 @@ bool ClassTablePut(ClassTable *table,
     }
 
     Class *cls = xmalloc(sizeof(*cls));
-    ClassInit(cls, ns, name, is_soft, scope, tags);
+    ClassInit(cls, ns, name, is_soft, scope, tags, comment);
 
     /* (cls->name != name) because canonification has happened. */
     char *fullname = StringConcatenate(3, ns, ":", cls->name);
