@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -35,6 +35,7 @@ typedef struct
     ContextScope scope;
     bool is_soft;
     StringSet *tags;
+    char *comment;
 } Class;
 
 
@@ -44,7 +45,8 @@ typedef struct ClassTableIterator_ ClassTableIterator;
 ClassTable *ClassTableNew(void);
 void ClassTableDestroy(ClassTable *table);
 
-bool ClassTablePut(ClassTable *table, const char *ns, const char *name, bool is_soft, ContextScope scope, const char *tags);
+bool ClassTablePut(ClassTable *table, const char *ns, const char *name, bool is_soft, ContextScope scope,
+                   StringSet *tags, const char *comment);
 Class *ClassTableGet(const ClassTable *table, const char *ns, const char *name);
 Class *ClassTableMatch(const ClassTable *table, const char *regex);
 bool ClassTableRemove(ClassTable *table, const char *ns, const char *name);
