@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -33,6 +33,7 @@
 #include <logging.h>
 #include <rlist.h>
 #include <string_lib.h>
+#include <unix.h>               /* GetUserID(), GetGroupID() */
 
 
 const char *MapAddress(const char *unspec_address)
@@ -220,56 +221,24 @@ char *Rlist2String(Rlist *list, char *sep)
 
 int SignalFromString(const char *s)
 {
-    int i = 0;
-    Item *ip, *names = SplitString(CF_SIGNALRANGE, ',');
+    char *signal_names[15] = {
+        "hup", "int", "trap", "kill", "pipe", "cont", "abrt", "stop",
+        "quit", "term", "child", "usr1", "usr2", "bus", "segv"
+    };
+    int signals[15] = {
+       SIGHUP, SIGINT, SIGTRAP, SIGKILL, SIGPIPE, SIGCONT, SIGABRT, SIGSTOP,
+       SIGQUIT, SIGTERM, SIGCHLD, SIGUSR1, SIGUSR2, SIGBUS, SIGSEGV
+    };
 
-    for (ip = names; ip != NULL; ip = ip->next)
+    for (size_t i = 0; i < 15; i++)
     {
-        if (strcmp(s, ip->name) == 0)
+        if (StringEqual(s, signal_names[i]))
         {
-            break;
+            return signals[i];
         }
-        i++;
-    }
-
-    DeleteItemList(names);
-
-    switch (i)
-    {
-    case cfa_hup:
-        return SIGHUP;
-    case cfa_int:
-        return SIGINT;
-    case cfa_trap:
-        return SIGTRAP;
-    case cfa_kill:
-        return SIGKILL;
-    case cfa_pipe:
-        return SIGPIPE;
-    case cfa_cont:
-        return SIGCONT;
-    case cfa_abrt:
-        return SIGABRT;
-    case cfa_stop:
-        return SIGSTOP;
-    case cfa_quit:
-        return SIGQUIT;
-    case cfa_term:
-        return SIGTERM;
-    case cfa_child:
-        return SIGCHLD;
-    case cfa_usr1:
-        return SIGUSR1;
-    case cfa_usr2:
-        return SIGUSR2;
-    case cfa_bus:
-        return SIGBUS;
-    case cfa_segv:
-        return SIGSEGV;
-    default:
-        return -1;
     }
 
+    return -1;
 }
 
 ContextScope ContextScopeFromString(const char *scope_str)
@@ -351,31 +320,41 @@ DataType ConstraintSyntaxGetDataType(const ConstraintSyntax *body_syntax, const
 
 /****************************************************************************/
 
+// Warning: Defaults to true on unexpected (non-bool) input
 bool BooleanFromString(const char *s)
 {
-    Item *list = SplitString(CF_BOOL, ','), *ip;
-    int count = 0;
+    assert(StringEqual(CF_BOOL, "true,false,yes,no,on,off"));
+    assert(s != NULL);
 
-    for (ip = list; ip != NULL; ip = ip->next)
-    {
-        if (strcmp(s, ip->name) == 0)
-        {
-            break;
-        }
-
-        count++;
-    }
-
-    DeleteItemList(list);
-
-    if (count % 2)
+    if (StringEqual(s, "false")
+        || StringEqual(s, "no")
+        || StringEqual(s, "off"))
     {
         return false;
     }
-    else
-    {
-        return true;
-    }
+    // Unnecessary to check here because the default is true anyway:
+    // if (StringEqual(s, "true")
+    //     || StringEqual(s, "yes")
+    //     || StringEqual(s, "on"))
+    // {
+    //     return true;
+    // }
+
+    // Default to true to preserve old behavior:
+    return true;
+}
+
+bool StringIsBoolean(const char *s)
+{
+    assert(StringEqual(CF_BOOL, "true,false,yes,no,on,off"));
+    assert(s != NULL);
+
+    return (StringEqual(s, "true")
+            || StringEqual(s, "false")
+            || StringEqual(s, "yes")
+            || StringEqual(s, "no")
+            || StringEqual(s, "on")
+            || StringEqual(s, "off"));
 }
 
 /****************************************************************************/
@@ -1086,84 +1065,75 @@ GidList *Rlist2GidList(Rlist *gidnames, const Promise *pp)
 
 uid_t Str2Uid(const char *uidbuff, char *usercopy, const Promise *pp)
 {
-    Item *ip, *tmplist;
-    struct passwd *pw;
-    int offset, uid = -2, tmp = -2;
-    char *machine, *user, *domain;
-
-    if (uidbuff[0] == '+')      /* NIS group - have to do this in a roundabout     */
-    {                           /* way because calling getpwnam spoils getnetgrent */
-        offset = 1;
-        if (uidbuff[1] == '@')
-        {
-            offset++;
-        }
-
-        setnetgrent(uidbuff + offset);
-        tmplist = NULL;
-
-        while (getnetgrent(&machine, &user, &domain))
-        {
-            if (user != NULL)
-            {
-                AppendItem(&tmplist, user, NULL);
-            }
-        }
-
-        endnetgrent();
-
-        for (ip = tmplist; ip != NULL; ip = ip->next)
-        {
-            if ((pw = getpwnam(ip->name)) == NULL)
-            {
-                Log(LOG_LEVEL_INFO, "Unknown user in promise '%s'", ip->name);
-
-                if (pp != NULL)
-                {
-                    PromiseRef(LOG_LEVEL_INFO, pp);
-                }
-
-                uid = CF_UNKNOWN_OWNER; /* signal user not found */
-            }
-            else
-            {
-                uid = pw->pw_uid;
-
-                if (usercopy != NULL)
-                {
-                    strcpy(usercopy, ip->name);
-                }
-            }
-        }
-
-        DeleteItemList(tmplist);
-        return uid;
+    if (StringEqual(uidbuff, "*"))
+    {
+        return CF_SAME_OWNER;        /* signals wildcard */
     }
 
     if (StringIsNumeric(uidbuff))
     {
-        sscanf(uidbuff, "%d", &tmp);
-        uid = (uid_t) tmp;
+#ifdef __hpux
+        /* sscanf("0", "%ju", &tmp) with 'uintmax_t' fails, 'int' and '%d' work */
+        int tmp;
+        NDEBUG_UNUSED int ret = sscanf(uidbuff, "%d", &tmp);
+        assert(ret == 1);
+#else
+        uintmax_t tmp;
+        NDEBUG_UNUSED int ret = sscanf(uidbuff, "%ju", &tmp);
+        assert(ret == 1);
+#endif
+        return (uid_t) tmp;
+    }
+
+    uid_t uid = CF_UNKNOWN_OWNER;
+    if (uidbuff[0] == '+')
+    {
+        if (uidbuff[1] == '@')
+        {
+            uidbuff++;
+        }
+
+        char *machine = NULL;
+        char *user = NULL;
+        char *domain = NULL;
+        setnetgrent(uidbuff);
+        while ((uid == CF_UNKNOWN_OWNER) && (getnetgrent(&machine, &user, &domain) == 1))
+        {
+            if (user != NULL)
+            {
+                if (GetUserID(user, &uid, LOG_LEVEL_INFO))
+                {
+                    if (usercopy != NULL)
+                    {
+                        strcpy(usercopy, user);
+                    }
+                }
+                else
+                {
+                    if (pp != NULL)
+                    {
+                        PromiseRef(LOG_LEVEL_INFO, pp);
+                    }
+                }
+            }
+        }
+        endnetgrent();
+
+        return uid;
+    }
+
+    if (GetUserID(uidbuff, &uid, LOG_LEVEL_INFO))
+    {
+        if (usercopy != NULL)
+        {
+            strcpy(usercopy, uidbuff);
+        }
     }
     else
     {
-        if (strcmp(uidbuff, "*") == 0)
+        if (pp)
         {
-            uid = CF_SAME_OWNER;        /* signals wildcard */
-        }
-        else if ((pw = getpwnam(uidbuff)) == NULL)
-        {
-            Log(LOG_LEVEL_INFO, "Unknown user '%s' in promise", uidbuff);
-            uid = CF_UNKNOWN_OWNER;     /* signal user not found */
-
-            if (usercopy != NULL)
-            {
-                strcpy(usercopy, uidbuff);
-            }
-        }
-        else
-        {
-            uid = pw->pw_uid;
+            PromiseRef(LOG_LEVEL_INFO, pp);
         }
     }
 
@@ -1174,39 +1144,39 @@ uid_t Str2Uid(const char *uidbuff, char *usercopy, const Promise *pp)
 
 gid_t Str2Gid(const char *gidbuff, char *groupcopy, const Promise *pp)
 {
-    struct group *gr;
-    int gid = -2, tmp = -2;
+    if (StringEqual(gidbuff, "*"))
+    {
+        return CF_SAME_GROUP;        /* signals wildcard */
+    }
 
     if (StringIsNumeric(gidbuff))
     {
-        sscanf(gidbuff, "%d", &tmp);
-        gid = (gid_t) tmp;
+#ifdef __hpux
+        /* sscanf("0", "%ju", &tmp) with 'uintmax_t' fails, 'int' and '%d' work */
+        int tmp;
+        NDEBUG_UNUSED int ret = sscanf(gidbuff, "%d", &tmp);
+        assert(ret == 1);
+#else
+        uintmax_t tmp;
+        NDEBUG_UNUSED int ret = sscanf(gidbuff, "%ju", &tmp);
+        assert(ret == 1);
+#endif
+        return (gid_t) tmp;
+    }
+
+    gid_t gid = CF_UNKNOWN_GROUP;
+    if (GetGroupID(gidbuff, &gid, LOG_LEVEL_INFO))
+    {
+        if (groupcopy != NULL)
+        {
+            strcpy(groupcopy, gidbuff);
+        }
     }
     else
     {
-        if (strcmp(gidbuff, "*") == 0)
+        if (pp)
         {
-            gid = CF_SAME_GROUP;        /* signals wildcard */
-        }
-        else if ((gr = getgrnam(gidbuff)) == NULL)
-        {
-            Log(LOG_LEVEL_INFO, "Unknown group '%s' in promise", gidbuff);
-
-            if (pp)
-            {
-                PromiseRef(LOG_LEVEL_INFO, pp);
-            }
-
-            gid = CF_UNKNOWN_GROUP;
-        }
-        else
-        {
-            gid = gr->gr_gid;
-
-            if (groupcopy != NULL)
-            {
-                strcpy(groupcopy, gidbuff);
-            }
+            PromiseRef(LOG_LEVEL_INFO, pp);
         }
     }
 
