@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -203,7 +203,7 @@ static
 bool IsSubHandle(DBHandle *handle, dbid id, const char *name)
 {
     char *sub_path = DBIdToSubPath(id, name);
-    bool result = StringSafeEqual(handle->filename, sub_path);
+    bool result = StringEqual(handle->filename, sub_path);
     free(sub_path);
     return result;
 }
@@ -452,7 +452,7 @@ DBHandle *GetDBHandleFromFilename(const char *db_file_name)
     ThreadLock(&db_handles_lock);
     for(dbid id=0; id < dbid_max; id++)
     {
-        if (StringSafeEqual(db_handles[id].filename, db_file_name))
+        if (StringEqual(db_handles[id].filename, db_file_name))
         {
             ThreadUnlock(&db_handles_lock);
             return &(db_handles[id]);
@@ -563,6 +563,13 @@ bool WriteDB(DBHandle *handle, const char *key, const void *src, int srcSz)
     return DBPrivWrite(handle->priv, key, strlen(key) + 1, src, srcSz);
 }
 
+bool OverwriteDB(DBHandle *handle, const char *key, const void *value, size_t value_size,
+                 OverwriteCondition Condition, void *data)
+{
+    assert(handle != NULL);
+    return DBPrivOverwrite(handle->priv, key, strlen(key) + 1, value, value_size, Condition, data);
+}
+
 bool HasKeyDB(DBHandle *handle, const char *key, int key_size)
 {
     return DBPrivHasKey(handle->priv, key, key_size);
