@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -74,6 +74,8 @@ struct DBCursorPriv_
 
 static int DB_MAX_READERS = -1;
 
+#define N_LMDB_EINVAL_RETRIES 5
+
 /******************************************************************************/
 
 static void HandleLMDBCorruption(MDB_env *env, const char *msg);
@@ -248,12 +250,17 @@ static bool RepairedAfterOpen(const char *lmdb_file, int fd_tstamp)
     time_t repaired_tstamp = -1;
     ssize_t n_read = read(fd_tstamp, &repaired_tstamp, sizeof(time_t));
     lseek(fd_tstamp, 0, SEEK_SET);
-    if (n_read == 0)
+
+    if (n_read < 0)
+    {
+        Log(LOG_LEVEL_ERR, "Failed to read %s: %s", lmdb_file, GetErrorStr());
+    }
+    else if (n_read == 0)
     {
         /* EOF (empty file) => never repaired */
         Log(LOG_LEVEL_VERBOSE, "DB '%s' never repaired before", lmdb_file);
     }
-    else if (n_read < sizeof(time_t))
+    else if ((size_t) n_read < sizeof(time_t))
     {
         /* error */
         Log(LOG_LEVEL_ERR, "Failed to read the timestamp of repair of the '%s' DB",
@@ -589,12 +596,31 @@ DBPriv *DBPrivOpenDB(const char *const dbpath, const dbid id)
                 dbpath);
         }
     }
+
+    /* There seems to be a race condition causing mdb_txn_begin() return
+     * EINVAL. We do a couple retries before giving up. */
     rc = mdb_txn_begin(db->env, NULL, MDB_RDONLY, &txn);
-    CheckLMDBCorrupted(rc, db->env);
-    if (rc)
+    int attempts = N_LMDB_EINVAL_RETRIES;
+    while ((rc != 0) && (attempts-- > 0))
+    {
+        CheckLMDBCorrupted(rc, db->env);
+        if (rc != EINVAL)
+        {
+            Log(LOG_LEVEL_ERR, "Could not open database txn %s: %s",
+                dbpath, mdb_strerror(rc));
+            goto err;
+        }
+#if HAVE_DECL_SCHED_YIELD && defined(HAVE_SCHED_YIELD)
+        // Not required for this to work, but makes it less likely that the race
+        // condition will persist.
+        sched_yield();
+#endif
+        rc = mdb_txn_begin(db->env, NULL, MDB_RDONLY, &txn);
+    }
+    if (rc != 0)
     {
         Log(LOG_LEVEL_ERR, "Could not open database txn %s: %s",
-              dbpath, mdb_strerror(rc));
+            dbpath, mdb_strerror(rc));
         goto err;
     }
     rc = mdb_open(txn, NULL, 0, &db->dbi);
@@ -757,7 +783,7 @@ bool DBPrivRead(
     const void *const key,
     const int key_size,
     void *const dest,
-    int dest_size)
+    size_t dest_size)
 {
     assert(db != NULL);
     assert(key_size >= 0);
