@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -25,6 +25,7 @@
 #ifndef CFENGINE_DBM_PRIV_H
 #define CFENGINE_DBM_PRIV_H
 
+#include <dbm_api_types.h>
 /* DBM implementation is supposed to define the following structures and
  * implement the following functions */
 
@@ -57,11 +58,16 @@ bool DBPrivHasKey(DBPriv *db, const void *key, int key_size);
 int DBPrivGetValueSize(DBPriv *db, const void *key, int key_size);
 
 bool DBPrivRead(DBPriv *db, const void *key, int key_size,
-            void *dest, int dest_size);
+            void *dest, size_t dest_size);
 
 bool DBPrivWrite(DBPriv *db, const void *key, int key_size,
              const void *value, int value_size);
 
+bool DBPrivOverwrite(DBPriv *handle,
+                     const char *key, int key_size,
+                     const void *value, size_t value_size,
+                     OverwriteCondition Condition, void *data);
+
 bool DBPrivDelete(DBPriv *db, const void *key, int key_size);
 
 
