@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -27,10 +27,11 @@
 
 #include <platform.h>
 #include <cf3.defs.h>
+#include <pipes.h> // OutputSelect
 
 bool IsExecutable(const char *file);
 bool ShellCommandReturnsZero(const char *command, ShellType shell);
-bool GetExecOutput(const char *command, char **buffer, size_t *buffer_size, ShellType shell);
+bool GetExecOutput(const char *command, char **buffer, size_t *buffer_size, ShellType shell, OutputSelect output_select, int *ret_out);
 void ActAsDaemon();
 void ArgGetExecutableAndArgs(const char *comm, char **exec, char **args);
 char **ArgSplitCommand(const char *comm);
