@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -236,8 +236,8 @@ static PromiseResult ExpandPromiseAndDo(EvalContext *ctx, PromiseIterator *iterc
         /* EVALUATE VARS PROMISES again, allowing redefinition of
          * variables. The theory behind this is that the "sampling rate" of
          * vars promise needs to be double than the rest. */
-        if (strcmp(pexp->parent_promise_type->name, "vars") == 0 ||
-            strcmp(pexp->parent_promise_type->name, "meta") == 0)
+        if (strcmp(PromiseGetPromiseType(pexp), "vars") == 0 ||
+            strcmp(PromiseGetPromiseType(pexp), "meta") == 0)
         {
             if (act_on_promise != &VerifyVarPromise)
             {
@@ -257,8 +257,16 @@ static PromiseResult ExpandPromiseAndDo(EvalContext *ctx, PromiseIterator *iterc
 PromiseResult ExpandPromise(EvalContext *ctx, const Promise *pp,
                             PromiseActuator *act_on_promise, void *param)
 {
+    assert(pp != NULL);
+
     if (!IsDefinedClass(ctx, pp->classes))
     {
+        Log(LOG_LEVEL_DEBUG,
+            "Skipping %s promise expansion with promiser '%s' due to class guard '%s::' (pass %d)",
+            PromiseGetPromiseType(pp),
+            pp->promiser,
+            pp->classes,
+            EvalContextGetPass(ctx));
         return PROMISE_RESULT_SKIPPED;
     }
 
@@ -310,7 +318,7 @@ PromiseResult ExpandPromise(EvalContext *ctx, const Promise *pp,
 /*********************************************************************/
 /*********************************************************************/
 
-Rval ExpandPrivateRval(EvalContext *ctx,
+Rval ExpandPrivateRval(const EvalContext *ctx,
                        const char *ns, const char *scope,
                        const void *rval_item, RvalType rval_type)
 {
@@ -403,7 +411,7 @@ static inline bool VariableDataOrListReference(const char *str)
     return false;
 }
 
-static Rval ExpandListEntry(EvalContext *ctx,
+static Rval ExpandListEntry(const EvalContext *ctx,
                             const char *ns, const char *scope,
                             int expandnaked, Rval entry)
 {
@@ -463,7 +471,7 @@ static Rval ExpandListEntry(EvalContext *ctx,
     return ret;
 }
 
-Rlist *ExpandList(EvalContext *ctx,
+Rlist *ExpandList(const EvalContext *ctx,
                   const char *ns, const char *scope,
                   const Rlist *list, int expandnaked)
 {
@@ -714,16 +722,16 @@ Rval EvaluateFinalRval(EvalContext *ctx, const Policy *policy,
 
 void BundleResolvePromiseType(EvalContext *ctx, const Bundle *bundle, const char *type, PromiseActuator *actuator)
 {
-    for (size_t j = 0; j < SeqLength(bundle->promise_types); j++)
+    for (size_t j = 0; j < SeqLength(bundle->sections); j++)
     {
-        PromiseType *pt = SeqAt(bundle->promise_types, j);
+        BundleSection *section = SeqAt(bundle->sections, j);
 
-        if (strcmp(pt->name, type) == 0)
+        if (strcmp(section->promise_type, type) == 0)
         {
-            EvalContextStackPushPromiseTypeFrame(ctx, pt);
-            for (size_t i = 0; i < SeqLength(pt->promises); i++)
+            EvalContextStackPushBundleSectionFrame(ctx, section);
+            for (size_t i = 0; i < SeqLength(section->promises); i++)
             {
-                Promise *pp = SeqAt(pt->promises, i);
+                Promise *pp = SeqAt(section->promises, i);
                 ExpandPromise(ctx, pp, actuator, NULL);
             }
             EvalContextStackPopFrame(ctx);
@@ -771,10 +779,12 @@ static void RemoveRemotelyInjectedVars(const EvalContext *ctx, const Bundle *bun
         while (var != NULL)
         {
             /* variables are stored together with their original promises (org_pp) */
-            if (var->promise && var->promise->org_pp == pp)
+            const Promise *var_promise = VariableGetPromise(var);
+            const VarRef *var_ref = VariableGetRef(var);
+            if (var_promise && var_promise->org_pp == pp)
             {
                 Log(LOG_LEVEL_ERR, "Ignoring remotely-injected variable '%s'",
-                    var->ref->lval);
+                    var_ref->lval);
                 /* avoid modifications of the variable table being iterated
                  * over and avoid trying to remove the same variable twice */
                 SeqAppendOnce(remove_vars, (void *) var, PointerCmp);
@@ -789,9 +799,10 @@ static void RemoveRemotelyInjectedVars(const EvalContext *ctx, const Bundle *bun
     for (size_t i = 0; i < remove_vars_length; i++)
     {
         Variable *var = (Variable *) SeqAt(remove_vars, i);
-        if (var->ref != NULL)
+        const VarRef *var_ref = VariableGetRef(var);
+        if (var_ref != NULL)
         {
-            EvalContextVariableRemove(ctx, var->ref);
+            EvalContextVariableRemove(ctx, var_ref);
         }
     }
     SeqDestroy(remove_vars);
@@ -933,7 +944,11 @@ static void ResolveControlBody(EvalContext *ctx, GenericAgentConfig *config,
 
             EvalContextVariableRemoveSpecial(ctx, SPECIAL_SCOPE_SYS, "domain");
             EvalContextVariableRemoveSpecial(ctx, SPECIAL_SCOPE_SYS, "fqhost");
-            snprintf(VFQNAME, CF_MAXVARSIZE, "%s.%s", VUQNAME, VDOMAIN);
+
+            // We don't expect hostname or domain name longer than 255,
+            // warnings are printed in sysinfo.c.
+            // Here we support up to 511 bytes, just in case, because we can:
+            snprintf(VFQNAME, CF_MAXVARSIZE, "%511s.%511s", VUQNAME, VDOMAIN);
             EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_SYS, "fqhost",
                                           VFQNAME, CF_DATA_TYPE_STRING,
                                           "inventory,source=agent,attribute_name=Host name");
@@ -947,16 +962,22 @@ static void ResolveControlBody(EvalContext *ctx, GenericAgentConfig *config,
         {
             Log(LOG_LEVEL_VERBOSE, "SET ignore_missing_inputs %s",
                 RvalScalarValue(evaluated_rval));
-            config->ignore_missing_inputs = BooleanFromString(
-                RvalScalarValue(evaluated_rval));
+            if (StringIsBoolean(RvalScalarValue(evaluated_rval)))
+            {
+                config->ignore_missing_inputs = BooleanFromString(
+                    RvalScalarValue(evaluated_rval));
+            }
         }
 
         if (strcmp(lval, CFG_CONTROLBODY[COMMON_CONTROL_IGNORE_MISSING_BUNDLES].lval) == 0)
         {
             Log(LOG_LEVEL_VERBOSE, "SET ignore_missing_bundles %s",
                 RvalScalarValue(evaluated_rval));
-            config->ignore_missing_bundles = BooleanFromString(
-                RvalScalarValue(evaluated_rval));
+            if (StringIsBoolean(RvalScalarValue(evaluated_rval)))
+            {
+                config->ignore_missing_bundles = BooleanFromString(
+                    RvalScalarValue(evaluated_rval));
+            }
         }
 
         if (strcmp(lval, CFG_CONTROLBODY[COMMON_CONTROL_CACHE_SYSTEM_FUNCTIONS].lval) == 0)
