@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -40,6 +40,6 @@ bool CopyRegularFileDiskPerms(const char *source, const char *destination,
                               const int mode);
 bool CopyRegularFileDisk(const char *source, const char *destination);
 bool CopyFilePermissionsDisk(const char *source, const char *destination);
-bool CopyFileExtendedAttributesDisk(const char *source, const char *destination);
+bool CopyFileExtendedAttributesDisk(const char *source, const char *destination, bool *change);
 
 #endif
