@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -33,10 +33,25 @@ bool FileWriteOver(char *filename, char *contents);
 
 bool LoadFileAsItemList(Item **liststart, const char *file, EditDefaults edits);
 
-bool MakeParentDirectory(const char *parentandchild, bool force);
-bool MakeParentDirectory2(char *parentandchild, int force, bool enforce_promise);
+/**
+ * @see     MakeParentDirectoryForPromise()
+ */
+bool MakeParentDirectory(const char *parentandchild, bool force, bool *created);
 
-void RotateFiles(char *name, int number);
+/**
+ * Create an internal directory (never in the changes chroot).
+ */
+bool MakeParentInternalDirectory(const char *parentandchild, bool force, bool *created);
+
+/**
+ * @warning This function will not behave right on Windows if the path
+ *          contains double (back)slashes!
+ **/
+bool MakeParentDirectoryForPromise(EvalContext *ctx, const Promise *pp, const Attributes *attr,
+                                   PromiseResult *result, const char *parentandchild,
+                                   bool force, bool *created);
+
+void RotateFiles(const char *name, int number);
 void CreateEmptyFile(char *name);
 
 
