@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -563,7 +563,7 @@ bool CompressPath(char *dest, size_t dest_size, const char *src)
 
     rootlen = RootDirLength(src);
 
-    if(rootlen >= dest_size)
+    if((size_t) rootlen >= dest_size)
     {
         Log(LOG_LEVEL_ERR,
             "Internal limit reached in CompressPath(),"
