@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -68,6 +68,7 @@ typedef struct
     // agent state
     bool ignore_missing_bundles;
     bool ignore_missing_inputs;
+    bool ignore_preferred_augments; // --ignore-preferred-augments
 
     struct
     {
@@ -106,7 +107,7 @@ typedef struct
 ENTERPRISE_VOID_FUNC_2ARG_DECLARE(void, GenericAgentSetDefaultDigest, HashMethod *, digest, int *, digest_len);
 const char *GenericAgentResolveInputPath(const GenericAgentConfig *config, const char *input_file);
 void MarkAsPolicyServer(EvalContext *ctx);
-void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config);
+void GenericAgentDiscoverContext(EvalContext *ctx, GenericAgentConfig *config, const char *program_name);
 bool GenericAgentCheckPolicy(GenericAgentConfig *config, bool force_validation, bool write_validated_file);
 
 ENTERPRISE_VOID_FUNC_1ARG_DECLARE(void, GenericAgentAddEditionClasses, EvalContext *, ctx);
@@ -124,6 +125,7 @@ Seq *ControlBodyConstraints(const Policy *policy, AgentType agent);
 void SetFacility(const char *retval);
 void CheckBundleParameters(char *scope, Rlist *args);
 void WritePID(char *filename);
+pid_t ReadPID(char *filename);
 
 bool GenericAgentConfigParseArguments(GenericAgentConfig *config, int argc, char **argv);
 bool GenericAgentConfigParseWarningOptions(GenericAgentConfig *config, const char *warning_options);
