@@ -1,16 +1,16 @@
 /*
-   Copyright 2018 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
-   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
+  This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
-   This program is free software; you can redistribute it and/or modify it
-   under the terms of the GNU General Public License as published by the
-   Free Software Foundation; version 3.
+  This program is free software; you can redistribute it and/or modify it
+  under the terms of the GNU General Public License as published by the
+  Free Software Foundation; version 3.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+  This program is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
@@ -140,9 +140,9 @@ void PrintItemList(const Item *list, Writer *w)
 
 /*********************************************************************/
 
-int ItemListSize(const Item *list)
+size_t ItemListSize(const Item *list)
 {
-    int size = 0;
+    size_t size = 0;
     const Item *ip = list;
     CYCLE_DECLARE(ip, slow, toggle);
 
@@ -447,7 +447,7 @@ void PrependItemList(Item **liststart, const char *itemstring)
 
 /*********************************************************************/
 
-int ListLen(const Item *list)
+size_t ListLen(const Item *list)
 {
     int count = 0;
     const Item *ip = list;
@@ -642,7 +642,7 @@ char *ItemList2CSV(const Item *list)
 {
     /* After each entry, we need space for either a ',' (before the
      * next entry) or a final '\0'. */
-    int s_size = ItemListSize(list) + ListLen(list);
+    size_t s_size = ItemListSize(list) + ListLen(list);
     if (s_size == 0)
     {
         s_size = 1;
@@ -777,7 +777,7 @@ void SetItemListCounter(Item *list, const char *item, int value)
 
 /*********************************************************************/
 
-int IsMatchItemIn(const Item *list, const char *item)
+bool IsMatchItemIn(const Item *list, const char *item)
 /* Solve for possible regex/fuzzy models unified */
 {
     if (item == NULL || item[0] == '\0')
@@ -868,7 +868,7 @@ void DeleteItem(Item **liststart, Item *item)
 
 /*********************************************************************/
 
-int DeleteItemGeneral(Item **list, const char *string, ItemMatchType type)
+bool DeleteItemGeneral(Item **list, const char *string, ItemMatchType type)
 {
     if (list == NULL)
     {
@@ -896,25 +896,29 @@ int DeleteItemGeneral(Item **list, const char *string, ItemMatchType type)
             switch (type)
             {
             case ITEM_MATCH_TYPE_LITERAL_START_NOT:
-                flip = true; /* and fall through */
+                flip = true;
+                /* fall through */
             case ITEM_MATCH_TYPE_LITERAL_START:
                 match = (strncmp(ip->name, string, strlen(string)) == 0);
                 break;
 
             case ITEM_MATCH_TYPE_LITERAL_COMPLETE_NOT:
-                flip = true; /* and fall through */
+                flip = true;
+                /* fall through */
             case ITEM_MATCH_TYPE_LITERAL_COMPLETE:
                 match = (strcmp(ip->name, string) == 0);
                 break;
 
             case ITEM_MATCH_TYPE_LITERAL_SOMEWHERE_NOT:
-                flip = true; /* and fall through */
+                flip = true;
+                /* fall through */
             case ITEM_MATCH_TYPE_LITERAL_SOMEWHERE:
                 match = (strstr(ip->name, string) != NULL);
                 break;
 
             case ITEM_MATCH_TYPE_REGEX_COMPLETE_NOT:
-                flip = true; /* and fall through */
+                flip = true;
+                /* fall through */
             case ITEM_MATCH_TYPE_REGEX_COMPLETE:
                 match = StringMatchFullWithPrecompiledRegex(rx, ip->name);
                 break;
@@ -964,49 +968,49 @@ int DeleteItemGeneral(Item **list, const char *string, ItemMatchType type)
 
 /*********************************************************************/
 
-int DeleteItemStarting(Item **list, const char *string)       /* delete 1st item starting with string */
+bool DeleteItemStarting(Item **list, const char *string)       /* delete 1st item starting with string */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_LITERAL_START);
 }
 
 /*********************************************************************/
 
-int DeleteItemNotStarting(Item **list, const char *string)    /* delete 1st item starting with string */
+bool DeleteItemNotStarting(Item **list, const char *string)    /* delete 1st item starting with string */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_LITERAL_START_NOT);
 }
 
 /*********************************************************************/
 
-int DeleteItemLiteral(Item **list, const char *string)  /* delete 1st item which is string */
+bool DeleteItemLiteral(Item **list, const char *string)  /* delete 1st item which is string */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_LITERAL_COMPLETE);
 }
 
 /*********************************************************************/
 
-int DeleteItemMatching(Item **list, const char *string)       /* delete 1st item fully matching regex */
+bool DeleteItemMatching(Item **list, const char *string)       /* delete 1st item fully matching regex */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_REGEX_COMPLETE);
 }
 
 /*********************************************************************/
 
-int DeleteItemNotMatching(Item **list, const char *string)    /* delete 1st item fully matching regex */
+bool DeleteItemNotMatching(Item **list, const char *string)    /* delete 1st item fully matching regex */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_REGEX_COMPLETE_NOT);
 }
 
 /*********************************************************************/
 
-int DeleteItemContaining(Item **list, const char *string)     /* delete first item containing string */
+bool DeleteItemContaining(Item **list, const char *string)     /* delete first item containing string */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_LITERAL_SOMEWHERE);
 }
 
 /*********************************************************************/
 
-int DeleteItemNotContaining(Item **list, const char *string)  /* delete first item containing string */
+bool DeleteItemNotContaining(Item **list, const char *string)  /* delete first item containing string */
 {
     return DeleteItemGeneral(list, string, ITEM_MATCH_TYPE_LITERAL_SOMEWHERE_NOT);
 }
@@ -1016,11 +1020,11 @@ int DeleteItemNotContaining(Item **list, const char *string)  /* delete first it
 bool RawSaveItemList(const Item *liststart, const char *filename, NewLineMode new_line_mode)
 {
     char new[CF_BUFSIZE];
-    strcpy(new, filename);
-    strcat(new, CF_EDITED);
+    snprintf(new, sizeof(new), "%s%s", filename, CF_EDITED);
     unlink(new);                /* Just in case of races */
 
-    FILE *fp = safe_fopen(new, (new_line_mode == NewLineMode_Native) ? "wt" : "w");
+    FILE *fp = safe_fopen(
+        new, (new_line_mode == NewLineMode_Native) ? "wt" : "w");
     if (fp == NULL)
     {
         Log(LOG_LEVEL_ERR, "Couldn't write file '%s'. (fopen: %s)", new, GetErrorStr());
