@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -333,8 +333,8 @@ static void MangleVarRefString(char *ref_str, size_t len)
     {
         *ns      = CF_MANGLED_NS;
         ref_str2 =  ns + 1;
+        assert(upto >= (ns + 1 - ref_str));
         upto    -= (ns + 1 - ref_str);
-        assert(upto >= 0);
     }
 
     bool mangled_scope = false;
