@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -83,9 +83,10 @@ int RemovePublicKey(const char *id)
 
         if (c && c[strlen(suffix)] == '\0')     /* dirp->d_name ends with suffix */
         {
-            char keyfilename[CF_BUFSIZE];
+            char keyfilename[sizeof(keysdir) + sizeof(dirp->d_name)];
+            // both sizeof's include NUL, so one of those is for the '/'
 
-            snprintf(keyfilename, CF_BUFSIZE, "%s/%s", keysdir, dirp->d_name);
+            snprintf(keyfilename, sizeof(keyfilename), "%s/%s", keysdir, dirp->d_name);
             MapName(keyfilename);
 
             if (unlink(keyfilename) < 0)
