@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -87,7 +87,7 @@ void LastSaw1(const char *ipaddress, const char *hashstr,
     UpdateLastSawHost(hashstr, mapip, role == LAST_SEEN_ROLE_ACCEPT, time(NULL));
 }
 
-void LastSaw(const char *ipaddress, const char *digest, LastSeenRole role)
+void LastSaw(const char *ipaddress, const unsigned char *digest, LastSeenRole role)
 {
     char databuf[CF_HOSTKEY_STRING_SIZE];
 
@@ -157,7 +157,7 @@ void UpdateLastSawHost(const char *hostkey, const char *address,
 static bool Address2HostkeyInDB(DBHandle *db, const char *address, char *result, size_t result_size)
 {
     char address_key[CF_BUFSIZE];
-    char hostkey[CF_BUFSIZE];
+    char hostkey[CF_HOSTKEY_STRING_SIZE];
 
     /* Address key: "a" + address */
     snprintf(address_key, CF_BUFSIZE, "a%s", address);
@@ -171,11 +171,11 @@ static bool Address2HostkeyInDB(DBHandle *db, const char *address, char *result,
     /* Check for inconsistencies. Return success even if db is found
      * inconsistent, since the reverse entry is already found. */
 
-    char hostkey_key[CF_BUFSIZE];
+    char hostkey_key[1 + CF_HOSTKEY_STRING_SIZE];
     char back_address[CF_BUFSIZE];
 
     /* Hostkey key: "k" + hostkey */
-    snprintf(hostkey_key, CF_BUFSIZE, "k%s", hostkey);
+    snprintf(hostkey_key, sizeof(hostkey_key), "k%s", hostkey);
 
     if (!ReadDB(db, hostkey_key, &back_address, sizeof(back_address)))
     {
@@ -241,6 +241,37 @@ bool Address2Hostkey(char *dst, size_t dst_size, const char *address)
     return retval;
 }
 
+char *HostkeyToAddress(const char *hostkey)
+{
+    DBHandle *db;
+    if (OpenDB(&db, dbid_lastseen))
+    {
+        char hostkey_key[CF_HOSTKEY_STRING_SIZE + 1];
+        char address[CF_BUFSIZE];
+
+        /* Hostkey key: "k" + hostkey */
+        snprintf(hostkey_key, sizeof(hostkey_key), "k%s", hostkey);
+
+        if (ReadDB(db, hostkey_key, &address, sizeof(address)))
+        {
+            CloseDB(db);
+            Log(LOG_LEVEL_DEBUG, "Found hostkey '%s' in lastseen LMDB", hostkey);
+            return xstrdup(address);
+        }
+        else
+        {
+            CloseDB(db);
+            Log(LOG_LEVEL_VERBOSE, "Could not find hostkey '%s' in lastseen LMDB", hostkey);
+            return NULL;
+        }
+    }
+    else
+    {
+        Log(LOG_LEVEL_ERR, "Failed to open lastseen DB");
+        return NULL;
+    }
+}
+
 /**
  * @brief detects whether input is a host/ip name or a key digest
  *
@@ -595,7 +626,7 @@ bool ScanLastSeenQuality(LastSeenQualityCallback callback, void *ctx)
 
         SeqAppend(hostkeys, xstrdup(key + 1));
     }
-    for (int i = 0; i < SeqLength(hostkeys); ++i)
+    for (size_t i = 0; i < SeqLength(hostkeys); ++i)
     {
         const char *hostkey = SeqAt(hostkeys, i);
 
@@ -732,4 +763,3 @@ int RemoveKeysFromLastSeen(const char *input, bool must_be_coherent,
 
     return 0;
 }
-
