@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -228,8 +228,8 @@ static void ShowContext(EvalContext *ctx)
         ClassTableIteratorDestroy(iter);
     }
 
-    SeqSort(soft_contexts, (SeqItemComparator)strcmp, NULL);
-    SeqSort(hard_contexts, (SeqItemComparator)strcmp, NULL);
+    SeqSort(soft_contexts, StrCmpWrapper, NULL);
+    SeqSort(hard_contexts, StrCmpWrapper, NULL);
 
     Log(LOG_LEVEL_VERBOSE, "----------------------------------------------------------------");
 
@@ -281,10 +281,10 @@ static void RenameMainBundle(EvalContext *ctx, Policy *policy)
     for (int i = 0; i < length; ++i)
     {
         Bundle *const bundle = SeqAt(bundles, i);
-        if (StringSafeEqual(bundle->name, "__main__"))
+        if (StringEqual(bundle->name, "__main__"))
         {
             char *abspath = GetRealPath(bundle->source_path);
-            if (StringSafeEqual(abspath, entry_point))
+            if (StringEqual(abspath, entry_point))
             {
                 Log(LOG_LEVEL_VERBOSE,
                     "Redefining __main__ bundle from file %s to be main",
@@ -531,8 +531,7 @@ Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
             {
                 Log(LOG_LEVEL_VERBOSE,
                     "Running full policy integrity checks");
-                PolicyCheckRunnable(ctx, policy, errors,
-                                    config->ignore_missing_bundles);
+                PolicyCheckRunnable(ctx, policy, errors);
             }
         }
 
@@ -544,6 +543,7 @@ Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
                 PolicyErrorWrite(writer, errors->data[i]);
             }
             WriterClose(writer);
+            SeqDestroy(errors);
             DoCleanupAndExit(EXIT_FAILURE); // TODO: do not exit
         }
 
@@ -575,10 +575,10 @@ Policy *LoadPolicy(EvalContext *ctx, GenericAgentConfig *config)
             Bundle *bp = SeqAt(policy->bundles, i);
             EvalContextStackPushBundleFrame(ctx, bp, NULL, false);
 
-            for (size_t j = 0; j < SeqLength(bp->promise_types); j++)
+            for (size_t j = 0; j < SeqLength(bp->sections); j++)
             {
-                PromiseType *sp = SeqAt(bp->promise_types, j);
-                EvalContextStackPushPromiseTypeFrame(ctx, sp);
+                BundleSection *sp = SeqAt(bp->sections, j);
+                EvalContextStackPushBundleSectionFrame(ctx, sp);
 
                 for (size_t ppi = 0; ppi < SeqLength(sp->promises); ppi++)
                 {
