@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -375,11 +375,11 @@ static bool NoOrObsoleteLock(LockData *entry, ARG_UNUSED size_t entry_size, size
     }
 
     time_t now = time(NULL);
-    if ((now - entry->time) <= *max_old)
+    if ((now - entry->time) <= (time_t) *max_old)
     {
         Log(LOG_LEVEL_DEBUG, "Giving time to process '%d' (holding lock for %ld s)", entry->pid, (now - entry->time));
     }
-    return ((now - entry->time) > *max_old);
+    return ((now - entry->time) > (time_t) *max_old);
 }
 
 void WaitForCriticalSection(const char *section_id)
@@ -413,7 +413,7 @@ void WaitForCriticalSection(const char *section_id)
 
     Log(LOG_LEVEL_DEBUG, "Acquiring critical section lock '%s'", section_id);
     bool got_lock = false;
-    while (!got_lock && ((time(NULL) - started) <= max_old))
+    while (!got_lock && ((time(NULL) - started) <= (time_t) max_old))
     {
         entry.time = time(NULL);
         got_lock = OverwriteDB(dbp, section_id, &entry, sizeof(entry),
@@ -507,7 +507,7 @@ static void RegisterLockCleanup(void)
  */
 static void PromiseTypeString(char *dst, size_t dst_size, const Promise *pp)
 {
-    char *sp       = pp->parent_promise_type->name;
+    const char *sp = PromiseGetPromiseType(pp);
     size_t sp_len  = strlen(sp);
 
     dst[0]         = '\0';
@@ -648,7 +648,7 @@ void PromiseRuntimeHash(const Promise *pp, const char *salt,
     Rlist *rp;
     FnCall *fp;
 
-    char *noRvalHash[] = { "mtime", "atime", "ctime", "stime_range", "ttime_range", "log_string", NULL };
+    char *noRvalHash[] = { "mtime", "atime", "ctime", "stime_range", "ttime_range", "log_string", "template_data", NULL };
     int doHash;
 
     md = HashDigestFromId(type);
@@ -680,20 +680,20 @@ void PromiseRuntimeHash(const Promise *pp, const char *salt,
         EVP_DigestUpdate(context, pp->comment, strlen(pp->comment));
     }
 
-    if (pp->parent_promise_type && pp->parent_promise_type->parent_bundle)
+    if (pp->parent_section && pp->parent_section->parent_bundle)
     {
-        if (pp->parent_promise_type->parent_bundle->ns)
+        if (pp->parent_section->parent_bundle->ns)
         {
             EVP_DigestUpdate(context,
-                             pp->parent_promise_type->parent_bundle->ns,
-                             strlen(pp->parent_promise_type->parent_bundle->ns));
+                             pp->parent_section->parent_bundle->ns,
+                             strlen(pp->parent_section->parent_bundle->ns));
         }
 
-        if (pp->parent_promise_type->parent_bundle->name)
+        if (pp->parent_section->parent_bundle->name)
         {
             EVP_DigestUpdate(context,
-                             pp->parent_promise_type->parent_bundle->name,
-                             strlen(pp->parent_promise_type->parent_bundle->name));
+                             pp->parent_section->parent_bundle->name,
+                             strlen(pp->parent_section->parent_bundle->name));
         }
     }
 
@@ -742,6 +742,16 @@ void PromiseRuntimeHash(const Promise *pp, const char *salt,
                 }
                 break;
 
+            case RVAL_TYPE_CONTAINER:
+                {
+                    const JsonElement *rval_json = RvalContainerValue(cp->rval);
+                    Writer *writer = StringWriter();
+                    JsonWriteCompact(writer, rval_json); /* sorts elements and produces canonical form */
+                    EVP_DigestUpdate(context, StringWriterData(writer), StringWriterLength(writer));
+                    WriterClose(writer);
+                    break;
+                }
+
             case RVAL_TYPE_FNCALL:
 
                 /* Body or bundle */
