@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -97,10 +97,10 @@ static bool StringAppendAbbreviatedPromise(char *dst, const char *src, size_t n,
         /* `src` contains a newline: abbreviate it by taking the first and last few characters */
         static const char sep[] = "...";
         char abbr[sizeof(sep) + 2 * max_fragment];
-        const int head = (nl > src + max_fragment) ? max_fragment : (nl - src);
+        const size_t head = (nl > src + max_fragment) ? max_fragment : (size_t) (nl - src);
         const char * last_line = strrchr(src, '\n') + 1;
         assert(last_line); /* not max_fragmentULL, we know we have at least one '\n' */
-        const int tail = strlen(last_line);
+        const size_t tail = strlen(last_line);
         if (tail > max_fragment)
         {
             last_line += tail - max_fragment;
@@ -148,11 +148,11 @@ void PromiseBanner(EvalContext *ctx, const Promise *pp)
 
     if (strlen(handle) > 0)
     {
-        Log(LOG_LEVEL_VERBOSE, "P: BEGIN promise '%s' of type \"%s\" (pass %d)", handle, pp->parent_promise_type->name, EvalContextGetPass(ctx));
+        Log(LOG_LEVEL_VERBOSE, "P: BEGIN promise '%s' of type \"%s\" (pass %d)", handle, PromiseGetPromiseType(pp), EvalContextGetPass(ctx));
     }
     else
     {
-        Log(LOG_LEVEL_VERBOSE, "P: BEGIN un-named promise of type \"%s\" (pass %d)", pp->parent_promise_type->name, EvalContextGetPass(ctx));
+        Log(LOG_LEVEL_VERBOSE, "P: BEGIN un-named promise of type \"%s\" (pass %d)", PromiseGetPromiseType(pp), EvalContextGetPass(ctx));
     }
 
     const size_t n = 2*CF_MAXFRAGMENT + 3;
