@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -34,6 +34,10 @@
 
 #include <audit.h>
 
+#ifdef __ANDROID__
+#  include <file_lib.h>    /* File_Copy() */
+#endif
+
 static char *cf_format_strtimestamp(struct tm *tm, char *buf);
 
 /*********************************************************/
@@ -193,6 +197,8 @@ bool LinkOrCopy(const char *from, const char *to, int sym)
         return false;
     }
 
+#elif __ANDROID__    /* link() not supported on ANDROID platform */
+    return File_Copy(from, to);
 #else /* !__MINGW32__ */
 
     if (sym)
