@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -132,7 +132,7 @@ Rlist *PipeReadData(const IOData *io, int pipe_timeout_secs, int pipe_terminatio
     return response_lines;
 }
 
-int PipeWrite(IOData *io, const char *data)
+ssize_t PipeWrite(IOData *io, const char *data)
 {
     /* If there is nothing to write close writing end of pipe. */
     if (data == NULL || strlen(data) == 0)
@@ -257,3 +257,22 @@ int PipeReadWriteData(const char *base_cmd, const char *args, const char *reques
     *response = res;
     return 0;
 }
+
+IOData cf_popen_full_duplex_streams(
+    const char *command, bool capture_stderr, bool require_full_path)
+{
+    IOData ret = cf_popen_full_duplex(
+        command, capture_stderr, require_full_path);
+
+    // On windows, these streams are already set up correctly:
+    if (ret.read_stream == NULL)
+    {
+        ret.read_stream = fdopen(ret.read_fd, "r");
+    }
+    if (ret.write_stream == NULL)
+    {
+        ret.write_stream = fdopen(ret.write_fd, "w");
+    }
+
+    return ret;
+}
