@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -92,6 +92,46 @@ static void ChildrenFDClose()
 }
 #endif
 
+static void ChildOutputSelectDupClose(int pd[2], OutputSelect output_select)
+{
+    close(pd[0]); // Don't need output from parent
+
+    if (pd[1] != 1) // TODO: When is pd[1] == 1 ???
+    {
+        if ((output_select == OUTPUT_SELECT_BOTH)
+            || (output_select == OUTPUT_SELECT_STDOUT))
+        {
+            // close our(child) stdout(1) and open (pd[1]) as stdout
+            dup2(pd[1], 1);
+            // Subsequent stdout output will go to parent (pd[1])
+        }
+        else
+        {
+            // Close / discard stdout
+            int nullfd = open(NULLFILE, O_WRONLY);
+            dup2(nullfd, 1);
+            close(nullfd);
+        }
+
+        if ((output_select == OUTPUT_SELECT_BOTH)
+            || (output_select == OUTPUT_SELECT_STDERR))
+        {
+            // close our(child) stderr(2) and open (pd[1]) as stderr
+            dup2(pd[1], 2);
+            // Subsequent stderr output will go to parent (pd[1])
+        }
+        else
+        {
+            // Close / discard stderr
+            int nullfd = open(NULLFILE, O_WRONLY);
+            dup2(nullfd, 2);
+            close(nullfd);
+        }
+
+        close(pd[1]);
+    }
+}
+
 /*****************************************************************************/
 
 static void ChildrenFDSet(int fd, pid_t pid)
@@ -256,7 +296,7 @@ IOData cf_popen_full_duplex(const char *command, bool capture_stderr, bool requi
     {
         Log(LOG_LEVEL_ERR, "Couldn't fork child process: %s", GetErrorStr());
         ArgFree(argv);
-        return (IOData) {-1, -1};
+        return (IOData) {-1, -1, NULL, NULL};
     }
 
     else if (pid > 0) // parent
@@ -267,6 +307,8 @@ IOData cf_popen_full_duplex(const char *command, bool capture_stderr, bool requi
         IOData io_desc;
         io_desc.write_fd = parent_pipe[WRITE];
         io_desc.read_fd = child_pipe[READ];
+        io_desc.read_stream = NULL;
+        io_desc.write_stream = NULL;
 
         ChildrenFDSet(parent_pipe[WRITE], pid);
         ChildrenFDSet(child_pipe[READ], pid);
@@ -329,7 +371,7 @@ IOData cf_popen_full_duplex(const char *command, bool capture_stderr, bool requi
     }
 }
 
-FILE *cf_popen(const char *command, const char *type, bool capture_stderr)
+FILE *cf_popen_select(const char *command, const char *type, OutputSelect output_select)
 {
     int pd[2];
     pid_t pid;
@@ -351,27 +393,7 @@ FILE *cf_popen(const char *command, const char *type, bool capture_stderr)
         switch (*type)
         {
         case 'r':
-
-            close(pd[0]);       /* Don't need output from parent */
-
-            if (pd[1] != 1)
-            {
-                dup2(pd[1], 1); /* Attach pp=pd[1] to our stdout */
-
-                if (capture_stderr)
-                {
-                    dup2(pd[1], 2); /* Merge stdout/stderr */
-                }
-                else
-                {
-                    int nullfd = open(NULLFILE, O_WRONLY);
-                    dup2(nullfd, 2);
-                    close(nullfd);
-                }
-
-                close(pd[1]);
-            }
-
+            ChildOutputSelectDupClose(pd, output_select);
             break;
 
         case 'w':
@@ -431,6 +453,14 @@ FILE *cf_popen(const char *command, const char *type, bool capture_stderr)
     return NULL;
 }
 
+FILE *cf_popen(const char *command, const char *type, bool capture_stderr)
+{
+    return cf_popen_select(
+        command,
+        type,
+        capture_stderr ? OUTPUT_SELECT_BOTH : OUTPUT_SELECT_STDOUT);
+}
+
 /*****************************************************************************/
 
 /*
@@ -557,7 +587,7 @@ FILE *cf_popensetuid(const char *command, const char *type,
 /* Shell versions of commands - not recommended for security reasons         */
 /*****************************************************************************/
 
-FILE *cf_popen_sh(const char *command, const char *type)
+FILE *cf_popen_sh_select(const char *command, const char *type, OutputSelect output_select)
 {
     int pd[2];
     pid_t pid;
@@ -576,15 +606,7 @@ FILE *cf_popen_sh(const char *command, const char *type)
         switch (*type)
         {
         case 'r':
-
-            close(pd[0]);       /* Don't need output from parent */
-
-            if (pd[1] != 1)
-            {
-                dup2(pd[1], 1); /* Attach pp=pd[1] to our stdout */
-                dup2(pd[1], 2); /* Merge stdout/stderr */
-                close(pd[1]);
-            }
+            ChildOutputSelectDupClose(pd, output_select);
 
             break;
 
@@ -640,6 +662,11 @@ FILE *cf_popen_sh(const char *command, const char *type)
     return NULL;
 }
 
+FILE *cf_popen_sh(const char *command, const char *type)
+{
+    return cf_popen_sh_select(command, type, OUTPUT_SELECT_BOTH);
+}
+
 /******************************************************************************/
 
 /*
@@ -879,17 +906,32 @@ int cf_pclose_full_duplex_side(int fd)
 /* We are assuming that read_fd part will be always open at this point. */
 int cf_pclose_full_duplex(IOData *data)
 {
+    assert(data != NULL);
     ThreadLock(cft_count);
 
     if (CHILDREN == NULL)
     {
         ThreadUnlock(cft_count);
-        if (data->read_fd >= 0)
+        if (data->read_stream != NULL)
+        {
+            // fclose closes the underlying fd / socket,
+            // but we need the fd for handling of processes below.
+            assert(data->read_fd >= 0);
+            fclose(data->read_stream);
+        }
+        else if (data->read_fd >= 0)
         {
             close(data->read_fd);
         }
 
-        if (data->write_fd >= 0)
+        if (data->write_stream != NULL)
+        {
+            // fclose closes the underlying fd / socket,
+            // but we need the fd for handling of processes below.
+            assert(data->write_fd >= 0);
+            fclose(data->write_stream);
+        }
+        else if (data->write_fd >= 0)
         {
             close(data->write_fd);
         }
@@ -919,7 +961,45 @@ int cf_pclose_full_duplex(IOData *data)
         ThreadUnlock(cft_count);
     }
 
-    if (close(data->read_fd) != 0 || (data->write_fd >= 0 && close(data->write_fd) != 0) || pid == 0)
+    if (data->read_stream != NULL)
+    {
+        // Stream is open, fclose it
+        if (fclose(data->read_stream) != 0)
+        {
+            return -1;
+        }
+    }
+    else
+    {
+        // No stream, just close fd
+        if (close(data->read_fd) != 0)
+        {
+            return -1;
+        }
+    }
+
+    if (data->write_fd >= 0)
+    {
+        // write fd needs to be closed
+        if (data->write_stream != NULL)
+        {
+            // Stream is open, fclose it
+            if (fclose(data->write_stream) != 0)
+            {
+                return -1;
+            }
+        }
+        else
+        {
+            // No stream, close fd directly
+            if (close(data->write_fd) != 0)
+            {
+                return -1;
+            }
+        }
+    }
+
+    if (pid == 0)
     {
         return -1;
     }
