@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -37,7 +37,7 @@ typedef enum
     POLICY_ELEMENT_TYPE_POLICY,
     POLICY_ELEMENT_TYPE_BUNDLE,
     POLICY_ELEMENT_TYPE_BODY,
-    POLICY_ELEMENT_TYPE_PROMISE_TYPE,
+    POLICY_ELEMENT_TYPE_BUNDLE_SECTION,
     POLICY_ELEMENT_TYPE_PROMISE,
     POLICY_ELEMENT_TYPE_CONSTRAINT
 } PolicyElementType;
@@ -55,6 +55,7 @@ struct Policy_
 
     Seq *bundles;
     Seq *bodies;
+    Seq *custom_promise_types;
     StringMap *policy_files_hashes;
 };
 
@@ -75,7 +76,8 @@ struct Bundle_
     char *ns;
     Rlist *args;
 
-    Seq *promise_types;
+    Seq *sections;
+    Seq *custom_sections;
 
     char *source_path;
     SourceOffset offset;
@@ -96,11 +98,11 @@ struct Body_
     SourceOffset offset;
 };
 
-struct PromiseType_
+struct BundleSection_
 {
     Bundle *parent_bundle;
 
-    char *name;
+    char *promise_type;
     Seq *promises;
 
     SourceOffset offset;
@@ -108,7 +110,7 @@ struct PromiseType_
 
 struct Promise_
 {
-    PromiseType *parent_promise_type;
+    BundleSection *parent_section;
 
     char *classes;
     char *comment;
@@ -160,10 +162,11 @@ void PolicyErrorDestroy(PolicyError *error);
 void PolicyErrorWrite(Writer *writer, const PolicyError *error);
 
 bool PolicyCheckPartial(const Policy *policy, Seq *errors);
-bool PolicyCheckRunnable(const EvalContext *ctx, const Policy *policy, Seq *errors, bool ignore_missing_bundles);
+bool PolicyCheckRunnable(const EvalContext *ctx, const Policy *policy, Seq *errors);
 
 Bundle *PolicyAppendBundle(Policy *policy, const char *ns, const char *name, const char *type, const Rlist *args, const char *source_path);
 Body *PolicyAppendBody(Policy *policy, const char *ns, const char *name, const char *type, Rlist *args, const char *source_path);
+Body *PolicyAppendPromiseBlock(Policy *policy, const char *ns, const char *name, const char *type, Rlist *args, const char *source_path);
 
 JsonElement *PolicyToJson(const Policy *policy);
 JsonElement *BundleToJson(const Bundle *bundle);
@@ -171,8 +174,8 @@ JsonElement *BodyToJson(const Body *body);
 Policy *PolicyFromJson(JsonElement *json_policy);
 void PolicyToString(const Policy *policy, Writer *writer);
 
-PromiseType *BundleAppendPromiseType(Bundle *bundle, const char *name);
-const PromiseType *BundleGetPromiseType(const Bundle *bp, const char *name);
+BundleSection *BundleAppendSection(Bundle *bundle, const char *promise_type);
+const BundleSection *BundleGetSection(const Bundle *bp, const char *promise_type);
 
 Constraint *BodyAppendConstraint(Body *body, const char *lval, Rval rval, const char *classes, bool references_body);
 Seq *BodyGetConstraint(Body *body, const char *lval);
@@ -180,8 +183,8 @@ bool BodyHasConstraint(const Body *body, const char *lval);
 
 const char *ConstraintGetNamespace(const Constraint *cp);
 
-Promise *PromiseTypeAppendPromise(PromiseType *type, const char *promiser, Rval promisee, const char *classes, const char *varclasses);
-void PromiseTypeDestroy(PromiseType *promise_type);
+Promise *BundleSectionAppendPromise(BundleSection *section, const char *promiser, Rval promisee, const char *classes, const char *varclasses);
+void BundleSectionDestroy(BundleSection *section);
 
 void PromiseDestroy(Promise *pp);
 
@@ -191,6 +194,12 @@ const char *PromiseGetNamespace(const Promise *pp);
 const Bundle *PromiseGetBundle(const Promise *pp);
 const Policy *PromiseGetPolicy(const Promise *pp);
 
+static inline const char *PromiseGetPromiseType(const Promise *pp)
+{
+    assert(pp != NULL);
+    return pp->parent_section->promise_type;
+}
+
 void PromisePath(Writer *w, const Promise *pp);
 const char *PromiseGetHandle(const Promise *pp);
 int PromiseGetConstraintAsInt(const EvalContext *ctx, const char *lval, const Promise *pp);
@@ -200,6 +209,8 @@ uid_t PromiseGetConstraintAsUid(const EvalContext *ctx, const char *lval, const
 gid_t PromiseGetConstraintAsGid(const EvalContext *ctx, char *lval, const Promise *pp);
 Rlist *PromiseGetConstraintAsList(const EvalContext *ctx, const char *lval, const Promise *pp);
 int PromiseGetConstraintAsBoolean(const EvalContext *ctx, const char *lval, const Promise *list);
+int PromiseGetConstraintAsBooleanWithDefault(const EvalContext *ctx, const char *lval, const Promise *pp,
+                                             int default_val, bool with_warning);
 Constraint *PromiseGetConstraintWithType(const Promise *promise, const char *lval, RvalType type);
 Constraint *PromiseGetImmediateConstraint(const Promise *promise, const char *lval);
 void *PromiseGetConstraintAsRval(const Promise *promise, const char *lval, RvalType type);
@@ -221,4 +232,6 @@ char *QualifiedNameScopeComponent(const char *qualified_name);
 bool BundleTypeCheck(const char *name);
 Rval DefaultBundleConstraint(const Promise *pp, char *promisetype);
 
+bool PolicyHasCustomPromiseType(const Policy *policy, const char *name);
+
 #endif
