@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -594,7 +594,7 @@ static bool SelectProcRegexMatch(const char *name1, const char *name2,
         }
         else
         {
-            int s, e;
+            size_t s, e;
             return StringMatch(regex, line[i], &s, &e);
         }
     }
@@ -677,7 +677,12 @@ static bool SplitProcLine(const char *line,
         return false;
     }
 
-    size_t linelen = strlen(line);
+    const size_t checklen = strlen(line);
+    if (checklen > INT_MAX) {
+        Log(LOG_LEVEL_ERR, "Proc line too long (%zu > %d)", checklen, INT_MAX);
+        return false;
+    }
+    int linelen = (int) checklen;
 
     if (LogGetGlobalLevel() >= LOG_LEVEL_DEBUG)
     {
