@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -204,8 +204,8 @@ static void AppendExpandedBodies(EvalContext *ctx, Promise *pcopy,
  */
 Promise *DeRefCopyPromise(EvalContext *ctx, const Promise *pp)
 {
-    Log(LOG_LEVEL_DEBUG, "DeRefCopyPromise(): "
-        "promiser:'%s'",
+    Log(LOG_LEVEL_DEBUG,
+        "DeRefCopyPromise(): promiser:'%s'",
         SAFENULL(pp->promiser));
 
     Promise *pcopy = xcalloc(1, sizeof(Promise));
@@ -237,7 +237,7 @@ Promise *DeRefCopyPromise(EvalContext *ctx, const Promise *pp)
 
     assert(pp->classes);
     pcopy->classes             = xstrdup(pp->classes);
-    pcopy->parent_promise_type = pp->parent_promise_type;
+    pcopy->parent_section      = pp->parent_section;
     pcopy->offset.line         = pp->offset.line;
     pcopy->comment             = pp->comment ? xstrdup(pp->comment) : NULL;
     pcopy->conlist             = SeqNew(10, ConstraintDestroy);
@@ -420,8 +420,8 @@ Promise *DeRefCopyPromise(EvalContext *ctx, const Promise *pp)
     } /* for all constraints */
 
     // Add default body for promise body types that are not present
-    char *bundle_type = pcopy->parent_promise_type->parent_bundle->type;
-    char *promise_type = pcopy->parent_promise_type->name;
+    char *bundle_type = pcopy->parent_section->parent_bundle->type;
+    const char *promise_type = PromiseGetPromiseType(pcopy);
     const PromiseTypeSyntax *syntax = PromiseTypeSyntaxGet(bundle_type, promise_type);
     AddDefaultBodiesToPromise(ctx, pcopy, syntax);
 
@@ -450,7 +450,7 @@ static void AddDefaultBodiesToPromise(EvalContext *ctx, Promise *promise, const
             if(!PromiseBundleOrBodyConstraintExists(ctx, constraint_type, promise)) {
                 const Policy *policy = PolicyFromPromise(promise);
                 // default format is <promise_type>_<body_type>
-                char* default_body_name = StringConcatenate(3, promise->parent_promise_type->name, "_", constraint_type);
+                char* default_body_name = StringConcatenate(3, PromiseGetPromiseType(promise), "_", constraint_type);
                 const Body *bp = EvalContextFindFirstMatchingBody(policy, constraint_type, "bodydefault", default_body_name);
                 if(bp) {
                     Log(LOG_LEVEL_VERBOSE, "Using the default body: %60s", default_body_name);
@@ -566,7 +566,7 @@ static void DereferenceAndPutComment(Promise* pp, const char *comment)
 Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
 {
     assert(pp != NULL);
-    assert(pp->parent_promise_type != NULL);
+    assert(pp->parent_section != NULL);
     assert(pp->promiser != NULL);
     assert(pp->classes != NULL);
     assert(excluded != NULL);
@@ -585,7 +585,9 @@ Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
     Promise *pcopy = xcalloc(1, sizeof(Promise));
     pcopy->promiser = RvalScalarValue(returnval);
 
-    if (!StringEqual("storage", pp->parent_promise_type->name))
+    /* TODO remove the conditions here for fixing redmine#7880. */
+    if ((strcmp("files", PromiseGetPromiseType(pp)) != 0) &&
+        (strcmp("storage", PromiseGetPromiseType(pp)) != 0))
     {
         EvalContextVariablePutSpecial(ctx, SPECIAL_SCOPE_THIS, "promiser", pcopy->promiser,
                                       CF_DATA_TYPE_STRING, "source=promise");
@@ -601,14 +603,14 @@ Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
     }
 
     pcopy->classes = xstrdup(pp->classes);
-    pcopy->parent_promise_type = pp->parent_promise_type;
+    pcopy->parent_section = pp->parent_section;
     pcopy->offset.line = pp->offset.line;
     pcopy->comment = pp->comment ? xstrdup(pp->comment) : NULL;
     pcopy->conlist = SeqNew(10, ConstraintDestroy);
     pcopy->org_pp = pp->org_pp;
 
     // if this is a class promise, check if it is already set, if so, skip
-    if (strcmp("classes", pp->parent_promise_type->name) == 0)
+    if (strcmp("classes", PromiseGetPromiseType(pp)) == 0)
     {
         if (IsDefinedClass(ctx, CanonifyName(pcopy->promiser)))
         {
@@ -683,7 +685,7 @@ Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
                 // We default to NOT skipping (since if would skip).
                 Log(LOG_LEVEL_VERBOSE,
                     "Not skipping %s promise '%s' with constraint '%s => %s' in last evaluation pass (since if would skip)",
-                    pp->parent_promise_type->name,
+                    PromiseGetPromiseType(pp),
                     pp->promiser,
                     unless->lval,
                     unless_string);
@@ -724,17 +726,6 @@ Promise *ExpandDeRefPromise(EvalContext *ctx, const Promise *pp, bool *excluded)
         }
     }
 
-    /* NOTE: We have to undefine the '$(this.promiser)' variable for a 'files'
-     *       promise now because it is later defined for the individual
-     *       expansions of the promise and so it has to be left unexpanded in
-     *       the constraints/attributes. It is, however, defined above just like
-     *       for any other promise so that it can be used in the common
-     *       if/ifvarclass/unless checking. */
-    if (StringEqual(pp->parent_promise_type->name, "files"))
-    {
-        EvalContextVariableRemoveSpecial(ctx, SPECIAL_SCOPE_THIS, "promiser");
-    }
-
     /* Evaluate all constraints. */
     for (size_t i = 0; i < SeqLength(pp->conlist); i++)
     {
