@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -41,6 +41,8 @@
 
 static Rlist *RlistPrependRval(Rlist **start, Rval rval);
 
+static char SECRET_RVAL[] = "************";
+
 RvalType DataTypeToRvalType(DataType datatype)
 {
     switch (datatype)
@@ -466,6 +468,11 @@ Rval RvalNew(const void *item, RvalType type)
     return RvalNewRewriter(item, type, NULL);
 }
 
+Rval RvalNewSecret()
+{
+    return ((Rval) {SECRET_RVAL, RVAL_TYPE_SCALAR});
+}
+
 Rval RvalCopyRewriter(Rval rval, JsonElement *map)
 {
     return RvalNewRewriter(rval.item, rval.type, map);
@@ -600,7 +607,7 @@ Rlist *RlistAppendAllTypes(Rlist **start, const void *item, RvalType type, bool
             return RlistAppendRval(start, (Rval) { JsonCopy((JsonElement*) item), RVAL_TYPE_CONTAINER });
         }
 
-        // note falls through!
+        // fall through
 
     default:
         Log(LOG_LEVEL_DEBUG, "Cannot append %c to rval-list '%s'", type, (char *) item);
@@ -939,7 +946,7 @@ Rlist *RlistParseString(const char *string)
 
 void RvalDestroy(Rval rval)
 {
-    if (rval.item == NULL)
+    if (rval.item == NULL || rval.item == SECRET_RVAL)
     {
         return;
     }
@@ -1043,7 +1050,7 @@ static size_t SubStrnCopyChr(Rlist **to, const char *from, char sep, char lstrip
         }
     }
 
-    size_t consume = (end == NULL) ? strlen(from) : (end - from);
+    size_t consume = (end == NULL) ? strlen(from) : (size_t)(end - from);
     assert(consume >= escapes);
     char copy[1 + consume - escapes], *dst = copy;
 
@@ -1078,7 +1085,7 @@ Rlist *RlistFromSplitString(const char *string, char sep)
     for (const char *sp = string; *sp != '\0';)
     {
         sp += SubStrnCopyChr(&liststart, sp, sep, '\0');
-        assert(sp - string <= strlen(string));
+        assert((size_t) (sp - string) <= strlen(string));
         if (*sp)
         {
             assert(*sp == sep && (sp == string || sp[-1] != '\\'));
@@ -1122,7 +1129,7 @@ Rlist *RlistFromStringSplitLines(const char *string, bool detect_crlf)
     for (const char *sp = string; *sp != '\0';)
     {
         sp += SubStrnCopyChr(&liststart, sp, '\r', '\n');
-        assert(sp - string <= strlen(string));
+        assert((size_t) (sp - string) <= strlen(string));
         if (*sp)
         {
             assert(*sp == '\r' && (sp == string || sp[-1] != '\\'));
