@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -27,6 +27,7 @@
 #include <json.h>
 #include <files_names.h>
 #include <mod_files.h>
+#include <mod_custom.h>
 #include <item_lib.h>
 #include <conversion.h>
 #include <expand.h>
@@ -72,6 +73,26 @@ static const PromiseTypeSyntax *PromiseTypeSyntaxGetStrict(const char *bundle_ty
     return NULL;
 }
 
+bool IsBuiltInPromiseType(const char *const promise_type)
+{
+    // Any built in promise type, regardless of bundle (agent) type
+    assert(promise_type != NULL);
+
+    for (int module_index = 0; module_index < CF3_MODULES; module_index++)
+    {
+        const PromiseTypeSyntax *const module = CF_ALL_PROMISE_TYPES[module_index];
+        for (int promise_type_index = 0; module[promise_type_index].promise_type; promise_type_index++)
+        {
+            const PromiseTypeSyntax *promise_type_syntax = &(module[promise_type_index]);
+            if (StringEqual(promise_type, promise_type_syntax->promise_type))
+            {
+                return true;
+            }
+        }
+    }
+    return false;
+}
+
 const PromiseTypeSyntax *PromiseTypeSyntaxGet(const char *bundle_type, const char *promise_type)
 {
     const PromiseTypeSyntax *pts = PromiseTypeSyntaxGetStrict(bundle_type, promise_type);
@@ -86,13 +107,12 @@ static const ConstraintSyntax *GetCommonConstraint(const char *lval)
 {
     for (int i = 0; CF_COMMON_PROMISE_TYPES[i].promise_type; i++)
     {
-        const PromiseTypeSyntax promise_type_syntax = CF_COMMON_PROMISE_TYPES[i];
-
-        for (int j = 0; promise_type_syntax.constraints[j].lval; j++)
+        const ConstraintSyntax *constraints = CF_COMMON_PROMISE_TYPES[i].constraints;
+        for (int j = 0; constraints[j].lval != NULL; j++)
         {
-            if (strcmp(promise_type_syntax.constraints[j].lval, lval) == 0)
+            if (StringEqual(constraints[j].lval, lval))
             {
-                return &promise_type_syntax.constraints[j];
+                return &(constraints[j]);
             }
         }
     }
@@ -146,8 +166,19 @@ const ConstraintSyntax *PromiseTypeSyntaxGetConstraintSyntax(const PromiseTypeSy
     return GetCommonConstraint(lval);
 }
 
-const BodySyntax *BodySyntaxGet(const char *body_type)
+const BodySyntax *BodySyntaxGet(ARG_UNUSED ParserBlock block, const char *body_type)
 {
+    if (block == PARSER_BLOCK_PROMISE)
+    {
+        // Required: promise agent <id>
+        if (!StringEqual(body_type, "agent"))
+        {
+            return NULL;
+        }
+        return &CUSTOM_PROMISE_BLOCK_SYNTAX;
+    }
+
+    assert(block == PARSER_BLOCK_BODY);
     for (int i = 0; i < CF3_MODULES; i++)
     {
         const PromiseTypeSyntax *promise_type_syntax = CF_ALL_PROMISE_TYPES[i];
