@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -29,6 +29,7 @@
 
 #include <sequence.h>
 #include <json.h>
+#include <parser_helpers.h>
 
 #include <stdio.h>
 
@@ -79,10 +80,11 @@ SyntaxTypeMatch CheckParseContext(const char *context, const char *range);
 DataType StringDataType(EvalContext *ctx, const char *string);
 DataType ExpectedDataType(const char *lvalname);
 
+bool IsBuiltInPromiseType(const char *const promise_type);
 const PromiseTypeSyntax *PromiseTypeSyntaxGet(const char *bundle_type, const char *promise_type);
 const ConstraintSyntax *PromiseTypeSyntaxGetConstraintSyntax(const PromiseTypeSyntax *promise_type_syntax, const char *lval);
 
-const BodySyntax *BodySyntaxGet(const char *body_type);
+const BodySyntax *BodySyntaxGet(ParserBlock block, const char *body_type);
 const ConstraintSyntax *BodySyntaxGetConstraintSyntax(const ConstraintSyntax *body_syntax, const char *lval);
 
 const char *SyntaxStatusToString(SyntaxStatus status);
