@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -46,6 +46,11 @@ void SetSyslogFacility(int facility)
     SYSLOG_FACILITY = facility;
 }
 
+int GetSyslogFacility()
+{
+    return SYSLOG_FACILITY;
+}
+
 bool SetSyslogHost(const char *host)
 {
     if (strlen(host) < sizeof(SYSLOG_HOST))
