@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -29,6 +29,7 @@
 
 #include <string_expressions.h>
 #include <policy.h>
+#include <string_lib.h>         /* StringContains() */
 
 /**
    VarRef is immutable, which means that after allocated the members never
@@ -86,5 +87,24 @@ bool VarRefEqual_untyped(const void *a, const void *b);
 unsigned int VarRefHash_untyped(const void *ref,
                                 unsigned int seed);
 
+static inline bool StringContainsUnresolved(const char *str)
+{
+    // clang-format off
+    return (StringContains(str, "$(")
+            || StringContains(str, "${")
+            || StringContains(str, "@{")
+            || StringContains(str, "@("));
+    // clang-format on
+}
+
+/** Whether #str is of form "@{something}"/"@(something)" or not. */
+static inline bool StringIsBareNonScalarRef(const char *str)
+{
+    assert(str != NULL);
+    const size_t str_len = strlen(str);
+    return ((str[0] == '@') &&
+            (((str[1] == '(') && (str[str_len - 1] == ')')) ||
+             ((str[1] == '{') && (str[str_len - 1] == '}'))));
+}
 
 #endif
