@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -28,14 +28,113 @@
 #include <writer.h>
 #include <conversion.h>                                 /* DataTypeToString */
 
+#define VARIABLE_TAG_SECRET "secret"
 
-static void VariableDestroy(Variable *var);                 /* forward declaration */
+struct Variable_
+{
+    VarRef *ref;
+    Rval rval;
+    DataType type;
+    StringSet *tags;
+    char *comment;
+    const Promise *promise; // The promise that set the present value
+};
+
+static Variable *VariableNew(VarRef *ref, Rval rval, DataType type,
+                             StringSet *tags, char *comment,
+                             const Promise *promise)
+{
+    Variable *var = xmalloc(sizeof(Variable));
+
+    var->ref = ref;
+    var->rval = rval;
+    var->type = type;
+    var->tags = tags;
+    var->comment = comment;
+    var->promise = promise;
+
+    return var;
+}
+
+/* DO NOT EXPORT, this is for internal (hash table) use only, and it doesn't
+ * free everything in Variable, in particular it leaves var->ref to be handled
+ * by the Map implementation calling the key-destroy function. */
+static void VariableDestroy(Variable *var)
+{
+    if (var != NULL)
+    {
+        RvalDestroy(var->rval);
+        StringSetDestroy(var->tags);
+        free(var->comment);
+        // Nothing to do for ->promise
+
+        free(var);
+    }
+}
 
 static void VariableDestroy_untyped(void *var)
 {
     VariableDestroy(var);
 }
 
+const VarRef *VariableGetRef(const Variable *var)
+{
+    assert(var != NULL);
+    return var->ref;
+}
+
+DataType VariableGetType(const Variable *var)
+{
+    assert(var != NULL);
+    return var->type;
+}
+
+RvalType VariableGetRvalType(const Variable *var)
+{
+    assert(var != NULL);
+    return var->rval.type;
+}
+
+StringSet *VariableGetTags(const Variable *var)
+{
+    assert(var != NULL);
+    return var->tags;
+}
+
+const char *VariableGetComment(const Variable *var)
+{
+    assert(var != NULL);
+    return var->comment;
+}
+
+const Promise *VariableGetPromise(const Variable *var)
+{
+    assert(var != NULL);
+    return var->promise;
+}
+
+bool VariableIsSecret(const Variable *var)
+{
+    assert(var != NULL);
+    return ((var->tags != NULL) && StringSetContains(var->tags, VARIABLE_TAG_SECRET));
+}
+
+Rval VariableGetRval(const Variable *var, bool get_secret)
+{
+    assert(var != NULL);
+    if (!get_secret && VariableIsSecret(var))
+    {
+        return RvalNewSecret();
+    }
+    return var->rval;
+}
+
+void VariableSetRval(Variable *var, Rval new_rval)
+{
+    assert(var != NULL);
+    RvalDestroy(var->rval);
+    var->rval = new_rval;
+}
 
 /**
    Define "VarMap" hash table.
@@ -61,21 +160,6 @@ struct VariableTableIterator_
     MapIterator iter;
 };
 
-/* DO NOT EXPORT, this is for internal (hash table) use only, and it doesn't
- * free everything in Variable, in particular it leaves var->ref to be handled
- * by the Map implementation calling the key-destroy function. */
-static void VariableDestroy(Variable *var)
-{
-    if (var)
-    {
-        RvalDestroy(var->rval);
-        StringSetDestroy(var->tags);
-        // Nothing to do for ->promise
-
-        free(var);
-    }
-}
-
 VariableTable *VariableTableNew(void)
 {
     VariableTable *table = xmalloc(sizeof(VariableTable));
@@ -146,30 +230,10 @@ bool VariableTableRemove(VariableTable *table, const VarRef *ref)
     return VarMapRemove(table->vars, ref);
 }
 
-static Variable *VariableNew(VarRef *ref, Rval rval, DataType type,
-                             StringSet *tags, const Promise *promise)
-{
-    Variable *var = xmalloc(sizeof(Variable));
-
-    var->ref = ref;
-    var->rval = rval;
-    var->type = type;
-    if (tags == NULL)
-    {
-        var->tags = StringSetFromString("", ',');
-    }
-    else
-    {
-        var->tags = tags;
-    }
-    var->promise = promise;
-
-    return var;
-}
-
 bool VariableTablePut(VariableTable *table, const VarRef *ref,
                       const Rval *rval, DataType type,
-                      const char *tags, const Promise *promise)
+                      StringSet *tags, char *comment,
+                      const Promise *promise)
 {
     assert(VarRefIsQualified(ref));
 
@@ -189,7 +253,7 @@ bool VariableTablePut(VariableTable *table, const VarRef *ref,
               "Only iterables (Rlists) are allowed to be NULL");
 
     Variable *var = VariableNew(VarRefCopy(ref), RvalCopy(*rval), type,
-                                StringSetFromString(tags, ','), promise);
+                                tags, comment, promise);
     return VarMapInsert(table->vars, var->ref, var);
 }
 
