@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -28,24 +28,45 @@
 
 #include <var_expressions.h>
 
-typedef struct
-{
-    VarRef *ref;
-    Rval rval;
-    DataType type;
-    StringSet *tags;
-    const Promise *promise; // The promise that set the present value
-} Variable;
-
+typedef struct Variable_ Variable;
 typedef struct VariableTable_ VariableTable;
 typedef struct VariableTableIterator_ VariableTableIterator;
 
+const VarRef *VariableGetRef(const Variable *var);
+DataType VariableGetType(const Variable *var);
+RvalType VariableGetRvalType(const Variable *var);
+StringSet *VariableGetTags(const Variable *var);
+const char *VariableGetComment(const Variable *var);
+const Promise *VariableGetPromise(const Variable *var);
+
+/**
+ * Get the variable's value.
+ *
+ * @param get_secret Whether to get the secret value in case the variable is tagged as secret
+ *
+ * @note #get_secret should only be %true if the potentially secret value is
+ *       really required, i.e. being actually used not logged, reported,...
+ */
+Rval VariableGetRval(const Variable *var, bool get_secret);
+
+/**
+ * Set new value of variable.
+ *
+ * @note Destroys the old variable's value.
+ */
+void VariableSetRval(Variable *var, Rval new_rval);
+
+/**
+ * Whether the variable is marked as secret or not.
+ */
+bool VariableIsSecret(const Variable *var);
+
 VariableTable *VariableTableNew(void);
 void VariableTableDestroy(VariableTable *table);
 
 bool VariableTablePut(VariableTable *table, const VarRef *ref,
-                      const Rval *rval, DataType type, const
-                      char *tags, const Promise *promise);
+                      const Rval *rval, DataType type,
+                      StringSet *tags, char *comment, const Promise *promise);
 Variable *VariableTableGet(const VariableTable *table, const VarRef *ref);
 bool VariableTableRemove(VariableTable *table, const VarRef *ref);
 
