@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -163,13 +163,9 @@ bool IsCf3VarString(const char *str)
         /* Some chars cannot be in variable ids, e.g.
            $(/bin/cat file) is legal in bash */
 
-        if (bracks > 0)
+        if ((bracks > 0) && (*sp == '/'))
         {
-            switch (*sp)
-            {
-            case '/':
-                return false;
-            }
+            return false;
         }
 
         if (left == '(' && right == ')' && dollar && (bracks == 0))
@@ -242,13 +238,9 @@ static bool IsCf3Scalar(char *str)
         /* Some chars cannot be in variable ids, e.g.
            $(/bin/cat file) is legal in bash */
 
-        if (bracks > 0)
+        if ((bracks > 0) && (*sp == '/'))
         {
-            switch (*sp)
-            {
-            case '/':
-                return false;
-            }
+            return false;
         }
 
         if (left == '(' && right == ')' && dollar && (bracks == 0))
@@ -384,7 +376,7 @@ bool ExtractScalarReference(Buffer *out, const char *str, size_t len, bool extra
     }
 
     const char *dollar_point = memchr(str, '$', len);
-    if (!dollar_point || (dollar_point - str) == len)
+    if (!dollar_point || (size_t) (dollar_point - str) == len)
     {
         return false;
     }
