@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -43,7 +43,7 @@
 #include <eval_context.h>
 #include <actuator.h>
 
-static bool PrintFile(const char *filename, size_t max_lines);
+static bool PrintFile(const char *filename, ssize_t max_lines);
 static void ReportToFile(const char *logfile, const char *message);
 static void ReportToLog(const char *message);
 
@@ -102,7 +102,7 @@ PromiseResult VerifyReportPromise(EvalContext *ctx, const Promise *pp)
 
     PromiseBanner(ctx, pp);
 
-    if (a.transaction.action == cfa_warn)
+    if (DONTDO || (a.transaction.action == cfa_warn))
     {
         cfPS(ctx, LOG_LEVEL_WARNING, PROMISE_RESULT_WARN, pp, &a, "Need to repair reports promise: %s", pp->promiser);
         YieldCurrentLock(thislock);
@@ -159,31 +159,57 @@ static void ReportToFile(const char *logfile, const char *message)
     }
 }
 
-static bool PrintFile(const char *filename, size_t max_lines)
+static bool PrintFile(const char *filename, ssize_t max_lines)
 {
     if (!filename)
     {
-        Log(LOG_LEVEL_VERBOSE, "Printfile promise was incomplete, with no filename.");
+        Log(LOG_LEVEL_VERBOSE,
+            "Printfile promise was incomplete, with no filename.");
         return false;
     }
 
     FILE *fp = safe_fopen(filename, "r");
     if (!fp)
     {
-        Log(LOG_LEVEL_ERR, "Printing of file '%s' was not possible. (fopen: %s)", filename, GetErrorStr());
+        Log(LOG_LEVEL_ERR,
+            "Printing of file '%s' was not possible. (fopen: %s)",
+            filename, GetErrorStr());
         return false;
     }
 
     size_t line_size = CF_BUFSIZE;
     char *line = xmalloc(line_size);
 
-    for (size_t i = 0; i < max_lines; i++)
+    ssize_t skip_lines = 0;
+    if (max_lines < 0)
+    {
+        skip_lines = max_lines;
+        max_lines = ABS(max_lines);
+
+        while (CfReadLine(&line, &line_size, fp) != -1)
+        {
+            skip_lines++;
+        }
+        if (ferror(fp))
+        {
+            Log(LOG_LEVEL_ERR,
+                "Failed to read line from stream, (getline: %s)",
+                GetErrorStr());
+            free(line);
+            return false;
+        }
+        rewind(fp);
+    }
+
+    for (ssize_t i = 0; i < skip_lines + max_lines; i++)
     {
         if (CfReadLine(&line, &line_size, fp) == -1)
         {
             if (ferror(fp))
             {
-                Log(LOG_LEVEL_ERR, "Failed to read line from stream, (getline: %s)", GetErrorStr());
+                Log(LOG_LEVEL_ERR,
+                    "Failed to read line from stream, (getline: %s)",
+                    GetErrorStr());
                 free(line);
                 return false;
             }
@@ -192,8 +218,10 @@ static bool PrintFile(const char *filename, size_t max_lines)
                 break;
             }
         }
-
-        ReportToLog(line);
+        if (i >= skip_lines)
+        {
+            ReportToLog(line);
+        }
     }
 
     fclose(fp);
