@@ -1,5 +1,5 @@
 /*
-  Copyright 2019 Northern.tech AS
+  Copyright 2021 Northern.tech AS
 
   This file is part of CFEngine 3 - written and maintained by Northern.tech AS.
 
@@ -109,6 +109,8 @@ static bool IsLegalVariableName(EvalContext *ctx, const Promise *pp)
 PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp,
                                ARG_UNUSED void *param)
 {
+    assert(pp != NULL);
+
     ConvergeVariableOptions opts = CollectConvergeVariableOptions(ctx, pp);
 
     Log(LOG_LEVEL_DEBUG, "Evaluating vars promise: %s", pp->promiser);
@@ -132,18 +134,21 @@ PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp,
     /* Warn if promise locking was used with a promise that doesn't support it
      * (which applies to all of 'vars', 'meta' and 'defaults' promises handled
      * by this code).
+     * 'ifelapsed => "0"' (e.g. with 'action => immediate') can however be used
+     * to make sure cached functions are called every time. [ENT-7478]
      * (Only do this in the first pass in cf-promises, we don't have to repeat
      * the warning over and over.) */
     if (EvalContextGetPass(ctx) == 0 && THIS_AGENT_TYPE == AGENT_TYPE_COMMON)
     {
         int ifelapsed = PromiseGetConstraintAsInt(ctx, "ifelapsed", pp);
-        if (ifelapsed != CF_NOINT)
+        if ((ifelapsed != CF_NOINT) &&
+            ((ifelapsed != 0) || !StringEqual(PromiseGetPromiseType(pp), "vars")))
         {
             Log(LOG_LEVEL_WARNING,
                 "ifelapsed attribute specified in action body for %s promise '%s',"
                 " but %s promises do not support promise locking",
-                pp->parent_promise_type->name, pp->promiser,
-                pp->parent_promise_type->name);
+                PromiseGetPromiseType(pp), pp->promiser,
+                PromiseGetPromiseType(pp));
         }
         int expireafter = PromiseGetConstraintAsInt(ctx, "expireafter", pp);
         if (expireafter != CF_NOINT)
@@ -151,15 +156,15 @@ PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp,
             Log(LOG_LEVEL_WARNING,
                 "expireafter attribute specified in action body for %s promise '%s',"
                 " but %s promises do not support promise locking",
-                pp->parent_promise_type->name, pp->promiser,
-                pp->parent_promise_type->name);
+                PromiseGetPromiseType(pp), pp->promiser,
+                PromiseGetPromiseType(pp));
         }
     }
 
     a.classes = GetClassDefinitionConstraints(ctx, pp);
 
     VarRef *ref = VarRefParseFromBundle(pp->promiser, PromiseGetBundle(pp));
-    if (strcmp("meta", pp->parent_promise_type->name) == 0)
+    if (strcmp("meta", PromiseGetPromiseType(pp)) == 0)
     {
         VarRefSetMeta(ref, true);
     }
@@ -395,12 +400,8 @@ PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp,
 
             for (const Rlist *rp = RvalRlistValue(rval); rp; rp = rp->next)
             {
-                switch (rp->val.type)
+                if (rp->val.type != RVAL_TYPE_SCALAR)
                 {
-                case RVAL_TYPE_SCALAR:
-                    break;
-
-                default:
                     // Cannot assign variable because value is a list containing a non-scalar item
                     VarRefDestroy(ref);
                     RvalDestroy(rval);
@@ -459,8 +460,37 @@ PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp,
             return PROMISE_RESULT_FAIL;
         }
 
+        StringSet *tags = StringSetNew();
+        StringSetAdd(tags, xstrdup("source=promise"));
+
+        Rlist *promise_meta = PromiseGetConstraintAsList(ctx, "meta", pp);
+        if (promise_meta)
+        {
+            Buffer *print;
+            for (const Rlist *rp = promise_meta; rp; rp = rp->next)
+            {
+                StringSetAdd(tags, xstrdup(RlistScalarValue(rp)));
+                if (WouldLog(LOG_LEVEL_DEBUG))
+                {
+                    print = StringSetToBuffer(tags, ',');
+                    Log(LOG_LEVEL_DEBUG,
+                        "Added tag %s to variable %s tags (now [%s])",
+                        RlistScalarValue(rp), pp->promiser, BufferData(print));
+                    BufferDestroy(print);
+                }
+            }
+        }
+
+        const char *comment = PromiseGetConstraintAsRval(pp, "comment", RVAL_TYPE_SCALAR);
+
         /* WRITE THE VARIABLE AT LAST. */
-        bool success = EvalContextVariablePut(ctx, ref, rval.item, required_datatype, "source=promise");
+        bool success = EvalContextVariablePutTagsSetWithComment(ctx, ref, rval.item, required_datatype,
+                                                                tags, comment);
+        if (success && (comment != NULL))
+        {
+            Log(LOG_LEVEL_VERBOSE, "Added variable '%s' with comment '%s'",
+                pp->promiser, comment);
+        }
 
         if (!success)
         {
@@ -471,25 +501,10 @@ PromiseResult VerifyVarPromise(EvalContext *ctx, const Promise *pp,
 
             VarRefDestroy(ref);
             RvalDestroy(rval);
+            StringSetDestroy(tags);
             return PROMISE_RESULT_FAIL;
         }
 
-        Rlist *promise_meta = PromiseGetConstraintAsList(ctx, "meta", pp);
-        if (promise_meta)
-        {
-            StringSet *class_meta = EvalContextVariableTags(ctx, ref);
-            Buffer *print;
-            for (const Rlist *rp = promise_meta; rp; rp = rp->next)
-            {
-                StringSetAdd(class_meta, xstrdup(RlistScalarValue(rp)));
-                print = StringSetToBuffer(class_meta, ',');
-                Log(LOG_LEVEL_DEBUG,
-                    "Added tag %s to class %s, tags now [%s]",
-                    RlistScalarValue(rp), pp->promiser, BufferData(print));
-                BufferDestroy(print);
-            }
-        }
-
         result = PROMISE_RESULT_NOOP;
     }
     else
@@ -521,7 +536,7 @@ static bool CompareRval(const void *rval1_item, RvalType rval1_type,
 {
     if (rval1_type != rval2_type)
     {
-        return -1;
+        return false;
     }
 
     switch (rval1_type)
@@ -530,7 +545,7 @@ static bool CompareRval(const void *rval1_item, RvalType rval1_type,
 
         if (IsCf3VarString(rval1_item) || IsCf3VarString(rval2_item))
         {
-            return -1;          // inconclusive
+            return false;          // inconclusive
         }
 
         if (strcmp(rval1_item, rval2_item) != 0)
@@ -544,10 +559,10 @@ static bool CompareRval(const void *rval1_item, RvalType rval1_type,
         return RlistEqual(rval1_item, rval2_item);
 
     case RVAL_TYPE_FNCALL:
-        return -1;
+        return false;
 
     default:
-        return -1;
+        return false;
     }
 
     return true;
